/* ------------------------------------------------------------
// Copyright 2000 ST Microelectronics
// ------------------------------------------------------------
// This software is the property of ST Microelectronics which
// specifically grants the user the right to modify, use and
// distribute this software provided this notice is not removed
// or altered. All other rights are reserved by ST.
// ------------------------------------------------------------
// ST MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, WITH
// REGARD TO THIS SOFTWARE. IN NO EVENT SHALL ST BE LIABLE FOR
// INCIDENTAL OR CONSEQUENTIAL DAMAGES IN CONNECTION WITH OR
// ARISING FROM THE FURNISHING, PERFORMANCE, OR USE OF THIS
// SOFTWARE.
// ------------------------------------------------------------
// So that all may benefit from your experience, please report
// any problems or suggestions about this software back to ST.
// Please include your name, company, telephone number, email,
// ST product requiring support and question or problem
// encountered.
// ------------------------------------------------------------
// ST Microelectronics
// Technoparc du pays de gex
// 165 rue Edouard Branly bp112
// 06637 St Genis Pouilly cedex
// France
// ------------------------------------------------------------
// To contact us : http://www.st.com/stpc
// ------------------------------------------------------------*/

/* ------------------------------------------------------------
// Copyright (C) 2000 STMicroelectronics
// ------------------------------------------------------------
// NVG/SICL/System Solution
// ------------------------------------------------------------
// FILE NAME	: crtc.c
// LIBRARY		: STPC Graphics Development Kit
// MODULE		: libhard\crtc
// PORTABILITY	: ANSI C
// ------------------------------------------------------------
// REVISION		: Rev 1.4
// DATE			: August, 25th 2000
// AUTHOR		: Thierry Vuillaume
// ------------------------------------------------------------
// DESCRIPTION	: Functions to initialize the graphic output,
//				  monitor, TV or TFT and manage the graphic
//				  surfaces.
// ------------------------------------------------------------
// VARIABLES	: HWCRTC_ulOffsetX
//				  HWCRTC_ulOffsetY
//				  HWCRTC_pucPixelDepthTable
//				  HWCRTC_ulCurrentOffset
// ------------------------------------------------------------
// MACROS		: -
// ------------------------------------------------------------
// FUNCTIONS	: HWCRTC_InitializeInternalVGA
//				  HWCRTC_Set256ColorsDefaultPalette
//				  HWCRTC_SetLinearGammaCorrection
//				  HWCRTC_SetRefreshRate
//				  HWCRTC_SetPixelDepth
//				  HWCRTC_ScanGraphicsMode
//				  HWCRTC_SetCRTCTimings
//				  HWCRTC_SetInternalDCLK
//				  HWCRTC_SetExternalDCLK
//				  HWCRTC_InitializePixelClock
//				  HWCRTC_SetWaterMark
//				  HWCRTC_GetNumberOfModes
//				  HWCRTC_GetFirstMode
//				  HWCRTC_GetNextMode
//				  HWCRTC_GetModeInfo
//				  HWCRTC_InitGraphicMode
//				  HWCRTC_GetBase
//				  HWCRTC_GetPitch
//				  HWCRTC_GetPixelDepth
//				  HWCRTC_GetScreenWidth
//				  HWCRTC_GetScreenHeight
//				  HWCRTC_GetOffsetX
//				  HWCRTC_GetOffsetY
//				  HWCRTC_FlipSurface
//				  HWCRTC_FlipBase
//				  HWCRTC_ScrollSurface
//				  HWCRTC_ChangePaletteType
//				  HWCRTC_SetPaletteEntry3Bytes
//				  HWCRTC_SetPaletteEntry4Bytes
//				  HWCRTC_WaitForVerticalRetrace
//				  HWCRTC_WaitForVerticalBlank
//				  HWCRTC_StopCRTC
//				  HWCRTC_StartCRTC
//				  HWCRTC_InitAdditionnalController
//				  HWCRTC_SetTextModeDefaultPalette
//				  HWCRTC_Load8xNTextFont
//				  HWCRTC_SetTextMode
//				  HWCRTC_GetCurrentDotClock
//				  HWCRTC_SelectCurrentDotClock
// ------------------------------------------------------------*/

/* ------------------------------------------------------------
// REVISION		: Rev 1.4
// DATE			: August, 25th 2000
// AUTHOR		: Thierry Vuillaume
// MODIFICATIONS: Modify the HWCRTC_SetDotClock to integrate
//				  the HCMOS7 PLLs (STPC Consumer II and STPC Atlas)
// ------------------------------------------------------------
// REVISION		: Rev 1.3
// DATE			: August, 14th 2000
// AUTHOR		: Thierry Vuillaume
// MODIFICATIONS: Modify the HWCRTC_SetWaterMark to interpolate
//				  the water mark values when the resolution is
//				  not in the table
//				  Added HWCRTC_GetFirstMode, HWCRTC_GetNextMode
//				  and HWCRTC_GetNumberOfMode functions
// ------------------------------------------------------------
// REVISION		: Rev 1.2
// DATE			: July, 18th 2000
// AUTHOR		: Thierry Vuillaume
// MODIFICATIONS: Add new functions to get screen parameters,
//				  HWCRTC_GetScreenWidth, HWCRTC_GetScreenHeight
//				  Contend of HWCRTC_InitGraphics mode have been 
//				  re-organize and split into different functions.
//				  Add support to shifted screen like in NTSC
//				  (16 lines missing in the top of screen)
//				  Add support of external 27MHz clock for TV.
// ------------------------------------------------------------
// REVISION		: Rev 1.1
// DATE			: March, 27th 2000
// AUTHOR		: Thierry Vuillaume
// MODIFICATIONS: Add support of 80x25 text mode and modify the
//				  HWCRTC_InitGraphicMode function to extract
//				  common part with text mode.
//				  New functions : 
//				  HWCRTC_InitAdditionnalController,
//				  HWCRTC_SetTextModeDefaultPalette,
//				  HWCRTC_Load8xNTextFont,
//				  HWCRTC_SetTextMode
// ------------------------------------------------------------
// REVISION		: Rev 1.00
// DATE			: March, 1st 2000
// AUTHOR		: Thierry Vuillaume
// MODIFICATIONS: Initial source code	
// ------------------------------------------------------------*/

/* include file */
#include "trace.h"
#include "host.h"
#include "generic.h"
#include "crtc.h"
#include "tft.h"
#include "stv.h"

/* ------------------------------------------------------------
// VARIABLE		: HWCRTC_ulBaseXOffset
// ------------------------------------------------------------
// DESCRIPTION	: Horizontal offset between the CRTC base address
//				  and the first displayed pixels
// ------------------------------------------------------------*/
unsigned long HWCRTC_ulOffsetX;

/* ------------------------------------------------------------
// VARIABLE		: HWCRTC_ulOffsetY
// ------------------------------------------------------------
// DESCRIPTION	: Vertical offset between the CRTC base address
//				  and the first displayed pixels
// ------------------------------------------------------------*/
unsigned long HWCRTC_ulOffsetY;

/* ------------------------------------------------------------
// VARIABLE		: HWCRTC_pucPixelDepthTable
// ------------------------------------------------------------
// DESCRIPTION	: Table of available pixel depth
// ------------------------------------------------------------*/
unsigned char HWCRTC_pucPixelDepthTable[] =
{
	0xff, 8, 15, 16, 24, 32, 0xff, 0xff
};

/* ------------------------------------------------------------
// VARIABLE		: HWCRTC_ulCurrentOffset
// ------------------------------------------------------------
// DESCRIPTION	: Current offset in the graphics mode table
//				  used by HWCRTC_GetFirstMode and HWCRTC_GetNextMode
// ------------------------------------------------------------*/
unsigned long HWCRTC_ulCurrentOffset;

/* ------------------------------------------------------------
// FUNCTIONS	: HWCRTC_InitializeInternalVGA
// ------------------------------------------------------------
// DESCRIPTION	: Enable the internal VGA, initialize the
//				  different controller : CRTC, Sequencer,
//				  graphic and attribut controller
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWCRTC_InitializeInternalVGA(void)
{
	long i;

	DBGTRACE_BEGINFUNCTION("HWCRTC_InitializeInternalVGA");

	/* Enable access to VGA controller I/O */
	REGHOST_SET_MOTHERBOARD_REG(0x00);		
	REGHOST_SET_VIDEOSUBSYSTEM1_REG(0x01);
	REGHOST_SET_MOTHERBOARD_REG(0x28);
	REGHOST_SET_VIDEOSUBSYSTEM2_REG(0x01);

	/* Set CRTC registers access to 3d4 */
	REGHOST_SET_MISCELLANEOUS_REG(0x03);

	/* Enable access to extended CRTC registers */
	REGHOST_SET_SEQUENCER_REG(0x06, 0x57);

	/* Initialize sequencer */
	REGHOST_SET_SEQUENCER_REG(0x0, 0x00);
	REGHOST_SET_MISCELLANEOUS_REG(0x2b);
	REGHOST_SET_SEQUENCER_REG(0x0, 0x00);

	/* Don't enable CRTC if disabled */
	if(REGHOST_GET_SEQUENCER_REG(0x1) & 0x20)
	{
		REGHOST_SET_SEQUENCER_REG(0x1, 0x21);
	}
	else
	{
		REGHOST_SET_SEQUENCER_REG(0x1, 0x01);
	}

	REGHOST_SET_SEQUENCER_REG(0x2, 0x0f);
	REGHOST_SET_SEQUENCER_REG(0x3, 0x00);
	REGHOST_SET_SEQUENCER_REG(0x4, 0x0e);
	REGHOST_SET_SEQUENCER_REG(0x6, 0x57);
	REGHOST_SET_SEQUENCER_REG(0x0, 0x3);

	/* Initialize graphic controller */
	for(i = 0; i < 5; i ++)
	{
		REGHOST_SET_GRAPHICCONTROLLER_REG((unsigned char)i, 0);
	}

	REGHOST_SET_GRAPHICCONTROLLER_REG(5, 0x40);
	REGHOST_SET_GRAPHICCONTROLLER_REG(6, 0x05);
	REGHOST_SET_GRAPHICCONTROLLER_REG(7, 0x0f);
	REGHOST_SET_GRAPHICCONTROLLER_REG(8, 0xff);

	/* Initialize attribut register flip-flop */
	REGHOST_GET_INPUTSTATUS_REG();

	for(i=0; i<0x10; i++ )
	{
		REGHOST_SET_ATTRIBUT_REG(i);
		REGHOST_SET_ATTRIBUT_REG(i);
	}

	REGHOST_SET_ATTRIBUT_REG(0x10);
	REGHOST_SET_ATTRIBUT_REG(0x01);

	REGHOST_SET_ATTRIBUT_REG(0x11);
	REGHOST_SET_ATTRIBUT_REG(0x00);

	REGHOST_SET_ATTRIBUT_REG(0x12);
	REGHOST_SET_ATTRIBUT_REG(0x0f);

	REGHOST_SET_ATTRIBUT_REG(0x13);
	REGHOST_SET_ATTRIBUT_REG(0x00);

	REGHOST_SET_ATTRIBUT_REG(0x14);
	REGHOST_SET_ATTRIBUT_REG(0x00);

	/* Disable palette access & Enable display */
	REGHOST_SET_ATTRIBUT_REG(0x20);

	/* Enable common part of CRTC register */
	REGHOST_SET_CRTC_REG(0x11, 0x0c);
	REGHOST_SET_CRTC_REG(0x08, 0x00);
	REGHOST_SET_CRTC_REG(0x14, 0x00);
	REGHOST_SET_CRTC_REG(0x17, 0xE3);
	REGHOST_SET_CRTC_REG(0x18, 0xFF);
	REGHOST_SET_CRTC_REG(0x1A, 0x3C);
	REGHOST_SET_CRTC_REG(0x1C, 0x04);
	REGHOST_SET_CRTC_REG(0x21, 0xFF);
	REGHOST_SET_CRTC_REG(0x28, 0x10);
	REGHOST_SET_CRTC_REG(0x33, 0xFF);
	REGHOST_SET_CRTC_REG(0x39, 0xFF);
	REGHOST_SET_CRTC_REG(0x3C, 0x60);
	REGHOST_SET_CRTC_REG(0x40, 0xE1);

	DBGTRACE_ENDFUNCTION("HWCRTC_InitializeInternalVGA");
}

/* ------------------------------------------------------------
// FUNCTIONS	: HWCRTC_InitializeInternalVGAText
// ------------------------------------------------------------
// DESCRIPTION	: Enable the internal VGA for text mode, initialize 
//				  the different controller : CRTC, Sequencer,
//				  graphic and attribut controller
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWCRTC_InitializeInternalVGAText(void)
{
	long i;

	DBGTRACE_BEGINFUNCTION("HWCRTC_InitializeInternalVGAText");

	/* Enable access to VGA controller I/O */
	REGHOST_SET_MOTHERBOARD_REG(0x00);		
	REGHOST_SET_VIDEOSUBSYSTEM1_REG(0x01);
	REGHOST_SET_MOTHERBOARD_REG(0x28);
	REGHOST_SET_VIDEOSUBSYSTEM2_REG(0x01);
 
	/* Set CRTC registers access to 3d4 */
	REGHOST_SET_MISCELLANEOUS_REG(0x03);

	/* Enable access to extended CRTC registers */
	REGHOST_SET_SEQUENCER_REG(0x06, 0x57);

	/* Initialize sequencer */
	REGHOST_SET_SEQUENCER_REG(0x0, 0x00);
	REGHOST_SET_MISCELLANEOUS_REG(0x2b);
	REGHOST_SET_SEQUENCER_REG(0x0, 0x00);

	/* Don't enable CRTC if disabled */
	if(REGHOST_GET_SEQUENCER_REG(0x1) & 0x20)
	{
		REGHOST_SET_SEQUENCER_REG(0x1, 0x20);
	}
	else
	{
		REGHOST_SET_SEQUENCER_REG(0x1, 0x00);
	}

	/* Set sequencer register */
	REGHOST_SET_SEQUENCER_REG(0x2, 0x03);
	REGHOST_SET_SEQUENCER_REG(0x3, 0x00);
	REGHOST_SET_SEQUENCER_REG(0x4, 0x02);

	/* Initialize graphic controller */
	REGHOST_SET_SEQUENCER_REG(0x06, 0x57);
	REGHOST_SET_SEQUENCER_REG(0x0, 0x3);

	for(i = 0; i < 5; i ++)
	{
		REGHOST_SET_GRAPHICCONTROLLER_REG((unsigned char)i, 0);
	}

	REGHOST_SET_GRAPHICCONTROLLER_REG(5, 0x10);
	REGHOST_SET_GRAPHICCONTROLLER_REG(6, 0x0e);
	REGHOST_SET_GRAPHICCONTROLLER_REG(7, 0x00);
	REGHOST_SET_GRAPHICCONTROLLER_REG(8, 0xff);

	/* Initialize attribut register */
	REGHOST_GET_INPUTSTATUS_REG();

	for(i = 0; i <= 0x14; i++)
	{
		REGHOST_SET_ATTRIBUT_REG((unsigned char)i);
		REGHOST_SET_ATTRIBUT_REG(HWCRTC_pucTextModeAttributTable[i]);
	}

	/* Disable palette access & Enable display */
	REGHOST_SET_ATTRIBUT_REG(0x20);

	/* Enable common part of CRTC register */
	REGHOST_SET_CRTC_REG(0x11, 0x0c);
	REGHOST_SET_CRTC_REG(0x08, 0x00);
	REGHOST_SET_CRTC_REG(0x14, 0x1f);
	REGHOST_SET_CRTC_REG(0x17, 0xa3);
	REGHOST_SET_CRTC_REG(0x18, 0xFF);
	REGHOST_SET_CRTC_REG(0x1A, 0x3E);
	REGHOST_SET_CRTC_REG(0x1C, 0x00);
	REGHOST_SET_CRTC_REG(0x21, 0xFF);
	REGHOST_SET_CRTC_REG(0x33, 0xFF);
	REGHOST_SET_CRTC_REG(0x39, 0xFF);
	REGHOST_SET_CRTC_REG(0x3C, 0x60);
	REGHOST_SET_CRTC_REG(0x40, 0xE1);

	DBGTRACE_ENDFUNCTION("HWCRTC_InitializeInternalVGAText");
}

/* ------------------------------------------------------------
// FUNCTIONS	: HWCRTC_Load8xNTextFont
// ------------------------------------------------------------
// DESCRIPTION	: Load text font into frame buffer
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// pucFontData	: Font data, 256 blocks (one by ASCII code)
//				  of ulFontHeight bytes (one by character lines)
// ulFontHeight	: Number of lines for a character
// ulFontBlock	: Number of the font block to load (0 to 4) as
//				  VGA standard support 4 blocks of fonts
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWCRTC_Load8xNTextFont(unsigned char *pucFontData, unsigned long ulFontHeight, unsigned long ulFontBlock)
{
	unsigned long i, j;
	unsigned long ulOffset;

	DBGTRACE_BEGINFUNCTION("HWCRTC_Load8xNTextFont");
	DBGTRACE_POINTERPARAM(pucFontData);
	DBGTRACE_NUMERICPARAM(ulFontHeight);
	DBGTRACE_NUMERICPARAM(ulFontBlock);

	/* Offset in frame buffer of the font data */
	ulOffset = (ulFontBlock << 15) + 2;

	/* Copy the 256 ASCII characters into the frame buffer */
	for(i = 0; i < 256; i++)
	{
		/* Copy the ulFontHeight lines of a character */
		for(j = 0; j < ulFontHeight; j++)
		{
			/* Copy a character line (8 bits = 8 pixels) */
			REGHOST_SET_OFFSCREEN8(ulOffset, *pucFontData);

			/* Go to next character line */
			ulOffset += 4;
			pucFontData ++;
		}

		/* Go to next character */
		ulOffset += 4 * (32-ulFontHeight);
	}

	DBGTRACE_ENDFUNCTION("HWCRTC_Load8xNTextFont");
}

/* ------------------------------------------------------------
// FUNCTIONS	: HWCRTC_Set256ColorsDefaultPalette
// ------------------------------------------------------------
// DESCRIPTION	: Set the default color palette in 256 color 
//				  mode. This functions take default values from
//				  the paldata.c file
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWCRTC_Set256ColorsDefaultPalette(void)
{
	int i;

	DBGTRACE_BEGINFUNCTION("HWCRTC_Set256ColorsDefaultPalette");

	/* Enable palette access and disable display */
	REGHOST_SET_ATTRIBUT_REG(0x0);

	/* Set palette index to 0 */
	REGHOST_SET_RAMDACINDEX_REG(0x0);

	/* Copy default palette with conversion from 18 bits to 24 bits */
	for(i=0; i<256*3; i++)
	{
		REGHOST_SET_RAMDACDATA_REG(HWCRTC_pucDefaultPalette[i] << 2);
	}

	/* Set look up table mask : no bits masked */
	REGHOST_SET_PALETTEMASK_REG(0xff);

	/* Disable palette access & Enable display */
	REGHOST_SET_ATTRIBUT_REG(0x20);

	DBGTRACE_ENDFUNCTION("HWCRTC_Set256ColorsDefaultPalette");
}

/* ------------------------------------------------------------
// FUNCTIONS	: HWCRTC_SetTextModeDefaultPalette
// ------------------------------------------------------------
// DESCRIPTION	: Set the default color palette for text mode.
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWCRTC_SetTextModeDefaultPalette(void)
{
	int i;

	DBGTRACE_BEGINFUNCTION("HWCRTC_SetTextModeDefaultPalette");

	/* Enable palette access and disable display */ 
	REGHOST_SET_ATTRIBUT_REG(0x0);

	/* Set palette index to 0 */
	REGHOST_SET_RAMDACINDEX_REG(0x0);

	/* Copy default palette with conversion from 18 bits to 24 bits */
	for(i=0; i<256*3; i++)
	{
		REGHOST_SET_RAMDACDATA_REG(HWCRTC_pucTextModeDefaultPalette[i] << 2);
	}

	/* Set look up table mask : no bits masked */
	REGHOST_SET_PALETTEMASK_REG(0xff);

	/* Disable palette access & Enable display */
	REGHOST_SET_ATTRIBUT_REG(0x20);

	DBGTRACE_ENDFUNCTION("HWCRTC_SetTextModeDefaultPalette");
}

/* ------------------------------------------------------------
// FUNCTIONS	: HWCRTC_SetLinearGammaCorrection
// ------------------------------------------------------------
// DESCRIPTION	: Set a linear gamma correction for 64K and 16M 
//				  colors graphic mode (palette is used as gamma 
//				  correction in this graphics modes)
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWCRTC_SetLinearGammaCorrection(void)
{
	int i;

	DBGTRACE_BEGINFUNCTION("HWCRTC_SetLinearPalette");

	/* Enable palette access and disable display */
	REGHOST_SET_ATTRIBUT_REG(0x0);

	/* Set palette index to 0 */
	REGHOST_SET_RAMDACINDEX_REG(0x0);

	/* Set a linear gamma correction */
	for(i=0; i<256; i++)
	{
		REGHOST_SET_RAMDACDATA_REG(i);
		REGHOST_SET_RAMDACDATA_REG(i);
		REGHOST_SET_RAMDACDATA_REG(i);
	}

	/* Set look up table mask : no bits masked */
	REGHOST_SET_PALETTEMASK_REG(0xff);

	/* Disable palette access & Enable display */
	REGHOST_SET_ATTRIBUT_REG(0x20);

	/* Suppress palette bypass */
	REGHOST_SET_CRTC_REG(0x28, (unsigned char)(REGHOST_GET_CRTC_REG(0x28) & ~0x8));

	DBGTRACE_ENDFUNCTION("HWCRTC_SetLinearPalette");
}

/* ------------------------------------------------------------
// FUNCTIONS	: HWCRTC_GetCurrentDotClock
// ------------------------------------------------------------
// DESCRIPTION	: Get number of the current used DCLK
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: Current DCLK number
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
unsigned long HWCRTC_GetCurrentDotClock(void)
{
	unsigned long ulDCLK;

	DBGTRACE_BEGINFUNCTION("HWCRTC_GetCurrentDotClock");

	ulDCLK = (REGHOST_GET_MISCELLANEOUS_REG() >> 2) & 0x3;

	DBGTRACE_ENDFUNCTION("HWCRTC_GetCurrentDotClock");
	DBGTRACE_NUMERICPARAM(ulDCLK);

	return ulDCLK;
}

/* ------------------------------------------------------------
// FUNCTIONS	: HWCRTC_SelectCurrentDotClock
// ------------------------------------------------------------
// DESCRIPTION	: Select number of the current used DCLK
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulClock		: DCLK number between 0 and 3
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWCRTC_SelectCurrentDotClock(unsigned long ulClock)
{
	DBGTRACE_BEGINFUNCTION("HWCRTC_SelectCurrentDotClock");
	DBGTRACE_NUMERICPARAM(ulClock);

	if(ulClock > 3)
	{
		DBGTRACE_WARNING("Invalid clock number");
		return;
	}

	REGHOST_SET_MISCELLANEOUS_REG((REGHOST_GET_MISCELLANEOUS_REG() & 0xf3) + (ulClock << 2));

	DBGTRACE_ENDFUNCTION("HWCRTC_SelectCurrentDotClock");
}

/* ------------------------------------------------------------
// FUNCTIONS	: HWCRTC_SetDotClock
// ------------------------------------------------------------
// DESCRIPTION	: Set the CRTC pixel clock
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulClock		: Number of the clock to modify
// lFrequency	: Clock frequency in Hz
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWCRTC_SetDotClock(unsigned long ulClock, long lFrequency)
{
	unsigned long ulHardwareID;
	long M, N, P, MinM, MinN, MinP;
	long lVCOOut, lPhaseRef;
	long lError, lErrorMin;
	long lMaxP;

	DBGTRACE_BEGINFUNCTION("HWCRTC_SetDotClock");
	DBGTRACE_NUMERICPARAM(ulClock);
	DBGTRACE_NUMERICPARAM(lFrequency);

	/* Way to program the PLL is different depending on chip */
	ulHardwareID = HWGENERIC_GetHardwareID();
	if(ulHardwareID & HWGENERIC_HAS_HCMOS6PLL)
	{
		lMaxP = 4;
	}
	else
	{
		lMaxP = 5;
	}

	/* Find M, N and P best parameters to program the PLL */
	/* by scanning the whole range of possibility and finding */
	/* the nearest generated value */
	/* M = input divisor, N = feedback divisor, P = post divisor */
	/* (see the PLL programmation description) */

	/* Loop to cover the whole range of M values */
	lErrorMin = 100000000L;
	for(M = 1; M <= 255; M++)
	{
		lPhaseRef = 14318180L / M; /* 14.31818MHz = PLL input frequency */

		/* Phase ref need to reach this range to be a valid M */
		/* (see the PLL programmation description) */
		if((lPhaseRef < 1000000L) || (lPhaseRef > 2000000L))
			continue;

		/* Loop to cover the whole range of N values */
		for(N = 1; N <= 255; N++)
		{
			/* VCO output need to reach this range to be a valid N */
			/* (see the PLL programmation description) */
			if(ulHardwareID & HWGENERIC_HAS_HCMOS6PLL)
			{
				lVCOOut = lPhaseRef * N;
				if((lVCOOut < 128000000L) || (lVCOOut > 256000000L))
					continue;

			}
			else
			{
				lVCOOut = lPhaseRef * N * 2;
				if((lVCOOut < 200000000L) || (lVCOOut > 622000000L))
					continue;

			}

			/* Loop to cover the whole range of P value */
			for(P = 0; P <= lMaxP; P++)
			{
				/* Compute the difference between frequency */
				/* generated by this set of (M, N, P) and   */
				/* the required value                       */
				lError = (lVCOOut / (1 << P)) - lFrequency;
				if(lError < 0)
					lError = -lError;

				/* If nearest than previous value, select it */
				if(lError < lErrorMin)
				{
					lErrorMin = lError;
					MinM = M;
					MinN = N;
					MinP = P;
				}
			}
		}
	}

	/* Set new M, N and P PLL parameters */
	if(ulHardwareID & HWGENERIC_HAS_HCMOS7NEWPLL)
	{
		REGHOST_SET_CHIPSET_REG((unsigned char)(0x42 + ulClock * 2), (unsigned char)(((MinM & 0x0f) << 3) | MinP));
		REGHOST_SET_CHIPSET_REG((unsigned char)(0x42 + ulClock * 2 + 1), (unsigned char)MinN );
	}
	else
	{
		REGHOST_SET_CHIPSET_REG((unsigned char)(0x42 + ulClock * 2), (unsigned char)((((MinM & 0x0f) << 3) | ((MinN & 0xe0) >> 5))));
		REGHOST_SET_CHIPSET_REG((unsigned char)(0x42 + ulClock * 2 + 1), (unsigned char)((((MinP & 0x01) << 7) | ((MinN & 0x1f) << 2) | ((MinP & 0x06) >> 1))));
	}

	DBGTRACE_ENDFUNCTION("HWCRTC_SetDotClock");
}

/* ------------------------------------------------------------
// FUNCTIONS	: HWCRTC_SetRefreshRate
// ------------------------------------------------------------
// DESCRIPTION	: Set the screen refresh rate (set the DCLK PLL
//				  to the required screen refresh rate
//				  multiplied by the number of pixels in a
//				  screen including blank signal)
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulRefreshRate: Screen refresh rate in Hz
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWCRTC_SetRefreshRate(unsigned long ulRefreshRate)
{
	unsigned long ulDCLK;
	unsigned long ulCR6, ulCR7, ulCR25;
	unsigned long ulVerticalTotal, ulHorizontalTotal;
	long lFrequency;

	DBGTRACE_BEGINFUNCTION("HWCRTC_SetRefreshRate");
	DBGTRACE_NUMERICPARAM(ulRefreshRate);

	ulDCLK = HWCRTC_GetCurrentDotClock();

	/* Get current vertical total (number of lines including blank) */
	ulCR6 = ((unsigned long)REGHOST_GET_CRTC_REG(0x6));
	ulCR7 = ((unsigned long)REGHOST_GET_CRTC_REG(0x7));
	ulCR25 = ((unsigned long)REGHOST_GET_CRTC_REG(0x25));
	ulVerticalTotal = ulCR6 + ((ulCR7 & 0x1) << 8) + ((ulCR7 & 0x20) << 4) + ((ulCR25 & 0x1) << 10);

	/* Get current horizontal total (number of pixels per lines including blank) */
	if(REGHOST_GET_SEQUENCER_REG(0x1) & 0x1)
		ulHorizontalTotal = (((unsigned long)REGHOST_GET_CRTC_REG(0x0)) + 5) * 8;
	else
		ulHorizontalTotal = (((unsigned long)REGHOST_GET_CRTC_REG(0x0)) + 5) * 9;

	/* Required clock frequency = horizontal * vertical * refresh */
	lFrequency = ulRefreshRate * ulHorizontalTotal * ulVerticalTotal;

	/* Set DCLK */
	HWCRTC_SetDotClock(ulDCLK, lFrequency);

	DBGTRACE_ENDFUNCTION("HWCRTC_SetRefreshRate");
}

/* ------------------------------------------------------------
// FUNCTIONS	: HWCRTC_SetPixelDepth
// ------------------------------------------------------------
// DESCRIPTION	: Set the graphic mode pixel depth
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulBpp		: Number of bits per pixels
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -1 if the format is supported
//				  0 if the format is not supported
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
unsigned long HWCRTC_SetPixelDepth(unsigned long ulBpp)
{
	unsigned char ucReg28;
	int i;

	DBGTRACE_BEGINFUNCTION("HWCRTC_SetPixelDepth");
	DBGTRACE_NUMERICPARAM(ulBpp);

	ucReg28 = 0x10;

	/* Bypass the look-up table?? */
	if(ulBpp > 8)
	{
		ucReg28 |= 0x8;
	}

	/* Look for the color depth */
	for(i = 0; i < 8; i++)
	{
		if(HWCRTC_pucPixelDepthTable[i] == ulBpp)
		{
			ucReg28 += i;
			break;
		}
	}

	/* Not found ?? */
	if(i == 8)
	{
		DBGTRACE_WARNING("Unsupported color depth");
		return 0;
	}

	/* Set the palette control register */
	REGHOST_SET_CRTC_REG(0x28, ucReg28);

	DBGTRACE_ENDFUNCTION("HWCRTC_SetPixelDepth");

	return -1;
}

/* ------------------------------------------------------------
// FUNCTIONS	: HWCRTC_GetNumberOfModes
// ------------------------------------------------------------
// DESCRIPTION	: Get number of graphics modes available for
//				  the current hardware
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: Number of modes
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
unsigned long HWCRTC_GetNumberOfModes(void)
{
	unsigned long ulHardwareID;
	unsigned long ulNumber;
	int i;

	DBGTRACE_BEGINFUNCTION("HWCRTC_GetNumberOfModes");

	/* Scan the graphics mode list */
	DBGTRACE_LOOP("Loop in graphics mode table");
	ulHardwareID = HWGENERIC_GetHardwareID();
	ulNumber = 0;
	for(i = 0; HWCRTC_psGraphicsModeTable[i].ulGraphicsModeID != HWCRTC_ENDTABLE; i ++)
	{
		/* Is this graphic mode supported? */
		if(ulHardwareID & HWCRTC_psGraphicsModeTable[i].ulSupportedHardware)
		{
			ulNumber ++;
		}
	}

	DBGTRACE_ENDFUNCTION("HWCRTC_GetNumberOfModes");
	DBGTRACE_NUMERICPARAM(ulNumber);

	return ulNumber;
}

/* ------------------------------------------------------------
// FUNCTIONS	: HWCRTC_GetFirstMode
// ------------------------------------------------------------
// DESCRIPTION	: Get informations about the first graphics mode
//				  available in the current hardware
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// pulWidth		: Value to be filled with the width, NULL if not used
// pulHeight	: Value to be filled with the height, NULL if not used
// pulOutput	: Value to be filled with the type of output, NULL if not used
// pulRefresh	: Value to be filled with the possibility of refresh 
//				  rate, NULL if not used
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// pulWidth		: Value filled with the width
// pulHeight	: Value filled with the height
// pulOutput	: Value filled with the type of output
// pulRefresh	: Value filled with the possibility of refresh rate
//				  HWCRTC_DCLK_... flags
// ------------------------------------------------------------
// RETURN		: 0 if no graphics mode availables
//				  <> 0 if values are valid
// ------------------------------------------------------------
// MODIFIED		: HWCRTC_ulCurrentOffset
// ------------------------------------------------------------*/
unsigned long HWCRTC_GetFirstMode(unsigned long *pulWidth, unsigned long *pulHeight, unsigned long *pulOutput, unsigned long *pulRefresh)
{
	unsigned long ulReturnValue;

	DBGTRACE_BEGINFUNCTION("HWCRTC_GetFirstMode");

	HWCRTC_ulCurrentOffset = 0;
	ulReturnValue = HWCRTC_GetNextMode(pulWidth, pulHeight, pulOutput, pulRefresh);

	DBGTRACE_ENDFUNCTION("HWCRTC_GetNumberOfModes");

	return ulReturnValue;
}

/* ------------------------------------------------------------
// FUNCTIONS	: HWCRTC_GetNextMode
// ------------------------------------------------------------
// DESCRIPTION	: Get informations about the next graphics mode
//				  available in the current hardware. This functions
//				  must be called only after a HWCRTC_GetFirstMode
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// pulWidth		: Value to be filled with the width, NULL if not used
// pulHeight	: Value to be filled with the height, NULL if not used
// pulOutput	: Value to be filled with the type of output, NULL if not used
// pulRefresh	: Value to be filled with the possibility of refresh 
//				  rate, NULL if not used
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// pulWidth		: Value filled with the width
// pulHeight	: Value filled with the height
// pulOutput	: Value filled with the type of output
// pulRefresh	: Value filled with the possibility of refresh rate
//				  HWCRTC_DCLK_... flags
// ------------------------------------------------------------
// RETURN		: 0 if no more graphics mode availables
//				  <> 0 if values are valid
// ------------------------------------------------------------
// MODIFIED		: HWCRTC_ulCurrentOffset
// ------------------------------------------------------------*/
unsigned long HWCRTC_GetNextMode(unsigned long *pulWidth, unsigned long *pulHeight, unsigned long *pulOutput, unsigned long *pulRefresh)
{
	unsigned long ulReturn;
	unsigned long ulHardwareID;

	DBGTRACE_BEGINFUNCTION("HWCRTC_GetNextMode");

	/* Scan the graphics mode list */
	DBGTRACE_LOOP("Loop in graphics mode table");
	ulHardwareID = HWGENERIC_GetHardwareID();
	for(; HWCRTC_psGraphicsModeTable[HWCRTC_ulCurrentOffset].ulGraphicsModeID != HWCRTC_ENDTABLE; HWCRTC_ulCurrentOffset ++)
	{
		/* Is this graphic mode supported? */
		if(ulHardwareID & HWCRTC_psGraphicsModeTable[HWCRTC_ulCurrentOffset].ulSupportedHardware)
		{
			break;
		}
	}

	/* End of table ? */
	if(HWCRTC_psGraphicsModeTable[HWCRTC_ulCurrentOffset].ulGraphicsModeID == HWCRTC_ENDTABLE)
	{
		DBGTRACE_BRANCH("End of table");

		ulReturn = 0;
	}
	else
	{
		DBGTRACE_BRANCH("Found another mode");

		if(pulWidth != NULL)
		{
			*pulWidth = HWCRTC_GETWIDTHFROMID(HWCRTC_psGraphicsModeTable[HWCRTC_ulCurrentOffset].ulGraphicsModeID);
		}

		if(pulHeight != NULL)
		{
			*pulHeight = HWCRTC_GETHEIGHTFROMID(HWCRTC_psGraphicsModeTable[HWCRTC_ulCurrentOffset].ulGraphicsModeID);
		}

		if(pulOutput != NULL)
		{
			*pulOutput = HWCRTC_psGraphicsModeTable[HWCRTC_ulCurrentOffset].ulOutput;
		}

		if(pulRefresh != NULL)
		{
			*pulRefresh = HWCRTC_psGraphicsModeTable[HWCRTC_ulCurrentOffset].ulPixelClockFlag;
		}

		HWCRTC_ulCurrentOffset ++;
		ulReturn = 1;
	}

	DBGTRACE_ENDFUNCTION("HWCRTC_GetNextMode");
	DBGTRACE_NUMERICPARAM(ulReturn);

	return ulReturn;
}

/* ------------------------------------------------------------
// FUNCTIONS	: HWCRTC_GetNextMode
// ------------------------------------------------------------
// DESCRIPTION	: Get informations about the next graphics mode
//				  available in the current hardware. This functions
//				  must be called only after a HWCRTC_GetFirstMode
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulNumber		: Give info about the ulNumber graphics mode of
//				  the list (take care only of the mode available)
//				  in the current hardware.
// pulWidth		: Value to be filled with the width, NULL if not used
// pulHeight	: Value to be filled with the height, NULL if not used
// pulOutput	: Value to be filled with the type of output, NULL if not used
// pulRefresh	: Value to be filled with the possibility of refresh 
//				  rate, NULL if not used
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// pulWidth		: Value filled with the width
// pulHeight	: Value filled with the height
// pulOutput	: Value filled with the type of output
// pulRefresh	: Value filled with the possibility of refresh rate
//				  HWCRTC_DCLK_... flags
// ------------------------------------------------------------
// RETURN		: 0 if no graphics mode availables
//				  <> 0 if values are valid
// ------------------------------------------------------------
// MODIFIED		: HWCRTC_ulCurrentOffset
// ------------------------------------------------------------*/
unsigned long HWCRTC_GetModeInfo(unsigned long ulNumber, unsigned long *pulWidth, unsigned long *pulHeight, unsigned long *pulOutput, unsigned long *pulRefresh)
{
	unsigned long ulReturn;
	unsigned long ulHardwareID;
	unsigned long ulIndex;
	long lNumber;

	DBGTRACE_BEGINFUNCTION("HWCRTC_GetModeInfo");
	DBGTRACE_NUMERICPARAM(ulNumber);

	/* Scan the graphics mode list */
	DBGTRACE_LOOP("Loop in graphics mode table");
	ulHardwareID = HWGENERIC_GetHardwareID();

	ulIndex = 0;
	lNumber = ulNumber;
	for( ;lNumber >= 0; lNumber --)
	{
		for(; HWCRTC_psGraphicsModeTable[ulIndex].ulGraphicsModeID != HWCRTC_ENDTABLE; ulIndex ++)
		{
			/* Is this graphic mode supported? */
			if(ulHardwareID & HWCRTC_psGraphicsModeTable[ulIndex].ulSupportedHardware)
			{
				break;
			}
		}

		if(HWCRTC_psGraphicsModeTable[ulIndex].ulGraphicsModeID == HWCRTC_ENDTABLE)
			break;

		ulIndex ++;
	}

	/* End of table ? */
	if(HWCRTC_psGraphicsModeTable[ulIndex].ulGraphicsModeID == HWCRTC_ENDTABLE)
	{
		DBGTRACE_BRANCH("End of table");

		ulReturn = 0;
	}
	else
	{
		DBGTRACE_BRANCH("Found a mode");

		ulIndex --;

		if(pulWidth != NULL)
		{
			*pulWidth = HWCRTC_GETWIDTHFROMID(HWCRTC_psGraphicsModeTable[ulIndex].ulGraphicsModeID);
		}

		if(pulHeight != NULL)
		{
			*pulHeight = HWCRTC_GETHEIGHTFROMID(HWCRTC_psGraphicsModeTable[ulIndex].ulGraphicsModeID);
		}

		if(pulOutput != NULL)
		{
			*pulOutput = HWCRTC_psGraphicsModeTable[ulIndex].ulOutput;
		}

		if(pulRefresh != NULL)
		{
			*pulRefresh = HWCRTC_psGraphicsModeTable[ulIndex].ulPixelClockFlag;
		}

		ulReturn = 1;
	}

	DBGTRACE_ENDFUNCTION("HWCRTC_GetModeInfo");
	DBGTRACE_NUMERICPARAM(ulReturn);

	return ulReturn;
}

/* ------------------------------------------------------------
// FUNCTIONS	: HWCRTC_ScanGraphicsMode
// ------------------------------------------------------------
// DESCRIPTION	: Look for the graphics mode in the graphics
//				  mode table
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulWidth		: Width of the graphic screen to initialize
// ulHeight		: Height of the graphic screen to initialize
// ulOutput		: Type of graphic output (HWCRTC_OUTPUT_...)
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: Index of the graphics mode in the graphics
//				  mode table or
//				  HWCRTC_ERROR_INVALIDGRAPHICSMODE if the graphic 
//			      mode is not in the table
//				  HWCRTC_ERROR_UNSUPPORTEDGRAPHICSMODE if the graphic 
//			      mode is not supported by this hardware
//				  (< 0 if an error)
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
long HWCRTC_ScanGraphicsMode(unsigned long ulWidth, unsigned long ulHeight, unsigned long ulOutput)
{
	unsigned long ulGraphicsModeID;
	unsigned long ulHardwareID;
	unsigned long ulDetected;
	long lIndex;
	int i;

	DBGTRACE_BEGINFUNCTION("HWCRTC_ScanGraphicsMode");
	DBGTRACE_NUMERICPARAM(ulWidth);
	DBGTRACE_NUMERICPARAM(ulHeight);
	DBGTRACE_NUMERICPARAM(ulOutput);

	/* Scan the graphics mode list */
	DBGTRACE_LOOP("Look for graphic mode");
	ulDetected = 0;
	ulHardwareID = HWGENERIC_GetHardwareID();
	ulGraphicsModeID = HWCRTC_GRAPHICSMODE_ID(ulWidth, ulHeight);
	for(i = 0; HWCRTC_psGraphicsModeTable[i].ulGraphicsModeID != HWCRTC_ENDTABLE; i ++)
	{
		/* Is this graphic mode the one required? */
		if((ulGraphicsModeID == HWCRTC_psGraphicsModeTable[i].ulGraphicsModeID) &&
		   ((ulOutput & 0xfffff) == (HWCRTC_psGraphicsModeTable[i].ulOutput & 0xffffff)))
		{
			/* this graphic mode exist */
			ulDetected = 1;

			/* Is this graphic mode supported by the current hardware */
			if(ulHardwareID & HWCRTC_psGraphicsModeTable[i].ulSupportedHardware)
			{
				break;
			}
		}
	}

	/* Graphic mode not found? */
	if(HWCRTC_psGraphicsModeTable[i].ulGraphicsModeID == HWCRTC_ENDTABLE)
	{
		if(ulDetected)
		{
			DBGTRACE_WARNING("Graphics mode not supported in this hardware");
			lIndex = HWCRTC_ERROR_INVALIDGRAPHICSMODE;
		}
		else
		{
			DBGTRACE_WARNING("Graphics mode not supported");
			lIndex = HWCRTC_ERROR_UNSUPPORTEDGRAPHICSMODE;
		}
	}
	else
	{
		lIndex = i;
	}

	DBGTRACE_ENDFUNCTION("HWCRTC_ScanGraphicsMode");
	DBGTRACE_NUMERICPARAM(lIndex);

	return lIndex;
}

/* ------------------------------------------------------------
// FUNCTIONS	: HWCRTC_SetExternalDCLK
// ------------------------------------------------------------
// DESCRIPTION	: Set an external DCLK
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulNumber		: Number of the external clock to initialize
//				  HWCRTC_DCLK_...
// ulRefreshRate: Refresh rate to initialize
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWCRTC_SetExternalDCLK(unsigned long ulNumber, unsigned long ulRefreshRate)
{
	unsigned char ucGPIO;

	DBGTRACE_BEGINFUNCTION("HWCRTC_SetExternalDCLK");
	DBGTRACE_NUMERICPARAM(ulNumber);
	DBGTRACE_NUMERICPARAM(ulRefreshRate);

	if(HWGENERIC_GetHardwareID() & HWGENERIC_PLATFORM_GLORIA)
	{
		ucGPIO = REGHOST_GET_CHIPSET_REG(0x7b);

		/* Set DCLK as input */
		ucGPIO &= ~0x10;

		/* Select DCLK rather VCLK on board */
		ucGPIO |= 0x40;

		REGHOST_SET_CHIPSET_REG(0x7b, ucGPIO);
	}
	else
	{
		/* Set DCLK as input */
		REGHOST_SET_CHIPSET_REG(0x4c, (unsigned char)(REGHOST_GET_CHIPSET_REG(0x4c) & ~0x10));

		/* Unlock access to SMC registers */
		REGHOST_UNLOCK_SMCSUPERIO_REG();

		/* Select module number 8 (GPIOs) */
		REGHOST_SET_SMCSUPERIO_REG(0x07, 0x8);

		/* GPIO 1 is inverted ?? */
		if(REGHOST_GET_SMCSUPERIO_REG(0xe1) & 0x2)
		{
			DBGTRACE_BRANCH("SMC GPIO 1 is inverted");

			/* Lock access to SMC config registers */
			REGHOST_LOCK_SMCSUPERIO_REG();

			/* Enable external 27MHz oscillator */
			REGHOST_SET_SMCGPIO_REG(0x01, (unsigned char)(REGHOST_GET_SMCGPIO_REG(0x01) | 0x02));
		}
		else
		{
			DBGTRACE_BRANCH("SMC GPIO 1 is not inverted");

			/* Lock access to SMC config registers */
			REGHOST_LOCK_SMCSUPERIO_REG();

			/* Enable external 27MHz oscillator */
			REGHOST_SET_SMCGPIO_REG(0x01, (unsigned char)(REGHOST_GET_SMCGPIO_REG(0x01) & ~0x02));
		}
	}

	DBGTRACE_ENDFUNCTION("HWCRTC_SetExternalDCLK");
}

/* ------------------------------------------------------------
// FUNCTIONS	: HWCRTC_SetInternalDCLK
// ------------------------------------------------------------
// DESCRIPTION	: Set internal DCLK
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWCRTC_SetInternalDCLK(void)
{
	unsigned char ucGPIO;

	DBGTRACE_BEGINFUNCTION("HWCRTC_SetInternalDCLK");

	if(HWGENERIC_GetHardwareID() & HWGENERIC_PLATFORM_GLORIA)
	{
		ucGPIO = REGHOST_GET_CHIPSET_REG(0x7b);

		/* Set DCLK as output */
		ucGPIO |= 0x10;

		/* Select VCLK rather DCLK on board */
		ucGPIO &= ~0x40;

		REGHOST_SET_CHIPSET_REG(0x7b, ucGPIO);
	}
	else
	{
		/* Unlock access to SMC registers */
		REGHOST_UNLOCK_SMCSUPERIO_REG();

		/* Select module number 8 (GPIOs) */
		REGHOST_SET_SMCSUPERIO_REG(0x07, 0x8);

		/* GPIO 1 is inverted ?? */
		if(REGHOST_GET_SMCSUPERIO_REG(0xe1) | 0x2)
		{
			DBGTRACE_BRANCH("SMC GPIO 1 is inverted");

			/* Lock access to SMC config registers */
			REGHOST_LOCK_SMCSUPERIO_REG();

			/* Disable external 27MHz oscillator */
			REGHOST_SET_SMCGPIO_REG(0x01, (unsigned char)(REGHOST_GET_SMCGPIO_REG(0x01) & ~0x02));
		}
		else
		{
			DBGTRACE_BRANCH("SMC GPIO 1 is not inverted");

			/* Lock access to SMC config registers */
			REGHOST_LOCK_SMCSUPERIO_REG();

			/* Disable external 27MHz oscillator */
			REGHOST_SET_SMCGPIO_REG(0x01, (unsigned char)(REGHOST_GET_SMCGPIO_REG(0x01) | 0x02));
		}
		
		/* Set DCLK as output */
		REGHOST_SET_CHIPSET_REG(0x4c, (unsigned char)(REGHOST_GET_CHIPSET_REG(0x4c) | 0x10));

	}

	DBGTRACE_ENDFUNCTION("HWCRTC_SetInternalDCLK");
}

/* ------------------------------------------------------------
// FUNCTIONS	: HWCRTC_InitializePixelClock
// ------------------------------------------------------------
// DESCRIPTION	: 
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// lIndex		: Index of the mode in the graphics mode table
// ulRefreshRate: Refresh rate to initialize
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWCRTC_InitializePixelClock(long lIndex, unsigned long ulRefreshRate)
{
	unsigned long ulPixelClockFlag;
	unsigned long ulDCLK;

	DBGTRACE_BEGINFUNCTION("HWCRTC_InitializePixelClock");
	DBGTRACE_NUMERICPARAM(lIndex);
	DBGTRACE_NUMERICPARAM(ulRefreshRate);

	ulPixelClockFlag = HWCRTC_psGraphicsModeTable[lIndex].ulPixelClockFlag;

	if(ulPixelClockFlag & HWCRTC_DCLK_EXTERNAL)
	{
		DBGTRACE_BRANCH("External DCLK");
		HWCRTC_SetExternalDCLK(ulPixelClockFlag & 0xff, ulRefreshRate);
	}
	else
	{
		DBGTRACE_BRANCH("Internal DCLK");
		HWCRTC_SetInternalDCLK();

		if(ulPixelClockFlag & HWCRTC_DCLK_FIXED)
		{
			DBGTRACE_BRANCH("Use a fixed DCLK value");
			DBGTRACE_NUMERICPARAM(ulPixelClockFlag);

			/* Get current used clock */
			ulDCLK = HWCRTC_GetCurrentDotClock();

			/* Initialize this clock with the fixed values */
			HWCRTC_SetDotClock(ulDCLK, (ulPixelClockFlag & 0xffffff) * 1000);
//			REGHOST_SET_CHIPSET_REG((unsigned char)(0x42 + ulDCLK * 2), (unsigned char)(ulPixelClockFlag & 0xff));
//			REGHOST_SET_CHIPSET_REG((unsigned char)(0x42 + ulDCLK * 2 + 1), (unsigned char)((ulPixelClockFlag >> 8) & 0xff));
		}
		else
		{
			DBGTRACE_BRANCH("Adjust DCLK to reach the refresh rate");
			HWCRTC_SetRefreshRate(ulRefreshRate);
		}
	}

	DBGTRACE_ENDFUNCTION("HWCRTC_InitializePixelClock");
}

/* ------------------------------------------------------------
// FUNCTIONS	: HWCRTC_SetCRTCTimings
// ------------------------------------------------------------
// DESCRIPTION	: Set the CRTC timings registers, according
//				  to the graphic mode and the type of output
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// lIndex		: Index of the mode in the graphics mode table
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWCRTC_SetCRTCTimings(unsigned long lIndex)
{
	unsigned char ucMiscReg;
	unsigned char *pucCRTCData;
	int i;

	DBGTRACE_BEGINFUNCTION("HWCRTC_SetCRTCTimings");
	DBGTRACE_NUMERICPARAM(lIndex);

	/* Initialize CRTC timings */
	DBGTRACE_LOOP("Initialize CRTC timings registers");
	pucCRTCData = HWCRTC_psGraphicsModeTable[lIndex].pucTimings;
	for(i = 0; pucCRTCData[i] != 0xff; i += 2 )
	{
		REGHOST_SET_CRTC_REG(pucCRTCData[i], pucCRTCData[i+1]);
	}

	/* Set hsync anf vsync polarity */
	ucMiscReg = REGHOST_GET_MISCELLANEOUS_REG();
	ucMiscReg &= 0x0c;
	ucMiscReg += HWCRTC_psGraphicsModeTable[lIndex].ucMiscReg & 0xf3;
	REGHOST_SET_MISCELLANEOUS_REG(ucMiscReg);

	/* Update display offset */
	HWCRTC_ulOffsetX = HWCRTC_GetOffsetX();
	HWCRTC_ulOffsetY = HWCRTC_GetOffsetY();

	DBGTRACE_ENDFUNCTION("HWCRTC_SetCRTCTimings");
}

/* ------------------------------------------------------------
// FUNCTIONS	: HWCRTC_SetWaterMark
// ------------------------------------------------------------
// DESCRIPTION	: Set the low and high water mark to assure
//				  no glitch will appear on screen
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulWidth		: Width of the graphic screen to initialize
// ulHeight		: Height of the graphic screen to initialize
// ulBpp		: Number of bits per screen pixels
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -1 if the graphic mode is supported
//				  0 if the graphic mode is not supported
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
unsigned long HWCRTC_SetWaterMark(unsigned long ulWidth, unsigned long ulHeight, unsigned long ulBpp)
{
	unsigned long ulInferior;
	unsigned long ulSuperior;
	unsigned long ulFactor;
	unsigned long ulInferiorID;
	unsigned long ulSuperiorID;
	unsigned long ulWaterMarkID;
	unsigned long ulHardwareID;
	unsigned long ulLowWaterMark;
	unsigned long ulHighWaterMark;
	int i;

	DBGTRACE_BEGINFUNCTION("HWCRTC_SetWaterMark");
	DBGTRACE_NUMERICPARAM(ulWidth);
	DBGTRACE_NUMERICPARAM(ulHeight);
	DBGTRACE_NUMERICPARAM(ulBpp);

	/* Look for the closer water mark in the water mark list */
	DBGTRACE_LOOP("Look for graphic mode");
	ulHardwareID = HWGENERIC_GetHardwareID();
	ulWaterMarkID = HWCRTC_WATERMARK_ID(ulWidth, ulHeight, ulBpp);
	ulInferior = 0xffffffff;
	ulSuperior = 0xffffffff;
	ulInferiorID = 0x0;
	ulSuperiorID = 0xffffffff;
	for(i = 0; HWCRTC_psWaterMarkTable[i].ulWaterMarkID != HWCRTC_ENDTABLE; i ++)
	{
		/* This water mark concern the actual hardware ? */
		if(ulHardwareID & HWCRTC_psWaterMarkTable[i].ulSupportedHardware)
		{
			/* Closer than previous inferior value ? */
			if((HWCRTC_psWaterMarkTable[i].ulWaterMarkID <= ulWaterMarkID)
				&& (HWCRTC_psWaterMarkTable[i].ulWaterMarkID > ulInferiorID))
			{
				/* New inferior value */
				ulInferior = i;
				ulInferiorID = HWCRTC_psWaterMarkTable[i].ulWaterMarkID;
			}

			/* Closer than previous superior value ? */
			if((HWCRTC_psWaterMarkTable[i].ulWaterMarkID >= ulWaterMarkID)
				&& (HWCRTC_psWaterMarkTable[i].ulWaterMarkID < ulSuperiorID))
			{
				/* New inferior value */
				ulSuperior = i;
				ulSuperiorID = HWCRTC_psWaterMarkTable[i].ulWaterMarkID;
			}
		}
	}

	/* Water mark is in the list ? */
	if(ulInferiorID == ulSuperiorID)
	{
		DBGTRACE_BRANCH("Water mark found in the list");

		ulLowWaterMark = HWCRTC_psWaterMarkTable[ulInferior].ucLowWaterMark;
		ulHighWaterMark = HWCRTC_psWaterMarkTable[ulInferior].ucHighWaterMark;
	}
	else
	{
		DBGTRACE_BRANCH("Water mark not found in the list");

		if(ulInferior == 0xffffffff)
		{
			DBGTRACE_BRANCH("Take the minimum water mark value");

			ulLowWaterMark = HWCRTC_psWaterMarkTable[ulSuperior].ucLowWaterMark;
			ulHighWaterMark = HWCRTC_psWaterMarkTable[ulSuperior].ucHighWaterMark;
		}
		else if(ulSuperior == 0xffffffff)
		{
			DBGTRACE_BRANCH("Take the maximum water mark value");

			ulLowWaterMark = HWCRTC_psWaterMarkTable[ulInferior].ucLowWaterMark;
			ulHighWaterMark = HWCRTC_psWaterMarkTable[ulInferior].ucHighWaterMark;
		}
		else
		{
			DBGTRACE_BRANCH("Interpolate the water mark value");

			ulFactor = ((ulWaterMarkID - ulInferiorID) * 1024) / (ulSuperiorID - ulInferiorID);
			ulLowWaterMark = HWCRTC_psWaterMarkTable[ulSuperior].ucLowWaterMark - HWCRTC_psWaterMarkTable[ulInferior].ucLowWaterMark;
			ulHighWaterMark = HWCRTC_psWaterMarkTable[ulSuperior].ucHighWaterMark - HWCRTC_psWaterMarkTable[ulInferior].ucHighWaterMark;
			ulLowWaterMark *= ulFactor;
			ulHighWaterMark *= ulFactor;
			ulLowWaterMark /= 1024;
			ulHighWaterMark /= 1024;
			ulLowWaterMark += HWCRTC_psWaterMarkTable[ulInferior].ucLowWaterMark;
			ulHighWaterMark += HWCRTC_psWaterMarkTable[ulInferior].ucHighWaterMark;
		}
	}

	REGHOST_SET_CRTC_REG(0x1b, (unsigned char)ulLowWaterMark);
	REGHOST_SET_CRTC_REG(0x27, (unsigned char)ulHighWaterMark);

	DBGTRACE_ENDFUNCTION("HWCRTC_SetWaterMark");

	return -1;
}

/* ------------------------------------------------------------
// FUNCTIONS	: HWCRTC_InitAdditionnalController
// ------------------------------------------------------------
// DESCRIPTION	: Set the low and high water mark to assure
//				  no glitch will appear on screen
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulOutput		: Type of graphic output (HWCRTC_OUTPUT_...)
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWCRTC_InitAdditionnalController(unsigned long ulOutput)
{
	unsigned long ulController;
	int i;

	DBGTRACE_BEGINFUNCTION("HWCRTC_InitAdditionnalController");

	ulController = HWCRTC_CONTROLER(ulOutput);

	/* Look for an additionnal controller to initialize */
	for(i = 0; (HWCRTC_psControllerTable[i].ulController != HWCRTC_ENDTABLE) && (HWCRTC_psControllerTable[i].ulController != ulController); i++);

	/* Initialize controller if necessary */
	if(HWCRTC_psControllerTable[i].ulController == HWCRTC_ENDTABLE)
	{
		DBGTRACE_BRANCH("No additionnal controller to initialize");
	}
	else
	{
		DBGTRACE_BRANCH("Initilize additionnal controller");
		(*HWCRTC_psControllerTable[i].pfInitFunction)(HWCRTC_MODEL(ulOutput));
	}

	DBGTRACE_ENDFUNCTION("HWCRTC_InitAdditionnalController");
}

/* ------------------------------------------------------------
// FUNCTIONS	: HWCRTC_InitGraphicMode
// ------------------------------------------------------------
// DESCRIPTION	: Initialize a graphic mode and a the 
//				  associated output (monitor, TV, TFT...)
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// psSurface	: Graphic pixels to display (resolution and location)
// ulOutput		: Type of output : HWCRTC_OUTPUT_... parameter 
// ulRefreshRate: Output refresh rate in Hz
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: >= 0 if initialized
//				  HWCRTC_ERROR_... if error
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
long HWCRTC_InitGraphicMode(HWFRAMEBUF_SURFACE *psSurface, unsigned long ulOutput, unsigned long ulRefreshRate)
{
	long lGraphicsModeIndex;

	DBGTRACE_BEGINFUNCTION("HWCRTC_InitGraphicMode");
	DBGTRACE_FRAMEBUFSURFACEPARAM(psSurface);
	DBGTRACE_NUMERICPARAM(ulOutput);
	DBGTRACE_NUMERICPARAM(ulRefreshRate);

	/* Detect the graphics mode */
	lGraphicsModeIndex = HWCRTC_ScanGraphicsMode(psSurface->ulWidth, psSurface->ulHeight, ulOutput);
	if(lGraphicsModeIndex < 0)
	{
			return lGraphicsModeIndex;
	}

	/* Initialiaze common registers */
	HWCRTC_InitializeInternalVGA();

	/* Set color depth */
	if(!HWCRTC_SetPixelDepth(psSurface->ulBpp))
	{
		DBGTRACE_WARNING("Invalid pixel clock");
		return HWCRTC_ERROR_INVALIDPIXELDEPTH;
	}

	/* Set CRTC timings */
	HWCRTC_SetCRTCTimings(lGraphicsModeIndex);

	/* 320x200x8 is a special graphics mode*/
	if((psSurface->ulWidth == 320) &&
	   (psSurface->ulHeight == 200) &&
	   (psSurface->ulBpp == 8))
	{
		/* Set up the double width pixel mode */
		REGHOST_GET_INPUTSTATUS_REG();
		REGHOST_SET_ATTRIBUT_REG(0x10);
		REGHOST_SET_ATTRIBUT_REG(0x41);
		REGHOST_SET_ATTRIBUT_REG(0x20);

		/* Set color mode to 8bit VGA compatible */
		REGHOST_SET_CRTC_REG(0x28, REGHOST_GET_CRTC_REG(0x28) & ~0x7);
	}

	/* Initialize the pixels clock */
	HWCRTC_InitializePixelClock(lGraphicsModeIndex, ulRefreshRate);

	/* Set FIFO water marks */
	if(!HWCRTC_SetWaterMark(psSurface->ulWidth, psSurface->ulHeight, psSurface->ulBpp))
	{
		DBGTRACE_WARNING("Water mark not initilized");
	}

	/* Set source surface */
	HWCRTC_FlipSurface(psSurface->ulBase, psSurface->ulPitch);

	/* Initialize additionnal controller like TV output or TFT */
	HWCRTC_InitAdditionnalController(ulOutput);

	DBGTRACE_ENDFUNCTION("HWCRTC_InitGraphicMode");

	return 0;
}

/* ------------------------------------------------------------
// FUNCTIONS	: HWCRTC_InitTextMode
// ------------------------------------------------------------
// DESCRIPTION	: Initialize a text mode and a the 
//				  associated output (monitor, TV, TFT...)
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// psSurface	: Text resolution to display (resolution and location)
// ulOutput		: Type of output : HWCRTC_OUTPUT_... parameter 
// ulRefreshRate: Output refresh rate in Hz
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: >= 0 if initialized
//				  HWCRTC_ERROR_... if error
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
long HWCRTC_InitTextMode(HWFRAMEBUF_SURFACE *psSurface, unsigned long ulOutput, unsigned long ulRefreshRate)
{
	long lGraphicsModeIndex;

	DBGTRACE_BEGINFUNCTION("HWCRTC_InitGraphicMode");
	DBGTRACE_FRAMEBUFSURFACEPARAM(psSurface);
	DBGTRACE_NUMERICPARAM(ulOutput);
	DBGTRACE_NUMERICPARAM(ulRefreshRate);

	/* Detect the graphics mode */
	lGraphicsModeIndex = HWCRTC_ScanGraphicsMode(psSurface->ulWidth, psSurface->ulHeight, ulOutput);
	if(lGraphicsModeIndex < 0)
	{
			return lGraphicsModeIndex;
	}

	/* Initialiaze common registers */
	HWCRTC_InitializeInternalVGAText();

	/* Set the color palette */
	HWCRTC_SetTextModeDefaultPalette();

	/* Load the text mode font */
	HWCRTC_Load8xNTextFont(HWCRTC_pucFont8x15, 16, 0);

	/* Set CRTC timings */
	HWCRTC_SetCRTCTimings(lGraphicsModeIndex);

	/* Initialize the pixels clock */
	HWCRTC_InitializePixelClock(lGraphicsModeIndex, ulRefreshRate);

	/* Set FIFO water marks */
	if(!HWCRTC_SetWaterMark(psSurface->ulWidth, psSurface->ulHeight, psSurface->ulBpp))
	{
		DBGTRACE_WARNING("Water mark not initilized");
	}

	/* Set source surface */
	HWCRTC_FlipSurface(psSurface->ulBase, psSurface->ulPitch);

	/* Initialize additionnal controller like TV output or TFT */
	HWCRTC_InitAdditionnalController(ulOutput);
	
	DBGTRACE_ENDFUNCTION("HWCRTC_InitGraphicMode");

	return 0;
}

/* ------------------------------------------------------------
// FUNCTIONS	: HWCRTC_GetBase
// ------------------------------------------------------------
// DESCRIPTION	: Get the base address of screen pixels
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// psSurface	: Graphic pixels to display (resolution and location)
// ulOutput		: Type of output : HWCRTC_OUTPUT_... parameter 
// ulRefreshRate: Output refresh rate in Hz
//                (Locked to 50 Hz for TV output)
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: Base address of screen pixels
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
unsigned long HWCRTC_GetBase(void)
{
	unsigned long ulBase;

	DBGTRACE_BEGINFUNCTION("HWCRTC_GetBase");

	/* Get base address from CRTC registers */
	ulBase = REGHOST_GET_CRTC_REG(0x0d);
	ulBase += REGHOST_GET_CRTC_REG(0x0c) << 8;

	if(REGHOST_GET_SEQUENCER_REG(0x6))
	{
		DBGTRACE_BRANCH("Extended CRTC registers are enable");
		ulBase += (REGHOST_GET_CRTC_REG(0x19) & 0x0f) << 16;
	}

	ulBase <<= 2;

	ulBase -= HWCRTC_GetOffsetY() * HWCRTC_GetPitch() + HWCRTC_GetOffsetX() * ((HWCRTC_GetPixelDepth() + 7) >> 3);

	DBGTRACE_ENDFUNCTION("HWCRTC_GetBase");
	DBGTRACE_NUMERICPARAM(ulBase);

	return ulBase;
}

/* ------------------------------------------------------------
// FUNCTIONS	: HWCRTC_GetPitch
// ------------------------------------------------------------
// DESCRIPTION	: Get the number of bytes between two screen
//				  lines
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// psSurface	: Graphic pixels to display (resolution and location)
// ulOutput		: Type of output : HWCRTC_OUTPUT_... parameter
// ulRefreshRate: Output refresh rate in Hz
//                (Locked to 50 Hz for TV output)
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: Pitch of screen
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
unsigned long HWCRTC_GetPitch(void)
{
	unsigned long ulPitch;

	DBGTRACE_BEGINFUNCTION("HWCRTC_GetPitch");

	/* Get pitch from CRTC registers */
	ulPitch = REGHOST_GET_CRTC_REG(0x13) << 3;

	if(REGHOST_GET_SEQUENCER_REG(0x6))
	{
		DBGTRACE_BRANCH("Extended CRTC registers are enable");
		ulPitch += (REGHOST_GET_CRTC_REG(0x19) & 0x30) << 7;
	}

	DBGTRACE_ENDFUNCTION("HWCRTC_GetPitch");
	DBGTRACE_NUMERICPARAM(ulPitch);

	return ulPitch;
}

/* ------------------------------------------------------------
// FUNCTIONS	: HWCRTC_GetPixelDepth
// ------------------------------------------------------------
// DESCRIPTION	: Get the current graphic mode pixel depth
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: Current pixel depth
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
unsigned long HWCRTC_GetPixelDepth(void)
{
	unsigned long ulBpp;

	DBGTRACE_BEGINFUNCTION("HWCRTC_GetPixelDepth");

	ulBpp = HWCRTC_pucPixelDepthTable[REGHOST_GET_CRTC_REG(0x28) & 0x7];

	DBGTRACE_ENDFUNCTION("HWCRTC_GetPixelDepth");
	DBGTRACE_NUMERICPARAM(ulBpp);

	return ulBpp;
}

/* ------------------------------------------------------------
// FUNCTIONS	: HWCRTC_GetScreenWidth
// ------------------------------------------------------------
// DESCRIPTION	: Get width in pixels of the current displayed
//                screen
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: Width in pixels
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
unsigned long HWCRTC_GetScreenWidth(void)
{
	unsigned long ulWidth;

	DBGTRACE_BEGINFUNCTION("HWCRTC_GetScreenWidth");

	/* Width in characters */
	ulWidth = REGHOST_GET_CRTC_REG(0x1) + 1;

	/* Multiply by the size of a characters */
	if(REGHOST_GET_SEQUENCER_REG(0x1) & 0x1)
	{
		DBGTRACE_BRANCH("Character width is 8 pixels");
		ulWidth *= 8;
	}
	else
	{
		DBGTRACE_BRANCH("Character width is 9 pixels");
		ulWidth *= 9;
	}

	DBGTRACE_ENDFUNCTION("HWCRTC_GetScreenWidth");
	DBGTRACE_NUMERICPARAM(ulWidth);

	return ulWidth;
}

/* ------------------------------------------------------------
// FUNCTIONS	: HWCRTC_GetScreenHeight
// ------------------------------------------------------------
// DESCRIPTION	: Get height in pixels of the current displayed
//                screen
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: Height in pixels
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
unsigned long HWCRTC_GetScreenHeight(void)
{
	unsigned long ulHeight;
	unsigned long ulBlankStart;

	DBGTRACE_BEGINFUNCTION("HWCRTC_GetScreenHeight");

	/* End of active video from standard VGA registers */
	ulHeight = REGHOST_GET_CRTC_REG(0x12) + 1;
	ulHeight += (REGHOST_GET_CRTC_REG(0x7) & 0x2) << 7;
	ulHeight += (REGHOST_GET_CRTC_REG(0x7) & 0x40) << 3;

	/* Start of blanking signal from standard VGA registers */
	ulBlankStart = REGHOST_GET_CRTC_REG(0x15) + 1;
	ulBlankStart += (REGHOST_GET_CRTC_REG(0x7) & 0x8) << 5;
	ulBlankStart += (REGHOST_GET_CRTC_REG(0x9) & 0x20) << 4;

	/* If extended registers are used add the last bits */
	if(REGHOST_GET_SEQUENCER_REG(0x6))
	{
		DBGTRACE_BRANCH("Extended CRTC registers are enable");
		ulHeight += (REGHOST_GET_CRTC_REG(0x25) & 0x2) << 9;
		ulBlankStart += (REGHOST_GET_CRTC_REG(0x25) & 0x8) << 7;
	}

	DBGTRACE_INFO("Active video height is :");
	DBGTRACE_NUMERICPARAM(ulHeight);
	DBGTRACE_INFO("Blanking start is :");
	DBGTRACE_NUMERICPARAM(ulBlankStart);

	/* If blanking start before end of active video */
	/* the real displayed screen size is the blanking start */
	if(ulBlankStart < ulHeight)
	{
		DBGTRACE_BRANCH("Blanking start before end of active video");
		ulHeight = ulBlankStart;
	}

	DBGTRACE_ENDFUNCTION("HWCRTC_GetScreenHeight");
	DBGTRACE_NUMERICPARAM(ulHeight);

	return ulHeight;
}

/* ------------------------------------------------------------
// FUNCTIONS	: HWCRTC_GetOffsetX
// ------------------------------------------------------------
// DESCRIPTION	: Get the offset between CRTC base register
//				  and the first displayed pixel
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: Offset in pixels
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
unsigned long HWCRTC_GetOffsetX(void)
{
	unsigned long ulOffset;

	DBGTRACE_BEGINFUNCTION("HWCRTC_GetOffsetX");

	ulOffset = 0;

	DBGTRACE_ENDFUNCTION("HWCRTC_GetOffsetX");
	DBGTRACE_NUMERICPARAM(ulOffset);

	return ulOffset;
}

/* ------------------------------------------------------------
// FUNCTIONS	: HWCRTC_GetOffsetY
// ------------------------------------------------------------
// DESCRIPTION	: Get the offset between CRTC base register
//				  and the first displayed pixel
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: Offset in pixels
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
unsigned long HWCRTC_GetOffsetY(void)
{
	unsigned long ulOffset;
	unsigned long ulHeight;
	unsigned long ulBlankStart;

	DBGTRACE_BEGINFUNCTION("HWCRTC_GetOffsetY");

	/* End of active video from standard VGA registers */
	ulHeight = REGHOST_GET_CRTC_REG(0x12) + 1;
	ulHeight += (REGHOST_GET_CRTC_REG(0x7) & 0x2) << 7;
	ulHeight += (REGHOST_GET_CRTC_REG(0x7) & 0x40) << 3;

	/* Start of blanking signal from standard VGA registers */
	ulBlankStart = REGHOST_GET_CRTC_REG(0x15) + 1;
	ulBlankStart += (REGHOST_GET_CRTC_REG(0x7) & 0x8) << 5;
	ulBlankStart += (REGHOST_GET_CRTC_REG(0x9) & 0x20) << 4;

	/* If extended registers are used add the last bits */
	if(REGHOST_GET_SEQUENCER_REG(0x6))
	{
		DBGTRACE_BRANCH("Extended CRTC registers are enable");
		ulHeight += (REGHOST_GET_CRTC_REG(0x25) & 0x2) << 9;
		ulBlankStart += (REGHOST_GET_CRTC_REG(0x25) & 0x8) << 7;
	}

	DBGTRACE_INFO("Active video height is :");
	DBGTRACE_NUMERICPARAM(ulHeight);
	DBGTRACE_INFO("Blanking start is :");
	DBGTRACE_NUMERICPARAM(ulBlankStart);

	/* If blanking start before end of active video */
	/* it is an NTSC mode, 16 pixels of offset */
	if(ulBlankStart < ulHeight)
	{
		DBGTRACE_BRANCH("Blanking start before end of active video");
		ulOffset = 16;
	}
	else
	{
		DBGTRACE_BRANCH("Blanking start after end of active video");
		ulOffset = 0;
	}

	DBGTRACE_ENDFUNCTION("HWCRTC_GetOffsetY");
	DBGTRACE_NUMERICPARAM(ulOffset);

	return ulOffset;
}

/* ------------------------------------------------------------
// FUNCTIONS	: HWCRTC_FlipSurface
// ------------------------------------------------------------
// DESCRIPTION	: Change the location of displayed pixels
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulBase		: New base address of screen pixels
// ulPitch		: Number of bytes between two screen lines
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWCRTC_FlipSurface(unsigned long ulBase, unsigned long ulPitch)
{
	DBGTRACE_BEGINFUNCTION("HWCRTC_FlipSurface");
	DBGTRACE_NUMERICPARAM(ulBase);
	DBGTRACE_NUMERICPARAM(ulPitch);

	/* Set surface pitch */
	REGHOST_SET_CRTC_REG(0x13, (unsigned char)((ulPitch >> 3) & 0xff));
	REGHOST_SET_CRTC_REG(0x19, (unsigned char)((REGHOST_GET_CRTC_REG(0x19) & ~0x30) | ((ulPitch >> 7) & 0x30)));

	/* Set surface base */
	HWCRTC_FlipBase(ulBase);

	DBGTRACE_ENDFUNCTION("HWCRTC_FlipSurface");
}

/* ------------------------------------------------------------
// FUNCTIONS	: HWCRTC_FlipBase
// ------------------------------------------------------------
// DESCRIPTION	: Change the location of displayed pixels
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulBase		: New base address of screen pixels
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWCRTC_FlipBase(unsigned long ulBase)
{
	DBGTRACE_BEGINFUNCTION("HWCRTC_FlipBase");
	DBGTRACE_NUMERICPARAM(ulBase);

	/* Suppress offset */
	ulBase -= HWCRTC_ulOffsetY * HWCRTC_GetPitch() + HWCRTC_ulOffsetX * ((HWCRTC_GetPixelDepth() + 7) >> 3);

	/* Base address is multiple of 4 bytes */
	ulBase >>= 2;

	/* Set surface base */
	REGHOST_SET_CRTC_REG(0x0d, (unsigned char)(ulBase & 0xff));
	REGHOST_SET_CRTC_REG(0x0c, (unsigned char)((ulBase >> 8) & 0xff));
	REGHOST_SET_CRTC_REG(0x19, (unsigned char)((REGHOST_GET_CRTC_REG(0x19) & 0xf0) | ((ulBase >> 16) & 0x0f)));

	DBGTRACE_ENDFUNCTION("HWCRTC_FlipBase");
}

/* ------------------------------------------------------------
// FUNCTIONS	: HWCRTC_ScrollSurface
// ------------------------------------------------------------
// DESCRIPTION	: Change the location of displayed pixels
//				  inside a virtual desktop
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// psSurface	: The whole desktop
// ulX			: x coordinate of the rectangle to display
// ulY			: y coordinate of the rectangle to display
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWCRTC_ScrollSurface(HWFRAMEBUF_SURFACE *psSurface, unsigned long ulX, unsigned long ulY)
{
	unsigned long ulBase;
	unsigned long ulBppInBytes;

	DBGTRACE_BEGINFUNCTION("HWCRTC_ScrollSurface");
	DBGTRACE_FRAMEBUFSURFACEPARAM(psSurface);
	DBGTRACE_NUMERICPARAM(ulX);
	DBGTRACE_NUMERICPARAM(ulY);

	ulBppInBytes = (psSurface->ulBpp >> 3); 

	/* Compute the new base address */
	ulBase = psSurface->ulBase + ulY * psSurface->ulPitch + ulX * ulBppInBytes;
	if(ulBppInBytes == 3)
	{
		ulBase -= ulBase % 12;
	}
	HWCRTC_FlipBase(ulBase);

	DBGTRACE_ENDFUNCTION("HWCRTC_ScrollSurface");
}

/* ------------------------------------------------------------
// FUNCTIONS	: HWCRTC_ChangePaletteType
// ------------------------------------------------------------
// DESCRIPTION	: Change the type of palette
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulFlag		: Type of palette : HWCRTC_PALETTE_...
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWCRTC_ChangePaletteType(unsigned long ulFlag)
{
	DBGTRACE_BEGINFUNCTION("HWCRTC_ChangePaletteType");
	DBGTRACE_NUMERICPARAM(ulFlag);

	switch(ulFlag)
	{
		case HWCRTC_PALETTE_18BITS:
			REGHOST_SET_CRTC_REG(0x28, (unsigned char)(REGHOST_GET_CRTC_REG(0x28) & ~0x8));
			REGHOST_SET_CRTC_REG(0x1a, (unsigned char)(REGHOST_GET_CRTC_REG(0x1a) | 0x2));
			break;

		case HWCRTC_PALETTE_24BITS:
			REGHOST_SET_CRTC_REG(0x28, (unsigned char)(REGHOST_GET_CRTC_REG(0x28) & ~0x8));
			REGHOST_SET_CRTC_REG(0x1a, (unsigned char)(REGHOST_GET_CRTC_REG(0x1a) & ~0x2));
			break;

		case HWCRTC_PALETTE_BYPASS:
			REGHOST_SET_CRTC_REG(0x28, (unsigned char)(REGHOST_GET_CRTC_REG(0x28) | 0x8));
			break;
	}

	DBGTRACE_ENDFUNCTION("HWCRTC_ChangePaletteType");
}

/* ------------------------------------------------------------
// FUNCTIONS	: HWCRTC_SetPaletteEntry3Bytes
// ------------------------------------------------------------
// DESCRIPTION	: Modify palette entry
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// psEntry		: Table of RGB values, 3 bytes per entry 
// ulFirstEntry	: Number of the first entry to modify (0 - 256)
// ulNumEntries	: Number of entries to modify (up to 256)
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWCRTC_SetPaletteEntry3Bytes(unsigned char *pucEntry, unsigned long ulFirstEntry, unsigned long ulNumEntries)
{
	DBGTRACE_BEGINFUNCTION("HWCRTC_SetPaletteEntry3Bytes");
	DBGTRACE_POINTERPARAM(pucEntry);
	DBGTRACE_NUMERICPARAM(ulFirstEntry);
	DBGTRACE_NUMERICPARAM(ulNumEntries);

	REGHOST_SET_RAMDACINDEX_REG(ulFirstEntry ++);

	for( ; ulNumEntries; ulNumEntries -- )
	{
		REGHOST_SET_RAMDACDATA_REG(*(pucEntry ++));
		REGHOST_SET_RAMDACDATA_REG(*(pucEntry ++));
		REGHOST_SET_RAMDACDATA_REG(*(pucEntry ++));
	}

	DBGTRACE_ENDFUNCTION("HWCRTC_SetPaletteEntry3Bytes");
}

/* ------------------------------------------------------------
// FUNCTIONS	: HWCRTC_SetPaletteEntryBitmap
// ------------------------------------------------------------
// DESCRIPTION	: Modify palette entry from a .bmp style
//				  R and B value are inverted compare to hardware
//				  palette
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// psEntry		: Table of RGB values, 4 bytes per entry
//				  4th values is unused 
// ulFirstEntry	: Number of the first entry to modify (0 - 256)
// ulNumEntries	: Number of entries to modify (up to 256)
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWCRTC_SetPaletteEntryBitmap(unsigned char *pucEntry, unsigned long ulFirstEntry, unsigned long ulNumEntries)
{
	DBGTRACE_BEGINFUNCTION("HWCRTC_SetPaletteEntryBitmap");
	DBGTRACE_POINTERPARAM(pucEntry);
	DBGTRACE_NUMERICPARAM(ulFirstEntry);
	DBGTRACE_NUMERICPARAM(ulNumEntries);

	REGHOST_SET_RAMDACINDEX_REG(ulFirstEntry ++);

	for( ; ulNumEntries; ulNumEntries -- )
	{
		REGHOST_SET_RAMDACDATA_REG(*(pucEntry + 2));
		REGHOST_SET_RAMDACDATA_REG(*(pucEntry + 1));
		REGHOST_SET_RAMDACDATA_REG(*(pucEntry + 0));
		pucEntry += 4;
	}

	DBGTRACE_ENDFUNCTION("HWCRTC_SetPaletteEntryBitmap");
}

/* ------------------------------------------------------------
// FUNCTIONS	: HWCRTC_SetPaletteEntry4Bytes
// ------------------------------------------------------------
// DESCRIPTION	: Modify palette entry
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// psEntry		: Table of RGB values, 4 bytes per entry
//				  4th values is unused 
// ulFirstEntry	: Number of the first entry to modify (0 - 256)
// ulNumEntries	: Number of entries to modify (up to 256)
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWCRTC_SetPaletteEntry4Bytes(unsigned char *pucEntry, unsigned long ulFirstEntry, unsigned long ulNumEntries)
{
	DBGTRACE_BEGINFUNCTION("HWCRTC_SetPaletteEntry4Bytes");
	DBGTRACE_POINTERPARAM(pucEntry);
	DBGTRACE_NUMERICPARAM(ulFirstEntry);
	DBGTRACE_NUMERICPARAM(ulNumEntries);

	REGHOST_SET_RAMDACINDEX_REG(ulFirstEntry ++);

	for( ; ulNumEntries; ulNumEntries -- )
	{
		REGHOST_SET_RAMDACDATA_REG(*(pucEntry ++));
		REGHOST_SET_RAMDACDATA_REG(*(pucEntry ++));
		REGHOST_SET_RAMDACDATA_REG(*(pucEntry ++));
		pucEntry++;
	}

	DBGTRACE_ENDFUNCTION("HWCRTC_SetPaletteEntry4Bytes");
}

/* ------------------------------------------------------------
// FUNCTIONS	: HWCRTC_WaitForVerticalRetrace
// ------------------------------------------------------------
// DESCRIPTION	: Wait for the beginning of next graphic
//				  frame by polling
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWCRTC_WaitForVerticalRetrace(void)
{
	DBGTRACE_BEGINFUNCTION("HWCRTC_WaitForVerticalRetrace");

//	if (!(REGHOST_GET_CRTC_REG(0x17) & 0x80))
//	{
//		DBGTRACE_WARNING("CRTC is off");
//		return;
//	}

	/* While Retrace is Active */
	while (REGHOST_GET_INPUTSTATUS_REG() & 0x8);

	/* While Retrace is Inactive */
	while (!(REGHOST_GET_INPUTSTATUS_REG() & 0x8));

	DBGTRACE_ENDFUNCTION("HWCRTC_WaitForVerticalRetrace");
}

/* ------------------------------------------------------------
// FUNCTIONS	: HWCRTC_WaitForVerticalBlank
// ------------------------------------------------------------
// DESCRIPTION	: Wait for the beginning of next vertical blank
//				  by polling
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWCRTC_WaitForVerticalBlank(void)
{
	DBGTRACE_BEGINFUNCTION("HWCRTC_WaitForVerticalBlank");

//	if (!(REGHOST_GET_CRTC_REG(0x17) & 0x80))
//	{
//		DBGTRACE_WARNING("CRTC is off");
//		return;
//	}

	/* While Retrace is Inactive */
	while (!(REGHOST_GET_INPUTSTATUS_REG() & 0x8));

	/* While Retrace is Active */
	while (REGHOST_GET_INPUTSTATUS_REG() & 0x8);

	DBGTRACE_ENDFUNCTION("HWCRTC_WaitForVerticalBlank");
}

/* ------------------------------------------------------------
// FUNCTIONS	: HWCRTC_StopCRTC
// ------------------------------------------------------------
// DESCRIPTION	: Switch of the screen display
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWCRTC_StopCRTC(void)
{
	DBGTRACE_BEGINFUNCTION("HWCRTC_StopCRTC");

	/* Stop CRTC */
	REGHOST_SET_SEQUENCER_REG(0x01, (unsigned char)(REGHOST_GET_SEQUENCER_REG(0x01) | 0x20));

	DBGTRACE_BEGINFUNCTION("HWCRTC_StopCRTC");
}

/* ------------------------------------------------------------
// FUNCTIONS	: HWCRTC_StartCRTC
// ------------------------------------------------------------
// DESCRIPTION	: Switch on the screen display
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWCRTC_StartCRTC(void)
{
	DBGTRACE_BEGINFUNCTION("HWCRTC_StartCRTC");

	/* Activate CRTC */
	REGHOST_SET_SEQUENCER_REG(0x01, (unsigned char)(REGHOST_GET_SEQUENCER_REG(0x01) & ~0x20));

	DBGTRACE_ENDFUNCTION("HWCRTC_StartCRTC");
}

/* ------------------------------------------------------------
// FUNCTIONS	: HWCRTC_SetTextMode
// ------------------------------------------------------------
// DESCRIPTION	: Initialize text mode
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulOutput		: Type of output : HWCRTC_OUTPUT_... parameter 
// ulRefreshRate: Output refresh rate in Hz
//                (Locked to 50 Hz for TV output)
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWCRTC_SetTextMode(unsigned long ulOutput, unsigned long ulRefreshRate)
{
	long	i;

	/* Enable access to VGA controller I/O */
	REGHOST_SET_MOTHERBOARD_REG(0x00);		
	REGHOST_SET_VIDEOSUBSYSTEM1_REG(0x01);
	REGHOST_SET_MOTHERBOARD_REG(0x28);
	REGHOST_SET_VIDEOSUBSYSTEM2_REG(0x01);
 
	/* Set CRTC registers access to 3d4 */
	REGHOST_SET_MISCELLANEOUS_REG(0x03);

	/* Enable access to extended CRTC registers */
	REGHOST_SET_SEQUENCER_REG(0x06, 0x57);

	/* Initialize sequencer */
	REGHOST_SET_SEQUENCER_REG(0x0, 0x00);
	REGHOST_SET_MISCELLANEOUS_REG(0x2b);
	REGHOST_SET_SEQUENCER_REG(0x0, 0x00);

	/* Don't enable CRTC if disabled */
	if(REGHOST_GET_SEQUENCER_REG(0x1) & 0x20)
	{
		REGHOST_SET_SEQUENCER_REG(0x1, 0x20);
	}
	else
	{
		REGHOST_SET_SEQUENCER_REG(0x1, 0x00);
	}

	/* Set sequencer register */
	REGHOST_SET_SEQUENCER_REG(0x2, 0x03);
	REGHOST_SET_SEQUENCER_REG(0x3, 0x00);
	REGHOST_SET_SEQUENCER_REG(0x4, 0x02);

	/* Initialize graphic controller */
	REGHOST_SET_SEQUENCER_REG(0x06, 0x57);
	REGHOST_SET_SEQUENCER_REG(0x0, 0x3);

	for(i = 0; i < 5; i ++)
	{
		REGHOST_SET_GRAPHICCONTROLLER_REG((unsigned char)i, 0);
	}

	REGHOST_SET_GRAPHICCONTROLLER_REG(5, 0x10);
	REGHOST_SET_GRAPHICCONTROLLER_REG(6, 0x0e);
	REGHOST_SET_GRAPHICCONTROLLER_REG(7, 0x00);
	REGHOST_SET_GRAPHICCONTROLLER_REG(8, 0xff);

	/* Initialize attribut register */
	REGHOST_GET_INPUTSTATUS_REG();

	for(i = 0; i <= 0x14; i++)
	{
		REGHOST_SET_ATTRIBUT_REG((unsigned char)i);
		REGHOST_SET_ATTRIBUT_REG(HWCRTC_pucTextModeAttributTable[i]);
	}

	/* Disable palette access & Enable display */
	REGHOST_SET_ATTRIBUT_REG(0x20);

	/* Set CRTC registers */
	REGHOST_SET_CRTC_REG(0x11, 0x0c);

	for(i = 0; HWCRTC_pucTextModeCRTCTable[i] <= 0x44; i += 2)
	{
		REGHOST_SET_CRTC_REG(HWCRTC_pucTextModeCRTCTable[i], HWCRTC_pucTextModeCRTCTable[i + 1]);
	}

	/* Set the color palette */
	HWCRTC_SetTextModeDefaultPalette();

	/* Load the text mode font */
	HWCRTC_Load8xNTextFont(HWCRTC_pucFont8x15, 16, 0);

	/* Initialize refresh rate */
	HWCRTC_SetRefreshRate(ulRefreshRate);

	/* Initialize additionnal controller like TV output or TFT */
	HWCRTC_InitAdditionnalController(ulOutput);
}


