/* ------------------------------------------------------------
// Copyright 2000 ST Microelectronics
// ------------------------------------------------------------
// This software is the property of ST Microelectronics which
// specifically grants the user the right to modify, use and
// distribute this software provided this notice is not removed
// or altered. All other rights are reserved by ST.
// ------------------------------------------------------------
// ST MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, WITH
// REGARD TO THIS SOFTWARE. IN NO EVENT SHALL ST BE LIABLE FOR
// INCIDENTAL OR CONSEQUENTIAL DAMAGES IN CONNECTION WITH OR
// ARISING FROM THE FURNISHING, PERFORMANCE, OR USE OF THIS
// SOFTWARE.
// ------------------------------------------------------------
// So that all may benefit from your experience, please report
// any problems or suggestions about this software back to ST.
// Please include your name, company, telephone number, email,
// ST product requiring support and question or problem
// encountered.
// ------------------------------------------------------------
// ST Microelectronics
// Technoparc du pays de gex
// 165 rue Edouard Branly bp112
// 06637 St Genis Pouilly cedex
// France
// ------------------------------------------------------------
// To contact us : http://www.st.com/stpc
// ------------------------------------------------------------*/

/* ------------------------------------------------------------
// Copyright (C) 2000 STMicroelectronics
// ------------------------------------------------------------
// NVG/SICL/System Solution
// ------------------------------------------------------------
// FILE NAME	: cursor.c
// LIBRARY		: STPC Graphics Development Kit
// MODULE		: libhard\cursor
// PORTABILITY	: ANSI C
// ------------------------------------------------------------
// REVISION		: Rev 1.00
// DATE:		: March, 1st 2000
// AUTHOR:		: Thierry Vuillaume
// ------------------------------------------------------------
// DESCRIPTION	: Functions to manage the harulare cursor
// ------------------------------------------------------------
// VARIABLES	: HWCURSOR_ulCurrentHeight
//				  HWCURSOR_ulCurrentMask
//				  HWCURSOR_ulShiftedMask1
//				  HWCURSOR_ulShiftedMask2
//				  HWCURSOR_ulSwitch
//				  HWCURSOR_ulShifted
//				  HWCURSOR_ulOffsetX
//				  HWCURSOR_ulOffsetY
//				  HWCURSOR_ulScreenWidth
//				  HWCURSOR_ulIsHide
//				  HWCURSOR_lCurrentX
//				  HWCURSOR_lCurrentY
// ------------------------------------------------------------
// MACROS		: -
// ------------------------------------------------------------
// FUNCTIONS	: HWCURSOR_Init
//				  HWCURSOR_Enable
//				  HWCURSOR_Disable
//				  HWCURSOR_NotifyGraphicsModeChange
//				  HWCURSOR_SetWindow
//				  HWCURSOR_EnhancedChangeCursorPos
//				  HWCURSOR_ChangeCursorPos
//				  HWCURSOR_HideCursor
//				  HWCURSOR_ChangeCursor
//				  HWCURSOR_ChangeColor
// ------------------------------------------------------------*/

/* ------------------------------------------------------------
// REVISION		: Rev 1.1
// DATE			: July, 18th 2000
// AUTHOR		: Thierry Vuillaume
// MODIFICATIONS: Add the clipping of cursor on rigth side 
//				  (was wrongly displayed on the left for a high X
//				  value).
//				  Add the support of NTSC screen with an offset
//				  due to 16 lines missing on top of the screen
// ------------------------------------------------------------
// REVISION		: Rev 1.00
// DATE			: March, 1st 2000
// AUTHOR		: Thierry Vuillaume
// MODIFICATIONS: Initial source code	
// ------------------------------------------------------------*/

#include "trace.h"
#include "host.h"
#include "cursor.h"

/* ------------------------------------------------------------
// VARIABLE		: HWCURSOR_ulCurrentHeight 
// ------------------------------------------------------------
// DESCRIPTION	: Height of the current hardware cursor
// ------------------------------------------------------------*/
long HWCURSOR_ulCurrentHeight;

/* ------------------------------------------------------------
// VARIABLE		: HWCURSOR_ulCurrentMask 
// ------------------------------------------------------------
// DESCRIPTION	: Offset in frame buffer of the current 
//				  cursor masks
// ------------------------------------------------------------*/
unsigned long HWCURSOR_ulCurrentMask;

/* ------------------------------------------------------------
// VARIABLE		: HWCURSOR_ulShiftedMask1 
// ------------------------------------------------------------
// DESCRIPTION	: Offset in frame buffer of a space to store 
//				  temporary masks for a shifted mask
// ------------------------------------------------------------*/
unsigned long HWCURSOR_ulShiftedMask1;

/* ------------------------------------------------------------
// VARIABLE		: HWCURSOR_ulShiftedMask2
// ------------------------------------------------------------
// DESCRIPTION	: Offset in frame buffer of a space to store 
//				  temporary masks for a shifted mask
//				  (need a second mask to be able to build
//				  the next shifted cursor without destroy the
//				  one currently displayed)
// ------------------------------------------------------------*/
unsigned long HWCURSOR_ulShiftedMask2;

/* ------------------------------------------------------------
// VARIABLE		: HWCURSOR_ulSwitch 
// ------------------------------------------------------------
// DESCRIPTION	: Currently temporary used shifted masks
//				  0 : HWCURSOR_ulShiftedMask1 is used
//				  1 : HWCURSOR_ulShiftedMask2 is used
// ------------------------------------------------------------*/
unsigned long HWCURSOR_ulSwitch;

/* ------------------------------------------------------------
// VARIABLE		: HWCURSOR_ulShifted 
// ------------------------------------------------------------
// DESCRIPTION	: 1 if the current displayed cursor is shifted
//				  0 if the current displayed cursor is full
// ------------------------------------------------------------*/
unsigned long HWCURSOR_ulShifted;

/* ------------------------------------------------------------
// VARIABLE		: HWCURSOR_ulIsHide
// ------------------------------------------------------------
// DESCRIPTION	: <> 0 if cursor is in hide mode 
// ------------------------------------------------------------*/
unsigned long HWCURSOR_ulIsHide = 0;

/* ------------------------------------------------------------
// VARIABLE		: HWCURSOR_lCurrentX
// ------------------------------------------------------------
// DESCRIPTION	: Current X coordinate 
// ------------------------------------------------------------*/
long HWCURSOR_lCurrentX = 0xffff;

/* ------------------------------------------------------------
// VARIABLE		: HWCURSOR_lCurrentY
// ------------------------------------------------------------
// DESCRIPTION	: Current Y coordinate
// ------------------------------------------------------------*/
long HWCURSOR_lCurrentY = 0xffff;

/* ------------------------------------------------------------
// VARIABLE		: HWCURSOR_ulOffsetX
// ------------------------------------------------------------
// DESCRIPTION	: Offset between coordinate given to enhanced
//				  function and real screen coordinate
// ------------------------------------------------------------*/
unsigned long HWCURSOR_ulOffsetX;

/* ------------------------------------------------------------
// VARIABLE		: HWCURSOR_ulOffsetY
// ------------------------------------------------------------
// DESCRIPTION	: Offset between coordinate given to enhanced
//				  function and real screen coordinate
// ------------------------------------------------------------*/
unsigned long HWCURSOR_ulOffsetY;

/* ------------------------------------------------------------
// VARIABLE		: HWCURSOR_ulScreenWidth
// ------------------------------------------------------------
// DESCRIPTION	: Width in pixels of the current screen
// ------------------------------------------------------------*/
unsigned long HWCURSOR_ulScreenWidth;

/* ------------------------------------------------------------
// FUNCTION		: HWCURSOR_Init
// ------------------------------------------------------------
// DESCRIPTION	: Init the hardware cursor library
//				  This functions is necessary only when using
//				  HWCURSOR_EnhancedChangeCursorPos
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulTempMask1	: Offset in frame buffer of a space for a
//				  temporary cursor. This can be get from
//				  a HWFRAMEBUF_CursorAlloc call. This parameter
//				  is used only if cursor coordinate can be
//				  negative (need a temporary shifted cursor
//				  because the top and left clipping is not
//				  managed)
// ulTempMask2	: Offset in frame buffer of a space for a
//				  secondary temporary cursor.
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: HWCURSOR_ulShiftedMask1
//				  HWCURSOR_ulShiftedMask2
//				  HWCURSOR_ulSwitch
//				  HWCURSOR_ulShifted
//				  HWCURSOR_ulIsHide
//				  HWCURSOR_ulOffsetX
//				  HWCURSOR_ulOffsetY
//				  HWCURSOR_ulScreenWidth
// ------------------------------------------------------------*/
void HWCURSOR_Init(unsigned long ulTempMask1, unsigned long ulTempMask2)
{
	DBGTRACE_BEGINFUNCTION("HWCURSOR_Init");
	DBGTRACE_NUMERICPARAM(ulTempMask1);
	DBGTRACE_NUMERICPARAM(ulTempMask2);

	HWCURSOR_ulShiftedMask1 = ulTempMask1;
	HWCURSOR_ulShiftedMask2 = ulTempMask2;
	HWCURSOR_ulSwitch = 0;
	HWCURSOR_ulShifted = 0;
	HWCURSOR_ulIsHide = 0;

	HWCURSOR_NotifyGraphicsModeChange();

	DBGTRACE_ENDFUNCTION("HWCURSOR_Init");
}

/* ------------------------------------------------------------
// FUNCTIONS	: HWCURSOR_NotifyGraphicsModeChange
// ------------------------------------------------------------
// DESCRIPTION	: To be send after a graphics mode change
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWCURSOR_NotifyGraphicsModeChange(void)
{
	unsigned long ulBlankStart;
	unsigned long ulScreenHeight;

	DBGTRACE_BEGINFUNCTION("HWCURSOR_NotifyGraphicsModeChange");

	HWCURSOR_ulOffsetX = 0;
	HWCURSOR_ulOffsetY = 0;

	/* get screen width */
	/* Width in characters */
	HWCURSOR_ulScreenWidth = REGHOST_GET_CRTC_REG(0x1) + 1;

	/* Multiply by the size of a characters */
	if(REGHOST_GET_SEQUENCER_REG(0x1) & 0x1)
	{
		DBGTRACE_BRANCH("Character width is 8 pixels");	
		HWCURSOR_ulScreenWidth *= 8;
	}
	else
	{
		DBGTRACE_BRANCH("Character width is 9 pixels");	
		HWCURSOR_ulScreenWidth *= 9;
	}

	/* End of active video from standard VGA registers */
	ulScreenHeight = REGHOST_GET_CRTC_REG(0x12) + 1;
	ulScreenHeight += (REGHOST_GET_CRTC_REG(0x7) & 0x2) << 7;
	ulScreenHeight += (REGHOST_GET_CRTC_REG(0x7) & 0x40) << 3;

	/* Start of blanking signal from standard VGA registers */
	ulBlankStart = REGHOST_GET_CRTC_REG(0x15) + 1;
	ulBlankStart += (REGHOST_GET_CRTC_REG(0x7) & 0x8) << 5;
	ulBlankStart += (REGHOST_GET_CRTC_REG(0x9) & 0x20) << 4;

	/* If extended registers are used add the last bits */
	if(REGHOST_GET_SEQUENCER_REG(0x6))
	{
		DBGTRACE_BRANCH("Extended CRTC registers are enable");
		ulScreenHeight += (REGHOST_GET_CRTC_REG(0x25) & 0x2) << 9;
		ulBlankStart += (REGHOST_GET_CRTC_REG(0x25) & 0x8) << 7;
	}

	/* If blanking start before end of active video */
	/* the real displayed screen size is the blanking start */
	if(ulBlankStart < ulScreenHeight)
	{
		DBGTRACE_BRANCH("Blank signal start before end of active video");
		/* NTSC mode <=> offset of 16 pixels */
		HWCURSOR_ulOffsetY = 16;
	}

	DBGTRACE_ENDFUNCTION("HWCURSOR_NotifyGraphicsModeChange");
}

/* ------------------------------------------------------------
// FUNCTION		: HWCURSOR_EnhancedChangeCursorPos
// ------------------------------------------------------------
// DESCRIPTION	: Modify the location of the current cursor
//				  on screen. This functions manage clipping
//				  on top and left of the screen and the
//				  coordinate can be negative
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// lX			: New x screen coordinate of the left side
//				  of the cursor 
// lY			: New y screen coordinate of the top side
//				  of the cursor
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: HWCURSOR_ulSwitch
//				  HWCURSOR_ulShifted
// ------------------------------------------------------------*/
void HWCURSOR_EnhancedChangeCursorPos(long lX, long lY)
{
	long lShiftx;
	long lShifty;

	DBGTRACE_BEGINFUNCTION("HWCURSOR_EnhancedChangeCursorPos");
	DBGTRACE_NUMERICPARAM(lX);
	DBGTRACE_NUMERICPARAM(lY);

	/* Save current cursor coordinates */
	HWCURSOR_lCurrentX = lX;
	HWCURSOR_lCurrentY = lY;

	/* Hide cursor if out of screen or manualy switched off */
	if((lX >= (long)HWCURSOR_ulScreenWidth) || (HWCURSOR_ulIsHide))
	{
		HWCURSOR_HideCursor();
		return;
	}

	/* The cursor need to be clipped ?? */
	if((lX < 0) || (lY < 0))
	{
		/* One of the coordinate is negative, cursor need */
		/* to be clipped */
		DBGTRACE_BRANCH("Negative coordinate");

		/* Compute the required horizontal and vertical shift */
		lShiftx = lShifty = 0;

		/* Horizontal clipping ?? */
		if(lX < 0)
		{
			DBGTRACE_BRANCH("Negative x coordinate");
			lShiftx = -lX;
			lX = 0;
		}

		/* Horizontal clipping ?? */
		if(lY < 0)
		{
			DBGTRACE_BRANCH("Negative y coordinate");
			lShifty = -lY;
			lY = 0;
		}

		/* The cursor is out of the screen ?? */
		if((lShiftx > 63) || (lShifty >= HWCURSOR_ulCurrentHeight))
		{
			DBGTRACE_BRANCH("Cursor out of screen");
			HWCURSOR_HideCursor();
		}
		else
		{
			DBGTRACE_BRANCH("Cursor on screen");

			/* Build the temporary cursor in the free */
			/* temporary mask to not destroy the cursor */
			/* currently displayed */
			if(HWCURSOR_ulSwitch)
			{
				DBGTRACE_BRANCH("Use mask 1");

				/* Build a temporary shifted mask */
				HWCURSOR_ShiftMask(REGHOST_GetOffscreenPointer(HWCURSOR_ulCurrentMask), REGHOST_GetOffscreenPointer(HWCURSOR_ulShiftedMask1), HWCURSOR_ulCurrentHeight, lShiftx, lShifty);

				/* Display the temporary mask */
				REGHOST_SET_CRTC_REG(0x29, (unsigned char)(HWCURSOR_ulCurrentHeight - lShifty));
				REGHOST_SET_CRTC_REG(0x30, *((unsigned char *)(&HWCURSOR_ulShiftedMask1) + 1));
				REGHOST_SET_CRTC_REG(0x31, *((unsigned char *)(&HWCURSOR_ulShiftedMask1) + 2));
				REGHOST_SET_CRTC_REG(0x32, *((unsigned char *)(&HWCURSOR_ulShiftedMask1) + 3));

				/* Next time, need to use the other temporary mask */
				HWCURSOR_ulSwitch = 0;
			}
			else
			{
				DBGTRACE_BRANCH("Use mask 2");

				/* Build a temporary shifted mask */
				HWCURSOR_ShiftMask(REGHOST_GetOffscreenPointer(HWCURSOR_ulCurrentMask), REGHOST_GetOffscreenPointer(HWCURSOR_ulShiftedMask2), HWCURSOR_ulCurrentHeight, lShiftx, lShifty);

				/* Display the temporary mask */
				REGHOST_SET_CRTC_REG(0x29, (unsigned char)(HWCURSOR_ulCurrentHeight - lShifty));
				REGHOST_SET_CRTC_REG(0x30, (unsigned char)(*((unsigned char *)(&HWCURSOR_ulShiftedMask2) + 1)));
				REGHOST_SET_CRTC_REG(0x31, (unsigned char)(*((unsigned char *)(&HWCURSOR_ulShiftedMask2) + 2)));
				REGHOST_SET_CRTC_REG(0x32, (unsigned char)(*((unsigned char *)(&HWCURSOR_ulShiftedMask2) + 3)));

				/* Next time, need to use the other temporary mask */
				HWCURSOR_ulSwitch = 1;
			}

			/* Indicate the current displayed cursor is shifted */
			HWCURSOR_ulShifted = 1;
		}
	}
	else
	{
		/* Coordinate is positive, no need to clip the cursor */
		DBGTRACE_BRANCH("Positive coordinate");

		/* If last cursor was shifted, set the full cursor */
		if(HWCURSOR_ulShifted)
		{
			REGHOST_SET_CRTC_REG(0x29, (unsigned char)(HWCURSOR_ulCurrentHeight));
			REGHOST_SET_CRTC_REG(0x30, *((unsigned char *)(&HWCURSOR_ulCurrentMask) + 1));
			REGHOST_SET_CRTC_REG(0x31, *((unsigned char *)(&HWCURSOR_ulCurrentMask) + 2));
			REGHOST_SET_CRTC_REG(0x32, *((unsigned char *)(&HWCURSOR_ulCurrentMask) + 3));
			HWCURSOR_ulShifted = 0;
		}
	}

	/* Modify coordinate to reach screen coordinate */
	lX += HWCURSOR_ulOffsetX;
	lY += HWCURSOR_ulOffsetY;

	/* Change cursor screen location */
	HWCURSOR_ChangeCursorPos(lX, lY);

	DBGTRACE_ENDFUNCTION("HWCURSOR_EnhancedChangeCursorPos");
}

/* ------------------------------------------------------------
// FUNCTION		: HWCURSOR_Enable cursor
// ------------------------------------------------------------
// DESCRIPTION	: Enable the hardware cursor display after a disable
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulEnable		: -
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWCURSOR_Enable(void)
{
	DBGTRACE_BEGINFUNCTION("HWCURSOR_Enable");

	HWCURSOR_ulIsHide = 0;
	HWCURSOR_EnhancedChangeCursorPos(HWCURSOR_lCurrentX, HWCURSOR_lCurrentY);

	DBGTRACE_ENDFUNCTION("HWCURSOR_Enable");
}

/* ------------------------------------------------------------
// FUNCTION		: HWCURSOR_Disable cursor
// ------------------------------------------------------------
// DESCRIPTION	: Disable the hardware cursor display
//				  The cursor continue to move when disable
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWCURSOR_Disable(void)
{
	DBGTRACE_BEGINFUNCTION("HWCURSOR_Disable");

	HWCURSOR_ulIsHide = 1;
	HWCURSOR_HideCursor();

	DBGTRACE_ENDFUNCTION("HWCURSOR_Disable");
}

/* ------------------------------------------------------------
// FUNCTION		: HWCURSOR_ChangeCursorPos
// ------------------------------------------------------------
// DESCRIPTION	: Modify the location of the current cursor
//				  on screen. This functions don't manage 
//				  clipping on top and left of the screen and 
//				  the coordinate need to be positive
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulX			: New x screen coordinate of the left side
//				  of the cursor 
// ulY			: New y screen coordinate of the top side
//				  of the cursor
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWCURSOR_ChangeCursorPos(unsigned long ulX, unsigned long ulY)
{
	DBGTRACE_BEGINFUNCTION("HWCURSOR_ChangeCursorPos");
	DBGTRACE_NUMERICPARAM(ulX);
	DBGTRACE_NUMERICPARAM(ulY);

	REGHOST_SET_GE_CURSORXY_REG((ulY << 16) + ulX);

	DBGTRACE_ENDFUNCTION("HWCURSOR_ChangeCursorPos");
}

/* ------------------------------------------------------------
// FUNCTION		: HWCURSOR_HideCursor
// ------------------------------------------------------------
// DESCRIPTION	: Temporary hide the cursor
//				  Use of HWCURSOR_ChangeCursorPos or
//				  HWCURSOR_EnhancedChangeCursorPos will
//				  display again the cursor
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWCURSOR_HideCursor(void)
{
	DBGTRACE_BEGINFUNCTION("HWCURSOR_HideCursor");

	REGHOST_SET_GE_CURSORXY_REG(0xffffffff);

	DBGTRACE_ENDFUNCTION("HWCURSOR_HideCursor");
}

/* ------------------------------------------------------------
// FUNCTION		: HWCURSOR_ChangeCursor
// ------------------------------------------------------------
// DESCRIPTION	: Change the cursor shape
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulBase		: Offset of the new cursor in the frame buffer
//				  (The two mask need to be loaded with
//				  the HWCURSOR_LoadAndMask and 
//				  HWCURSOR_LoadXorMask first)
// ulHeight		: Height of the cursor
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: HWCURSOR_ulCurrentHeight
//				  HWCURSOR_ulCurrentMask
// ------------------------------------------------------------*/
void HWCURSOR_ChangeCursor(unsigned long ulBase, unsigned long ulHeight)
{
	DBGTRACE_BEGINFUNCTION("HWCURSOR_ChangeCursor");
	DBGTRACE_NUMERICPARAM(ulBase);
	DBGTRACE_NUMERICPARAM(ulHeight);

	HWCURSOR_ulCurrentHeight = ulHeight;
	HWCURSOR_ulCurrentMask = ulBase;

	REGHOST_SET_CRTC_REG(0x29, (unsigned char)(ulHeight));
	REGHOST_SET_CRTC_REG(0x30, (unsigned char)(*((unsigned char *)(&ulBase) + 1)));
	REGHOST_SET_CRTC_REG(0x31, (unsigned char)(*((unsigned char *)(&ulBase) + 2)));
	REGHOST_SET_CRTC_REG(0x32, (unsigned char)(*((unsigned char *)(&ulBase) + 3)));

	DBGTRACE_ENDFUNCTION("HWCURSOR_ChangeCursor");
}

/* ------------------------------------------------------------
// FUNCTION		: HWCURSOR_ChangeColor
// ------------------------------------------------------------
// DESCRIPTION	: Change the cursor shape
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulColor0		: First cursor color (RGB888 value)
// ulColor1		: Second cursor color (RGB888 value)
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: HWCURSOR_ulCurrentHeight
//				  HWCURSOR_ulCurrentMask
// ------------------------------------------------------------*/
void HWCURSOR_ChangeColor(unsigned long ulColor0, unsigned long ulColor1)
{
	DBGTRACE_BEGINFUNCTION("HWCURSOR_ChangeColor");
	DBGTRACE_NUMERICPARAM(ulColor0);
	DBGTRACE_NUMERICPARAM(ulColor1);

	REGHOST_SET_CRTC_REG(0x2a, (unsigned char)(ulColor0 & 0xff));
	REGHOST_SET_CRTC_REG(0x2b, (unsigned char)((ulColor0 >> 8) & 0xff));
	REGHOST_SET_CRTC_REG(0x2c, (unsigned char)((ulColor0 >> 16) & 0xff));
	REGHOST_SET_CRTC_REG(0x2d, (unsigned char)(ulColor1 & 0xff));
	REGHOST_SET_CRTC_REG(0x2e, (unsigned char)((ulColor1 >> 8) & 0xff));
	REGHOST_SET_CRTC_REG(0x2f, (unsigned char)((ulColor1 >> 16) & 0xff));

	DBGTRACE_ENDFUNCTION("HWCURSOR_ChangeColor");
}


