/* ------------------------------------------------------------
// Copyright 2000 ST Microelectronics
// ------------------------------------------------------------
// This software is the property of ST Microelectronics which
// specifically grants the user the right to modify, use and
// distribute this software provided this notice is not removed
// or altered. All other rights are reserved by ST.
// ------------------------------------------------------------
// ST MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, WITH
// REGARD TO THIS SOFTWARE. IN NO EVENT SHALL ST BE LIABLE FOR
// INCIDENTAL OR CONSEQUENTIAL DAMAGES IN CONNECTION WITH OR
// ARISING FROM THE FURNISHING, PERFORMANCE, OR USE OF THIS
// SOFTWARE.
// ------------------------------------------------------------
// So that all may benefit from your experience, please report
// any problems or suggestions about this software back to ST.
// Please include your name, company, telephone number, email,
// ST product requiring support and question or problem
// encountered.
// ------------------------------------------------------------
// ST Microelectronics
// Technoparc du pays de gex
// 165 rue Edouard Branly bp112
// 06637 St Genis Pouilly cedex
// France
// ------------------------------------------------------------
// To contact us : http://www.st.com/stpc
// ------------------------------------------------------------*/

/* ------------------------------------------------------------
// Copyright (C) 2000 STMicroelectronics
// ------------------------------------------------------------
// NVG/SICL/System Solution
// ------------------------------------------------------------
// FILE NAME	: framebuf.c
// LIBRARY		: STPC Graphics Development Kit
// MODULE		: libhard\framebuf
// PORTABILITY	: ANSI C
// ------------------------------------------------------------
// REVISION		: Rev 1.1
// DATE			: October, 20th 2000
// AUTHOR		: Thierry Vuillaume
// ------------------------------------------------------------
// DESCRIPTION	: Functions to manage frame buffer memory
//				  (memory allocations...)
// ------------------------------------------------------------
// VARIABLES	: HWFRAMEBUF_ulFBSize
//				  HWFRAMEBUF_psFirstNode
// ------------------------------------------------------------
// MACROS		: -
// ------------------------------------------------------------
// FUNCTIONS	: HWFRAMEBUF_Init
//				  HWFRAMEBUF_GetFrameBufferSize
//				  HWFRAMEBUF_Alloc
//				  HWFRAMEBUF_Free
//				  HWFRAMEBUF_GESurfaceAlloc
//				  HWFRAMEBUF_PatternAlloc
//				  HWFRAMEBUF_HWCursorAlloc
//				  HWFRAMEBUF_ClearFrameBuffer
//				  HWFRAMEBUF_Load8bppPattern
//				  HWFRAMEBUF_Load16bppPattern
//				  HWFRAMEBUF_Load24bppPattern
//				  HWFRAMEBUF_Load8bppConstantPattern
//				  HWFRAMEBUF_Load16bppConstantPattern
//				  HWFRAMEBUF_Load24bppConstantPattern
// ------------------------------------------------------------*/

/* ------------------------------------------------------------
// REVISION		: Rev 1.1
// DATE			: October, 20th 2000
// AUTHOR		: Thierry Vuillaume
// MODIFICATIONS: Added functions to load patterns into frame buffer
//				  HWFRAMEBUF_Load...Pattern
// ------------------------------------------------------------
// REVISION		: Rev 1.0
// DATE			: March, 1st 2000
// AUTHOR		: Thierry Vuillaume
// MODIFICATIONS: Initial source code	
// ------------------------------------------------------------*/

/* include file */
#include "trace.h"
#include "ge.h"
#include "host.h"
#include "framebuf.h"

/* ------------------------------------------------------------
// VARIABLE		: HWFRAMEBUF_ulFBSize 
// ------------------------------------------------------------
// DESCRIPTION	: Size of the frame buffer area reserved
//				  for the allocation 
// ------------------------------------------------------------*/
unsigned long HWFRAMEBUF_ulFBSize;

/* ------------------------------------------------------------
// VARIABLE		: HWFRAMEBUF_psFirstNode 
// ------------------------------------------------------------
// DESCRIPTION	: First node of the allocated block list 
// ------------------------------------------------------------*/
HWFRAMEBUF_NODE *HWFRAMEBUF_psFirstNode = NULL;

/* ------------------------------------------------------------
// FUNCTION		: HWFRAMEBUF_Init
// ------------------------------------------------------------
// DESCRIPTION	: Initialize the library
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulBeginFree	: Offset of the frame buffer block used for
//				  memory allocation by this library
//				  (0 if all frame buffer is used)
// ulSizeFree	: Size of the frame buffer block used for
//				  memory allocation by this library
//				  (0xffffffff to use a block from ulBeginFree
//				  to the end of the frame buffer)
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: HWFRAMEBUF_psFirstNode
//				  HWFRAMEBUF_ulFBSize
// ------------------------------------------------------------*/
void HWFRAMEBUF_Init(unsigned long ulBeginFree, unsigned long ulSizeFree)
{
	unsigned long ulHandle;

	DBGTRACE_BEGINFUNCTION("HWFRAMEBUF_Init");
	DBGTRACE_NUMERICPARAM(ulBeginFree);
	DBGTRACE_NUMERICPARAM(ulSizeFree);

	/* Can be initialize one time only */
	if(HWFRAMEBUF_psFirstNode == NULL)
	{
		/* Get frame buffer size */
		HWFRAMEBUF_ulFBSize = ((unsigned long)REGHOST_GET_CHIPSET_REG(0x36) & 0x3F) << 17;

		/* Check if end offset is higher than frame buffer size */
		if(ulBeginFree + ulSizeFree < HWFRAMEBUF_ulFBSize)
		{
			HWFRAMEBUF_ulFBSize = ulBeginFree + ulSizeFree;
		}

		HWFRAMEBUF_ulFBSize -= ulBeginFree;

		/* Alloc a new node */
		ulHandle = OSAL_MemoryAlloc(sizeof(HWFRAMEBUF_NODE));
		HWFRAMEBUF_psFirstNode = (HWFRAMEBUF_NODE *)OSAL_MemoryGetPtr(ulHandle);

		/* Initialize root node */
		HWFRAMEBUF_psFirstNode->ulHandle = ulHandle;
		HWFRAMEBUF_psFirstNode->pNext = NULL;
		HWFRAMEBUF_psFirstNode->ulBase = ulBeginFree;
		HWFRAMEBUF_psFirstNode->ulSize = 0;
	}

	DBGTRACE_ENDFUNCTION("HWFRAMEBUF_Init");
}

/* ------------------------------------------------------------
// FUNCTION		: HWFRAMEBUF_GetFrameBufferSize
// ------------------------------------------------------------
// DESCRIPTION	: Return the size of the frame buffer in bytes
//				  (between 128Kb and 4Mb)
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
unsigned long HWFRAMEBUF_GetFrameBufferSize(void)
{
	unsigned long ulFrameBufferSize;

	DBGTRACE_BEGINFUNCTION("HWFRAMEBUF_GetFrameBufferSize");

	ulFrameBufferSize = ((unsigned long)REGHOST_GET_CHIPSET_REG(0x36) & 0x3F) << 17;

	DBGTRACE_ENDFUNCTION("HWFRAMEBUF_GetFrameBufferSize");
	DBGTRACE_NUMERICPARAM(ulFrameBufferSize);

	return ulFrameBufferSize;
}

/* ------------------------------------------------------------
// FUNCTION		: HWFRAMEBUF_Alloc
// ------------------------------------------------------------
// DESCRIPTION	: Alloc a block in the frame buffer
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulSize		: Size of the block to allocate
// ulAlign		: Number of byte of alignment
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: Offset of the allocate block in the frame
//				  buffer
//				  Return 0xffffffff if no more free space
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
unsigned long HWFRAMEBUF_Alloc(unsigned long ulSize, unsigned long ulAlign)
{
	HWFRAMEBUF_NODE *psCurrentNode;
	HWFRAMEBUF_NODE *psSelectedNode;
	unsigned long ulCurrent, ulSuivant;
	unsigned long ulWasteSpace;
	unsigned long ulFreeSize;
	unsigned long ulOffset;
	unsigned long ulHandle;

	DBGTRACE_BEGINFUNCTION("HWFRAMEBUF_Alloc");
	DBGTRACE_NUMERICPARAM(ulSize);
	DBGTRACE_NUMERICPARAM(ulAlign);

	/* Scan the list of allocated block */
	/* to insert the new block in the smallest */
	/* place */
	DBGTRACE_LOOP("Scan block list");
	psCurrentNode = HWFRAMEBUF_psFirstNode;
	psSelectedNode = NULL;
	ulWasteSpace = 0xffffffff;
	while(psCurrentNode != NULL)
	{
		/* Begin of the next allocated block */
		if(psCurrentNode->pNext == NULL)
			ulSuivant = HWFRAMEBUF_ulFBSize;
		else
			ulSuivant = psCurrentNode->pNext->ulBase;

		/* End of current allocated block */
		ulCurrent = psCurrentNode->ulBase + psCurrentNode->ulSize;

		/* Alignment */
		ulCurrent += ulAlign - 1;
		ulCurrent -= (ulCurrent % ulAlign);
		
		/* Free space between the two allocated block */
		ulFreeSize = ulSuivant - ulCurrent;

		/* Choose this node only if enough space */
		/* and if the waste space is smaller than the previous */
		/* solution */
		if((ulFreeSize >= ulSize) && (ulFreeSize - ulSize < ulWasteSpace))
		{
			/* Select this node */
			psSelectedNode = psCurrentNode;
			ulWasteSpace = ulFreeSize - ulSize;
			ulOffset = ulCurrent;
		}

		/* Go to next allocated block */
		psCurrentNode = psCurrentNode->pNext;
	}

	/* No more place */
	if(psSelectedNode == NULL)
	{
		DBGTRACE_WARNING("Frame buffer full");
		return 0xffffffffL;
	}

	/* Add the node in the list */
	psCurrentNode = psSelectedNode->pNext;

	/* Allocate a new node */
	ulHandle = OSAL_MemoryAlloc(sizeof(HWFRAMEBUF_NODE));
	psSelectedNode->pNext = (HWFRAMEBUF_NODE *)OSAL_MemoryGetPtr(ulHandle);

	/* Initialize the node */
	psSelectedNode->pNext->ulHandle = ulHandle;
	psSelectedNode->pNext->pNext = psCurrentNode;
	psSelectedNode->pNext->ulBase = ulOffset;
	psSelectedNode->pNext->ulSize = ulSize;

	DBGTRACE_ENDFUNCTION("HWFRAMEBUF_Alloc");
	DBGTRACE_NUMERICPARAM(ulOffset);

	return ulOffset;
}

/* ------------------------------------------------------------
// FUNCTION		: HWFRAMEBUF_Free
// ------------------------------------------------------------
// DESCRIPTION	: Free a block allocated in the frame buffer
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulBase		: Base address of the block to free
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWFRAMEBUF_Free(unsigned long ulBase)
{
	unsigned long ulFound;

	HWFRAMEBUF_NODE *psCurrentNode;
	HWFRAMEBUF_NODE *sSaveNode;

	DBGTRACE_BEGINFUNCTION("HWFRAMEBUF_Free");
	DBGTRACE_NUMERICPARAM(ulBase);

	/* Look for node to supress */
	DBGTRACE_LOOP("Scan the block list");
	psCurrentNode = HWFRAMEBUF_psFirstNode;
	ulFound = 0;
	while(psCurrentNode->pNext != NULL)
	{
		/* Supress the node if found */
		if(psCurrentNode->pNext->ulBase == ulBase)
		{
			sSaveNode = psCurrentNode->pNext;
			psCurrentNode->pNext = psCurrentNode->pNext->pNext;

			OSAL_MemoryFree(sSaveNode->ulHandle);
			ulFound = 1;
			break;
		}

		/* Go to next node */
		psCurrentNode = psCurrentNode->pNext;
	}

	/* Check if the block have been found */
	if(!ulFound)
	{
		DBGTRACE_WARNING("Block not found");
	}

	DBGTRACE_ENDFUNCTION("HWFRAMEBUF_Free");
}

/* ------------------------------------------------------------
// FUNCTION		: HWFRAMEBUF_GESurfaceAlloc
// ------------------------------------------------------------
// DESCRIPTION	: Alloc a 2D surface compatible with
//				  alignment and pitch of the Graphic Engine 
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// psSurface	: Size and pixel format of the surface to
//				  allocate, the pitch can be set to 0
//				  to be automaticly selected or filled
//				  to the required minimum value (will be
//				  ajusted to reach the Graphic Engine
//				  specification)
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// psSurface	: Fill the structure with the base address
//				  and a pitch compatible with the Graphic
//				  engine
//				  Base address = 0xffffffff if no more space
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWFRAMEBUF_GESurfaceAlloc(HWFRAMEBUF_SURFACE *psSurface)
{
	unsigned long ulMinimumPitch;

	DBGTRACE_BEGINFUNCTION("HWFRAMEBUF_GESurfaceAlloc");
	DBGTRACE_FRAMEBUFSURFACEPARAM(psSurface);

	/* Compute the minimum required pitch */
	ulMinimumPitch = (psSurface->ulWidth * psSurface->ulBpp + 7) >> 3;

	/* Check if the required pitch is valide */
	if(psSurface->ulPitch < ulMinimumPitch)
	{
		DBGTRACE_BRANCH("Adjust the pitch");
		psSurface->ulPitch = ulMinimumPitch;
	}

	/* Adjust the pitch to reach the Graphic Engine specifications */
	psSurface->ulPitch = HWGE_GetNearestPitch(psSurface->ulPitch);

	/* Allocate the surface */
	psSurface->ulBase = HWFRAMEBUF_Alloc(psSurface->ulPitch * psSurface->ulHeight, 32);

	DBGTRACE_ENDFUNCTION("HWFRAMEBUF_GESurfaceAlloc");
	DBGTRACE_FRAMEBUFSURFACEPARAM(psSurface);
}

/* ------------------------------------------------------------
// FUNCTION		: HWFRAMEBUF_PatternAlloc
// ------------------------------------------------------------
// DESCRIPTION	: Alloc space for a Graphic Engine pattern
//				  in the frame buffer
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: Offset in the frame buffer of the allocated
//				  space
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
unsigned long HWFRAMEBUF_PatternAlloc(void)
{
	unsigned long ulOffset;

	DBGTRACE_BEGINFUNCTION("HWFRAMEBUF_PatternAlloc");

	ulOffset = HWFRAMEBUF_Alloc(256, 256);

	DBGTRACE_ENDFUNCTION("HWFRAMEBUF_PatternAlloc");
	DBGTRACE_NUMERICPARAM(ulOffset);

	return ulOffset;
}

/* ------------------------------------------------------------
// FUNCTION		: HWFRAMEBUF_HWCursorAlloc
// ------------------------------------------------------------
// DESCRIPTION	: Alloc space for a Graphic Engine hardware
//				  cursor in the frame buffer
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: Offset in the frame buffer of the allocated
//				  space
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
unsigned long HWFRAMEBUF_HWCursorAlloc(void)
{
	unsigned long ulOffset;

	DBGTRACE_BEGINFUNCTION("HWFRAMEBUF_HWCursorAlloc");

	ulOffset = HWFRAMEBUF_Alloc(1024, 1024);

	DBGTRACE_ENDFUNCTION("HWFRAMEBUF_HWCursorAlloc");
	DBGTRACE_NUMERICPARAM(ulOffset);

	return ulOffset;
}

/* ------------------------------------------------------------
// FUNCTION		: HWFRAMEBUF_ClearFrameBuffer
// ------------------------------------------------------------
// DESCRIPTION	: Clear the whole frame buffer
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWFRAMEBUF_ClearFrameBuffer(void)
{
	unsigned long i;

	DBGTRACE_BEGINFUNCTION("HWFRAMEBUF_ClearFrameBuffer");

	for(i = 0; i < HWFRAMEBUF_ulFBSize; i+= 4)
	{
		REGHOST_SET_ONSCREEN32(i, 0);
	}

	DBGTRACE_ENDFUNCTION("HWFRAMEBUF_ClearFrameBuffer");
}

/* ------------------------------------------------------------
// FUNCTION		: HWFRAMEBUF_Load8bppPattern
// ------------------------------------------------------------
// DESCRIPTION	: Load a pattern into frame buffer
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulBase		: Offset in frame buffer to set the pattern
// pPattern		: Pattern pixels (same format as the screen)
//				  8 by 8 pixels
// ulPitch		: Number if bytes between two lines
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWFRAMEBUF_Load8bppPattern(unsigned long ulBase, unsigned char *pPattern, unsigned long ulPitch)
{
	unsigned char *pFBPattern;
	int j;

	DBGTRACE_BEGINFUNCTION("HWFRAMEBUF_Load8bppPattern");
	DBGTRACE_NUMERICPARAM(ulBase);
	DBGTRACE_POINTERPARAM(pPattern);
	DBGTRACE_NUMERICPARAM(ulPitch);

	// load pattern in frame buffer
	pFBPattern = (unsigned char *)REGHOST_GetOffscreenPointer(ulBase);
	for(j = 0; j < 8; j++)
	{
		// Copy one line
		*(unsigned long *)pFBPattern = *(unsigned long *)pPattern;
		*(unsigned long *)(pFBPattern + 4) = *(unsigned long *)(pPattern + 4);

		// Go to next line
		pFBPattern += 32;  // Line size is fixed to 32 in graphic engine
		pPattern += ulPitch;
	}

	DBGTRACE_ENDFUNCTION("HWFRAMEBUF_Load8bppPattern");
}

/* ------------------------------------------------------------
// FUNCTION		: HWFRAMEBUF_Load16bppPattern
// ------------------------------------------------------------
// DESCRIPTION	: Load a pattern into frame buffer
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulBase		: Offset in frame buffer to set the pattern
// pPattern		: Pattern pixels (same format as the screen)
//				  8 by 8 pixels
// ulPitch		: Number if bytes between two lines
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWFRAMEBUF_Load16bppPattern(unsigned long ulBase, unsigned char *pPattern, unsigned long ulPitch)
{
	unsigned char *pFBPattern;
	int j;

	DBGTRACE_BEGINFUNCTION("HWFRAMEBUF_Load16bppPattern");
	DBGTRACE_NUMERICPARAM(ulBase);
	DBGTRACE_POINTERPARAM(pPattern);
	DBGTRACE_NUMERICPARAM(ulPitch);

	// load pattern in frame buffer
	pFBPattern = (unsigned char *)REGHOST_GetOffscreenPointer(ulBase);
	for(j = 0; j < 8; j++)
	{
		// Copy one line
		*(unsigned long *)pFBPattern = *(unsigned long *)pPattern;
		*(unsigned long *)(pFBPattern + 4)  = *(unsigned long *)(pPattern + 4);
		*(unsigned long *)(pFBPattern + 8)  = *(unsigned long *)(pPattern + 8);
		*(unsigned long *)(pFBPattern + 12) = *(unsigned long *)(pPattern + 12);

		// Go to next line
		pFBPattern += 32;  // Line size is fixed to 32 in graphic engine
		pPattern += ulPitch;
	}

	DBGTRACE_ENDFUNCTION("HWFRAMEBUF_Load16bppPattern");
}

/* ------------------------------------------------------------
// FUNCTION		: HWFRAMEBUF_Load24bppPattern
// ------------------------------------------------------------
// DESCRIPTION	: Load a pattern into frame buffer
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulBase		: Offset in frame buffer to set the pattern
// pPattern		: Pattern pixels (same format as the screen)
//				  8 by 8 pixels
// ulPitch		: Number if bytes between two lines
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWFRAMEBUF_Load24bppPattern(unsigned long ulBase, unsigned char *pPattern, unsigned long ulPitch)
{
	unsigned char *pFBPattern;
	int j;

	DBGTRACE_BEGINFUNCTION("HWFRAMEBUF_Load24bppPattern");
	DBGTRACE_NUMERICPARAM(ulBase);
	DBGTRACE_POINTERPARAM(pPattern);
	DBGTRACE_NUMERICPARAM(ulPitch);

	// load pattern in frame buffer
	pFBPattern = (unsigned char *)REGHOST_GetOffscreenPointer(ulBase);
	for(j = 0; j < 8; j++)
	{
		// Copy one line
		*(unsigned long *)pFBPattern = *(unsigned long *)pPattern;
		*(unsigned long *)(pFBPattern + 4)  = *(unsigned long *)(pPattern + 4);
		*(unsigned long *)(pFBPattern + 8)  = *(unsigned long *)(pPattern + 8);
		*(unsigned long *)(pFBPattern + 12) = *(unsigned long *)(pPattern + 12);
		*(unsigned long *)(pFBPattern + 16) = *(unsigned long *)(pPattern + 16);
		*(unsigned long *)(pFBPattern + 20) = *(unsigned long *)(pPattern + 20);

		// Go to next line
		pFBPattern += 32;  // Line size is fixed to 32 in graphic engine
		pPattern += ulPitch;
	}

	DBGTRACE_ENDFUNCTION("HWFRAMEBUF_Load24bppPattern");
}

/* ------------------------------------------------------------
// FUNCTION		: HWFRAMEBUF_Load8bppConstantPattern
// ------------------------------------------------------------
// DESCRIPTION	: Load a constant pattern into frame buffer
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulBase		: Offset in frame buffer to set the pattern
// ulColor		: Constant color to set
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWFRAMEBUF_Load8bppConstantPattern(unsigned long ulBase, unsigned long ulColor)
{
	unsigned char *pFBPattern;
	int j;

	DBGTRACE_BEGINFUNCTION("HWFRAMEBUF_Load8bppConstantPattern");
	DBGTRACE_NUMERICPARAM(ulBase);
	DBGTRACE_NUMERICPARAM(ulColor);

	ulColor &= 0xff;
	ulColor = ulColor + (ulColor << 8);
	ulColor = ulColor + (ulColor << 16);

	// load pattern in frame buffer
	pFBPattern = (unsigned char *)REGHOST_GetOffscreenPointer(ulBase);
	for(j = 0; j < 8; j++)
	{
		// Copy one line
		*(unsigned long *)pFBPattern = ulColor;
		*(unsigned long *)(pFBPattern + 4) = ulColor;

		// Go to next line
		pFBPattern += 32;  // Line size is fixed to 32 in graphic engine
	}

	DBGTRACE_ENDFUNCTION("HWFRAMEBUF_Load8bppConstantPattern");
}

/* ------------------------------------------------------------
// FUNCTION		: HWFRAMEBUF_Load16bppConstantPattern
// ------------------------------------------------------------
// DESCRIPTION	: Load a constant pattern into frame buffer
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulBase		: Offset in frame buffer to set the pattern
// ulColor		: Constant color to set
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWFRAMEBUF_Load16bppConstantPattern(unsigned long ulBase, unsigned long ulColor)
{
	unsigned char *pFBPattern;
	int j;

	DBGTRACE_BEGINFUNCTION("HWFRAMEBUF_Load16bppConstantPattern");
	DBGTRACE_NUMERICPARAM(ulBase);
	DBGTRACE_NUMERICPARAM(ulColor);

	ulColor &= 0xffff;
	ulColor = ulColor + (ulColor << 16);

	// load pattern in frame buffer
	pFBPattern = (unsigned char *)REGHOST_GetOffscreenPointer(ulBase);
	for(j = 0; j < 8; j++)
	{
		// Copy one line
		*(unsigned long *)pFBPattern = ulColor;
		*(unsigned long *)(pFBPattern + 4) = ulColor;

		// Go to next line
		pFBPattern += 32;  // Line size is fixed to 32 in graphic engine
	}

	DBGTRACE_ENDFUNCTION("HWFRAMEBUF_Load16bppConstantPattern");
}

/* ------------------------------------------------------------
// FUNCTION		: HWFRAMEBUF_Load24bppConstantPattern
// ------------------------------------------------------------
// DESCRIPTION	: Load a constant pattern into frame buffer
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulBase		: Offset in frame buffer to set the pattern
// ulColor		: Constant color to set
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWFRAMEBUF_Load24bppConstantPattern(unsigned long ulBase, unsigned long ulColor)
{
	unsigned char *pFBPattern;
	int j;

	DBGTRACE_BEGINFUNCTION("HWFRAMEBUF_Load24bppConstantPattern");
	DBGTRACE_NUMERICPARAM(ulBase);
	DBGTRACE_NUMERICPARAM(ulColor);

	// load pattern in frame buffer
	pFBPattern = (unsigned char *)REGHOST_GetOffscreenPointer(ulBase);
	for(j = 0; j < 8; j++)
	{
		// Copy one line
		*(unsigned long *)pFBPattern = ulColor;
		*(unsigned long *)(pFBPattern + 4) = ulColor;

		// Go to next line
		pFBPattern += 32;  // Line size is fixed to 32 in graphic engine
	}

	DBGTRACE_ENDFUNCTION("HWFRAMEBUF_Load24bppConstantPattern");
}

