/* ------------------------------------------------------------
// Copyright 2000 ST Microelectronics
// ------------------------------------------------------------
// This software is the property of ST Microelectronics which
// specifically grants the user the right to modify, use and
// distribute this software provided this notice is not removed
// or altered. All other rights are reserved by ST.
// ------------------------------------------------------------
// ST MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, WITH
// REGARD TO THIS SOFTWARE. IN NO EVENT SHALL ST BE LIABLE FOR
// INCIDENTAL OR CONSEQUENTIAL DAMAGES IN CONNECTION WITH OR
// ARISING FROM THE FURNISHING, PERFORMANCE, OR USE OF THIS
// SOFTWARE.
// ------------------------------------------------------------
// So that all may benefit from your experience, please report
// any problems or suggestions about this software back to ST.
// Please include your name, company, telephone number, email,
// ST product requiring support and question or problem
// encountered.
// ------------------------------------------------------------
// ST Microelectronics
// Technoparc du pays de gex
// 165 rue Edouard Branly bp112
// 06637 St Genis Pouilly cedex
// France
// ------------------------------------------------------------
// To contact us : http://www.st.com/stpc
// ------------------------------------------------------------*/

/* ------------------------------------------------------------
// Copyright (C) 2000 STMicroelectronics
// ------------------------------------------------------------
// NVG/SICL/System Solution
// ------------------------------------------------------------
// FILE NAME	: ge.c
// LIBRARY		: STPC Graphics Development Kit
// MODULE		: libhard\ge
// PORTABILITY	: ANSI C
// ------------------------------------------------------------
// REVISION		: Rev 1.1
// DATE			: October, 18th 2000
// AUTHOR		: Thierry Vuillaume
// ------------------------------------------------------------
// DESCRIPTION	: Functions to manage graphic 2D acceleration
// ------------------------------------------------------------
// VARIABLES	: HWGE_pulPitchConversion
//				  HWGE_pucROPSuppressPattern
//				  HWGE_pucRopDorkTable
//				  HWGE_pulPitchTable
//				  HWGE_Shift0
//				  HWGE_Shift1
//				  HWGE_Shift2
//				  HWGE_Shift3
//				  HWGE_VirtualDataPort
//				  HWGE_VirtualGERegister
// ------------------------------------------------------------
// MACROS		: -
// ------------------------------------------------------------
// FUNCTIONS	: HWGE_Init
//				  HWGE_GetNearestPitch
//				  HWGE_GetPitchRegValue
//				  HWGE_IsPending
//				  HWGE_IsBusy
// ------------------------------------------------------------*/

/* ------------------------------------------------------------
// REVISION		: Rev 1.1
// DATE			: October, 18th 2000
// AUTHOR		: Thierry Vuillaume
// MODIFICATIONS: Add the suppression of pattern in the ROP,
//				  using a precomputed table built in HWGE_Init
//				  Macro HWGE_ROP_SUPPRESSPATTERN and
//				  HWGE_ROP_OTHERSUPPRESSPATTERN
// ------------------------------------------------------------
// REVISION		: Rev 1.0
// DATE			: March, 1st 2000
// AUTHOR		: Thierry Vuillaume
// ------------------------------------------------------------
// MODIFICATIONS: Initial source code	
// ------------------------------------------------------------*/

#include "trace.h"
#include "host.h"
#include "ge.h"

/* ------------------------------------------------------------
// VARIABLE		: HWGE_Registers
// ------------------------------------------------------------
// DESCRIPTION	: Shadow of registers if shadow mode
// ------------------------------------------------------------*/
HWGE_REGISTERS HWGE_sRegisters;

/* ------------------------------------------------------------
// VARIABLE		: HWGE_pulPitchConversion 
// ------------------------------------------------------------
// DESCRIPTION	: Table of precomputed pitch register value,
//				  used to get the value to be set
//				  in Graphic Engine pitch register
//				  Offset in the table is the pitch value 
//				  devided by 32 
// ------------------------------------------------------------*/
unsigned long HWGE_pulPitchConversion[256];

/* ------------------------------------------------------------
// VARIABLE		: HWGE_pulROPConversion 
// ------------------------------------------------------------
// DESCRIPTION	: Table of precomputed ROP register value,
//				  used to get the value to be set
//				  in Graphic Engine ROP register
//				  Offset in the table is the 8 bits ROP value 
// ------------------------------------------------------------*/
unsigned long HWGE_pulROPConversion[256];

/* ------------------------------------------------------------
// VARIABLE		: HWGE_pucROPSuppressPattern 
// ------------------------------------------------------------
// DESCRIPTION	: Table of precomputed ROP conversion to suppress
//				  the pattern source in a ROP
//				  To be used when the pattern is unified color
//				  for example
// ------------------------------------------------------------*/
unsigned char HWGE_pucROPSuppressPattern[256];

/* ------------------------------------------------------------
// VARIABLE		: HWGE_pucROPSuppressPattern 
// ------------------------------------------------------------
// DESCRIPTION	: Table of precomputed ROP conversion to suppress
//				  the pattern source in a ROP
//				  To be used when the pattern is unified color
//				  for example
// ------------------------------------------------------------*/
// The RopDorkTable is used for the optimization of a solid color brush
// and a ROP that has no source. In this case, we load the color registers
// of enterprise and change the rop from pattern to source. This table
// does the rop conversion. Basically, a rop is the sum of minterms
// a rop without a src (see the HWGE_ROP_HAS_SRC define) can have only 16
// combinations. By taking the rop and doing the approriate shifting
// we create and index into this table. 

#define dte(npnd,npd,pnd,pd) (\
   (npnd ? 0x11 : 0) |  /* !P & !D => !S & !D => (!P & !S & !D) | (P & !S & !D) */ \
   (npd  ? 0x22 : 0) |  /* !P &  D => !S &  D => (!P & !S &  D) | (P & !S &  D) */ \
   (pnd  ? 0x44 : 0) |  /*  P & !D =>  S & !D => (!P &  S & !D) | (P &  S & !D) */ \
   (pd   ? 0x88 : 0) )  /*  P &  D =>  S &  D => (!P &  S &  D) | (P &  S &  D) */

unsigned char HWGE_pucRopDorkTable[16] = {
   dte(0,0,0,0),
   dte(1,0,0,0),
   dte(0,1,0,0),
   dte(1,1,0,0),
   dte(0,0,1,0),
   dte(1,0,1,0),
   dte(0,1,1,0),
   dte(1,1,1,0),
   dte(0,0,0,1),
   dte(1,0,0,1),
   dte(0,1,0,1),
   dte(1,1,0,1),
   dte(0,0,1,1),
   dte(1,0,1,1),
   dte(0,1,1,1),
   dte(1,1,1,1)
};

/* ------------------------------------------------------------
// VARIABLE		: HWGE_pulPitchTable 
// ------------------------------------------------------------
// DESCRIPTION	: List of the pitch supported by the 
//				  Graphic Engine
// ------------------------------------------------------------*/
unsigned long HWGE_pulPitchTable[] =
{
	    0,   32,   64,   96,  128,  160,  192,  224,  256,  288,  320,  352,
	  384,  416,  448,  512,  544,  576,  608,  640,  672,  704,  768,  800,
	  832,  896, 1024, 1056, 1088, 1120, 1152, 1184, 1216, 1248, 1280, 1312,
	 1344, 1376, 1408, 1440, 1472, 1536, 1568, 1600, 1632, 1664, 1696, 1728,
	 1792, 1824, 1856, 1920, 2048, 2080, 2112, 2144, 2176, 2208, 2240, 2272,
	 2304, 2336, 2368, 2400, 2432, 2464, 2496, 2560, 2592, 2624, 2656, 2688,
	 2720, 2752, 2816, 2848, 2880, 2944, 3072, 3104, 3136, 3168, 3200, 3232,
	 3264, 3328, 3360, 3392, 3456, 3584, 3616, 3648, 3712, 3840, 4096, 4128,
	 4160, 4192, 4224, 4256, 4288, 4320, 4352, 4384, 4416, 4448, 4480, 4512,
	 4544, 4608, 4640, 4672, 4704, 4736, 4768, 4800, 4864, 4896, 4928, 4992,
	 5120, 5152, 5184, 5216, 5248, 5280, 5312, 5376, 5408, 5440, 5504, 5632,
	 5664, 5696, 5760, 5888, 6144, 6176, 6208, 6240, 6272, 6304, 6336, 6400,
	 6432, 6464, 6528, 6656, 6688, 6720, 6784, 6912, 7168, 7200, 7232, 7296,
	 7424, 7680
};

/* ------------------------------------------------------------
// VARIABLE		: HWGE_Shift... 
// ------------------------------------------------------------
// DESCRIPTION	: List of possible multiplication value to 
//				  compute a pitch in the format of the
//				  Graphic Engine
// ------------------------------------------------------------*/
unsigned long HWGE_Shift0[] = {0, 0xffffffff, 0xffffffff, 128, 256, 512, 1024, 2048};
unsigned long HWGE_Shift1[] = {0, 0xffffffff, 64, 128, 256, 512, 1024};
unsigned long HWGE_Shift2[] = {0, 32, 64, 128, 256, 512};
unsigned long HWGE_Shift3[] = {0, 1024, 2048, 4096};

/* ------------------------------------------------------------
// VARIABLE		: HWGE_VirtualDataPort 
// ------------------------------------------------------------
// DESCRIPTION	: Linear address of data port, used by the
//				  assembly language functions
// ------------------------------------------------------------*/
unsigned long HWGE_VirtualDataPort;

/* ------------------------------------------------------------
// VARIABLE		: HWGE_VirtualGERegister 
// ------------------------------------------------------------
// DESCRIPTION	: Linear address of the Graphic Engine register,
//				  used by the assembly language functions
// ------------------------------------------------------------*/
unsigned long HWGE_VirtualGERegister;

/* ------------------------------------------------------------
// FUNCTION		: HWGE_Init
// ------------------------------------------------------------
// DESCRIPTION	: Initialize graphic engine library
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: HWGE_pulPitchConversion
//				  HWGE_VirtualDataPort
//				  HWGE_VirtualGERegister
// ------------------------------------------------------------*/
void HWGE_Init(void)
{
	unsigned long ulTempPitch;
	unsigned long i;

	DBGTRACE_BEGINFUNCTION("HWGE_Init");

	/* Initialize table of pre-computed pitch values */
	for(i = 0; i < 256; i++)
	{
		ulTempPitch = HWGE_GetNearestPitch(i << 5);
		HWGE_pulPitchConversion[i] = HWGE_GetPitchRegValue(ulTempPitch);
	}

	/* Initialize table of pre-computed ROP values */
	for(i = 0; i < 256; i++)
	{
		HWGE_pulROPConversion[i] = i;

		if(HWGE_ROP_HAS_PAT(i))
		{
			HWGE_pulROPConversion[i] |= HWGE_ROPREG_PAT;
		}

		if(HWGE_ROP_HAS_DST(i))
		{
			HWGE_pulROPConversion[i] |= HWGE_ROPREG_DST;
		}
	}

	/* Initialize the pre-computed table for ROP pattern suppression */
	for(i = 0; i < 256; i++)
	{
		HWGE_pucROPSuppressPattern[i] = HWGE_pucRopDorkTable[(((unsigned char)i) & 3) | ((((unsigned char)i) & 0x30) >> 2)];
	}

	/* Set values for the assembly language functions */
	HWGE_VirtualDataPort = REGHOST_GetDataPortPointer(0);
	HWGE_VirtualGERegister = REGHOST_GetGERegisterPointer(0);

	DBGTRACE_ENDFUNCTION("HWGE_Init");
}

/* ------------------------------------------------------------
// FUNCTION		: HWGE_GetNearestPitch
// ------------------------------------------------------------
// DESCRIPTION	: Get the nearest pitch supported by the
//				  Graphic Engine (pitch = number of bytes
//				  between two lines of an image)
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulPitch		: Pitch value to process
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: Nearest supported pitch
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
unsigned long HWGE_GetNearestPitch(unsigned long ulPitch)
{
	int i;

	/* Look for nearest upper value of pitch */
	for(i = 0; (i < sizeof(HWGE_pulPitchTable) / sizeof(unsigned long) - 1) && (HWGE_pulPitchTable[i] < ulPitch); i++);

	ulPitch = HWGE_pulPitchTable[i];

	return ulPitch;
}

/* ------------------------------------------------------------
// FUNCTION		: HWGE_GetPitchRegValue
// ------------------------------------------------------------
// DESCRIPTION	: Get the nearest contend of Graphic Engine
//				  pitch register to set for a specific pitch
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulPitch		: Pitch value to process
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: Value to set in the Graphic Engine pitch
//				  register, 0 if the pitch is not supported
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
unsigned long HWGE_GetPitchRegValue(unsigned long ulPitch)
{
	int i;
	unsigned long ulPitchReg;

	ulPitchReg = 0;

	/* Find value of shift 3 */
	for(i = sizeof(HWGE_Shift3) / sizeof(unsigned long) - 1; (i > 0) && (ulPitch < HWGE_Shift3[i]); i--);
	ulPitch -= HWGE_Shift3[i];
	ulPitchReg += (i & 0x3) << 9;

	/* Find value of shift 0 */
	for(i = sizeof(HWGE_Shift0) / sizeof(unsigned long) - 1; (i > 0) && (ulPitch < HWGE_Shift0[i]); i--);
	ulPitch -= HWGE_Shift0[i];
	ulPitchReg += i & 0x7;

	/* Find value of shift 1 */
	for(i = sizeof(HWGE_Shift1) / sizeof(unsigned long) - 1; (i > 0) && (ulPitch < HWGE_Shift1[i]); i--);
	ulPitch -= HWGE_Shift1[i];
	ulPitchReg += (i & 0x7) << 3;

	/* Find value of shift 2 */
	for(i = sizeof(HWGE_Shift2) / sizeof(unsigned long) - 1; (i > 0) && (ulPitch < HWGE_Shift2[i]); i--);
	ulPitch -= HWGE_Shift2[i];
	ulPitchReg += (i & 0x7) << 6;

	/* if rest of pitch not zero, error : not supported value of pitch */
	if(ulPitch)
	{
		DBGTRACE_WARNING("Invalid pitch value");
		ulPitchReg = 0;
	}

	return ulPitchReg;
}

/* ------------------------------------------------------------
// FUNCTION		: HWGE_IsPending
// ------------------------------------------------------------
// DESCRIPTION	: Check if Graphic Engine is pending busy
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: <> 0 if pending busy
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
unsigned long HWGE_IsPending(void)
{
	return (HWGE_GET_STATUS_REG & 0x40000000);
}

/* ------------------------------------------------------------
// FUNCTION		: HWGE_IsBusy
// ------------------------------------------------------------
// DESCRIPTION	: Check if Graphic Engine is busy
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: <> 0 if busy
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
unsigned long HWGE_IsBusy(void)
{
	return (HWGE_GET_STATUS_REG & 0x80000000);
}

