/* ------------------------------------------------------------
// Copyright 2000 ST Microelectronics
// ------------------------------------------------------------
// This software is the property of ST Microelectronics which
// specifically grants the user the right to modify, use and
// distribute this software provided this notice is not removed
// or altered. All other rights are reserved by ST.
// ------------------------------------------------------------
// ST MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, WITH
// REGARD TO THIS SOFTWARE. IN NO EVENT SHALL ST BE LIABLE FOR
// INCIDENTAL OR CONSEQUENTIAL DAMAGES IN CONNECTION WITH OR
// ARISING FROM THE FURNISHING, PERFORMANCE, OR USE OF THIS
// SOFTWARE.
// ------------------------------------------------------------
// So that all may benefit from your experience, please report
// any problems or suggestions about this software back to ST.
// Please include your name, company, telephone number, email,
// ST product requiring support and question or problem
// encountered.
// ------------------------------------------------------------
// ST Microelectronics
// Technoparc du pays de gex
// 165 rue Edouard Branly bp112
// 06637 St Genis Pouilly cedex
// France
// ------------------------------------------------------------
// To contact us : http:/www.st.com/stpc
// ------------------------------------------------------------*/

/* ------------------------------------------------------------
// Copyright (C) 2000 STMicroelectronics
// ------------------------------------------------------------
// NVG/SICL/System Solution
// ------------------------------------------------------------
// FILE NAME	: generic.c
// LIBRARY		: STPC Graphics Development Kit
// MODULE		: libhard\generic
// PORTABILITY	: ANSI C
// ------------------------------------------------------------
// REVISION		: Rev 1.2
// DATE			: August, 25th 2000
// AUTHOR		: Thierry Vuillaume
// ------------------------------------------------------------
// DESCRIPTION	: Generic functions to manage STPC chipset
// ------------------------------------------------------------
// VARIABLES	: HWGENERIC_ulHardwareID
//				  HWGENERIC_psKeywordList
//				  HWGENERIC_ulCurrentEntry
//				  HWGENERIC_ulCurrentType
// ------------------------------------------------------------
// MACROS		: -
// ------------------------------------------------------------
// FUNCTIONS	: HWGENERIC_GetHardwareID
//				  HWGENERIC_GetNumberOfEntry
//				  HWGENERIC_GetFirstEntry
//				  HWGENERIC_GetNextEntry
//				  HWGENERIC_GetEntryFromKeyword
// ------------------------------------------------------------*/

/* ------------------------------------------------------------
// REVISION		: Rev 1.2
// DATE			: August, 25th 2000
// AUTHOR		: Thierry Vuillaume
// MODIFICATIONS: Add detection of STPC Atlas and do a correction
//				  on STPC Industrial detection
// ------------------------------------------------------------
// REVISION		: Rev 1.1
// DATE			: July, 18th 2000
// AUTHOR		: Thierry Vuillaume
// MODIFICATIONS: Add version number functions : 
//				  HWCRTC_GetGDKMinorVersion
//				  HWCRTC_GetGDKMajorVersion
//				  Add function to manage the list of keyword
// ------------------------------------------------------------
// REVISION		: Rev 1.00
// DATE			: March, 1st 2000
// AUTHOR		: Thierry Vuillaume
// MODIFICATIONS: Initial source code	
// ------------------------------------------------------------*/

#include "trace.h"
#include "generic.h"
#include "pci.h"

/* ------------------------------------------------------------
// VARIABLE		: HWGENERIC_ulHardwareID
// ------------------------------------------------------------
// DESCRIPTION	: ID of the current platform
// ------------------------------------------------------------*/
unsigned long HWGENERIC_ulHardwareID = 0xffffffff;

/* ------------------------------------------------------------
// VARIABLE		: HWGENERIC_ulCurrentEntry
// ------------------------------------------------------------
// DESCRIPTION	: Current entry in HWGENERIC_psKeywordList table
//				  when using a search (HWGENERIC_GetFirstEntry
//				  and HWGENERIC_GetNextEntry)
// ------------------------------------------------------------*/
unsigned long HWGENERIC_ulCurrentEntry;

/* ------------------------------------------------------------
// VARIABLE		: HWGENERIC_ulCurrentType
// ------------------------------------------------------------
// DESCRIPTION	: Current type
//				  when using a search (HWGENERIC_GetFirstEntry
//				  and HWGENERIC_GetNextEntry)
// ------------------------------------------------------------*/
unsigned long HWGENERIC_ulCurrentType;

/* ------------------------------------------------------------
// FUNCTION		: HWGENERIC_SetHardwareID
// ------------------------------------------------------------
// DESCRIPTION	: Fix the hardware detection manually
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulHardwareID		: Hardware ID to fix (HWGENERIC_PLATFORM_...)
//				0xffffffff to let the library check
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWGENERIC_SetHardwareID(unsigned long ulHardwareID)
{
	DBGTRACE_BEGINFUNCTION("HWGENERIC_SetHardwareID");
	DBGTRACE_NUMERICPARAM(ulHardwareID);

	HWGENERIC_ulHardwareID = ulHardwareID;

	DBGTRACE_ENDFUNCTION("HWGENERIC_SetHardwareID");
}

/* ------------------------------------------------------------
// FUNCTION		: HWGENERIC_GetHardwareID
// ------------------------------------------------------------
// DESCRIPTION	: Get an ID of the current hardware
//				  platform. This ID can be used with one of
//				  the HWGENERIC_IS_... value to identify the
//				  type of STPC (OR operation) and with one
//				  of the HWGENERIC_HAS_... value to understand
//				  if one feature is available on this platform
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: Identificateur of the hardware : one of the
//				  HWGENERIC_PLATFORM_... parameter
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
unsigned long HWGENERIC_GetHardwareID(void)
{
	unsigned long ulID;
	unsigned long ulNorthBridgeID;
	unsigned long ulSouthBridgeID;
	unsigned long ulAdditionnalID;

	DBGTRACE_BEGINFUNCTION("HWGENERIC_GetHardwareID");

	if(HWGENERIC_ulHardwareID == 0xffffffffL)
	{
		DBGTRACE_BRANCH("First call");

		/* Get PCI device IDs */
		ulNorthBridgeID = REGPCI_ConfigSpaceRead32(0, 0x0b, 0, 0);
		ulSouthBridgeID = REGPCI_ConfigSpaceRead32(0, 0x0c, 0, 0);

		/* Detect the chip */
		ulID = (ulNorthBridgeID & 0xffff0000) + (ulSouthBridgeID >> 16);

		DBGTRACE_INFO("PCI IDs");
		DBGTRACE_NUMERICPARAM(ulNorthBridgeID);
		DBGTRACE_NUMERICPARAM(ulSouthBridgeID);
		DBGTRACE_NUMERICPARAM(ulID);

		switch(ulID)
		{
			case 0x056455cc:
				DBGTRACE_BRANCH("STPC Client");
				HWGENERIC_ulHardwareID = HWGENERIC_PLATFORM_GLORIA;
				break;

			case 0x02090209:
				ulAdditionnalID = REGPCI_ConfigSpaceRead32(0, 0x0c, 1, 0);
				if(ulAdditionnalID == 0x0209104a)
				{
					DBGTRACE_BRANCH("STPC Consumer");
					HWGENERIC_ulHardwareID = HWGENERIC_PLATFORM_EVAL110;
				}
				else
				{
					DBGTRACE_BRANCH("STPC Industrial");
					HWGENERIC_ulHardwareID = HWGENERIC_PLATFORM_EVAL200PC;
				}
				break;

			case 0x020a0210:
				DBGTRACE_BRANCH("STPC ATLAS");
				HWGENERIC_ulHardwareID = HWGENERIC_PLATFORM_EVAL210;
				break;

			case 0x020a021a:
				DBGTRACE_BRANCH("STPC Consumer S");
				HWGENERIC_ulHardwareID = HWGENERIC_PLATFORM_EVAL120;
				break;

			case 0x020a021b:
				/* SIP130 = register 0x81 is always 0 */
				if(REGHOST_GET_CRTC_REG(0x81))
				{
					DBGTRACE_BRANCH("STPC Consumer IIA");
					HWGENERIC_ulHardwareID = HWGENERIC_PLATFORM_EVAL140;
				}
				else
				{
					/* if 0, could be either a SIP130 or a SIP140 */
					/* Try to modify the register                 */
					REGHOST_SET_CRTC_REG(0x81, 0x10);
					if(REGHOST_GET_CRTC_REG(0x81))
					{
						DBGTRACE_BRANCH("STPC Consumer II A");
						HWGENERIC_ulHardwareID = HWGENERIC_PLATFORM_EVAL140;
						REGHOST_SET_CRTC_REG(0x81, 0x0);
					}
					else
					{
						DBGTRACE_BRANCH("STPC Consumer II A");
						HWGENERIC_ulHardwareID = HWGENERIC_PLATFORM_EVAL130;
					}
				}
				break;

			default:
				DBGTRACE_WARNING("Unknown version of chip");
				HWGENERIC_ulHardwareID = 0;
		}
	}

	DBGTRACE_ENDFUNCTION("HWGENERIC_GetHardwareID");
	DBGTRACE_NUMERICPARAM(HWGENERIC_ulHardwareID);

	return HWGENERIC_ulHardwareID;
}

/* ------------------------------------------------------------
// FUNCTION		: SWGENERIC_CompareKeyword
// ------------------------------------------------------------
// DESCRIPTION	: Compare two keyword string
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// pucKeyword1	: First keyword to compare
// pucKeyword2	: Second keyword to compare
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: 0 if keyword are different
//				  <> 0 if keyword are the same
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
unsigned long HWGENERIC_CompareKeyword(unsigned char *pucKeyword1, unsigned char *pucKeyword2)
{
	int i;

	for(i = 0; pucKeyword1[i] && pucKeyword2[i] && ((pucKeyword1[i] | 0x20) == (pucKeyword2[i] | 0x20)); i++);

	return (!pucKeyword1[i]) && (!pucKeyword2[i]);
}

/* ------------------------------------------------------------
// FUNCTION		: HWGENERIC_GetNumberOfEntry
// ------------------------------------------------------------
// DESCRIPTION	: Get the number of values in the list
//				  corresponding to a specific type of value
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulType		: Type of value (HWGENERIC_KEYWORDTYPE_...)
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: Number of entries
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
unsigned long HWGENERIC_GetNumberOfEntry(unsigned long ulType)
{
	unsigned long ulNumber;
	int i;

	DBGTRACE_BEGINFUNCTION("HWGENERIC_GetNumberOfEntry");

	ulNumber = 0;

	for(i = 0; HWGENERIC_psKeywordList[i].pucKeyword != NULL; i ++)
	{
		if(HWGENERIC_psKeywordList[i].ulType == ulType)
		{
			ulNumber ++;
		}
	}

	DBGTRACE_ENDFUNCTION("HWGENERIC_GetNumberOfEntry");
	DBGTRACE_NUMERICPARAM(ulNumber);

	return ulNumber;
}

/* ------------------------------------------------------------
// FUNCTION		: HWGENERIC_GetFirstEntry
// ------------------------------------------------------------
// DESCRIPTION	: Get the index of the first entry corresponding
//				  to a type of value
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulType		: Type of value (HWGENERIC_KEYWORDTYPE_...)
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: Index in the table of the first entry
//				  HWGENERIC_NOENTRY if no entry
// ------------------------------------------------------------
// MODIFIED		: HWGENERIC_ulCurrentEntry
// ------------------------------------------------------------*/
unsigned long HWGENERIC_GetFirstEntry(unsigned long ulType)
{
	DBGTRACE_BEGINFUNCTION("HWGENERIC_GetFirstEntry");
	DBGTRACE_NUMERICPARAM(ulType);

	HWGENERIC_ulCurrentEntry = 0;
	HWGENERIC_ulCurrentType = ulType;

	DBGTRACE_ENDFUNCTION("HWGENERIC_GetFirstEntry");

	return HWGENERIC_GetNextEntry();
}

/* ------------------------------------------------------------
// FUNCTION		: HWGENERIC_GetNextEntry
// ------------------------------------------------------------
// DESCRIPTION	: Get the index of the next entry corresponding
//				  to a type of value
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: Index of the entry in the table
//				  HWGENERIC_NOENTRY if no entry
// ------------------------------------------------------------
// MODIFIED		: HWGENERIC_ulCurrentEntry
// ------------------------------------------------------------*/
unsigned long HWGENERIC_GetNextEntry(void)
{
	unsigned long ulEntry;
	unsigned long ulHardwareID;

	DBGTRACE_BEGINFUNCTION("HWGENERIC_GetNextEntry");

	/* Get the hardware ID */
	ulHardwareID = HWGENERIC_GetHardwareID();

	for(; (HWGENERIC_psKeywordList[HWGENERIC_ulCurrentEntry].pucKeyword != NULL) 
	    &&((HWGENERIC_psKeywordList[HWGENERIC_ulCurrentEntry].ulType != HWGENERIC_ulCurrentType) 
		  || (!(HWGENERIC_psKeywordList[HWGENERIC_ulCurrentEntry].ulSupportedHardware & ulHardwareID))
		  ); HWGENERIC_ulCurrentEntry ++);

	if(HWGENERIC_psKeywordList[HWGENERIC_ulCurrentEntry].pucKeyword == NULL)
	{
		ulEntry = HWGENERIC_NOENTRY;
	}
	else
	{
		ulEntry = HWGENERIC_ulCurrentEntry;
		HWGENERIC_ulCurrentEntry ++;
	}

	DBGTRACE_ENDFUNCTION("HWGENERIC_GetNextEntry");
	DBGTRACE_NUMERICPARAM(ulEntry);

	return ulEntry;
}

/* ------------------------------------------------------------
// FUNCTION		: HWGENERIC_GetEntryFromKeyword
// ------------------------------------------------------------
// DESCRIPTION	: Get the entry number in the list of a value
//				  identified by its keyword
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// psList		: List of values, last entry in the list
//				  contain NULL as pucKeyword parameter
// pucKeyword	: Keyword which identify the value
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: Entry number, HWGENERIC_INVALIDKEYWORD if not found
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
unsigned long HWGENERIC_GetEntryFromKeyword(unsigned char *pucKeyword)
{
	unsigned long ulEntry;

	DBGTRACE_BEGINFUNCTION("HWGENERIC_GetEntryFromKeyword");

	for(ulEntry = 0; 
	   (HWGENERIC_psKeywordList[ulEntry].pucKeyword != NULL) 
	&& (!HWGENERIC_CompareKeyword(pucKeyword, HWGENERIC_psKeywordList[ulEntry].pucKeyword))
	   ; ulEntry ++);

	if(HWGENERIC_psKeywordList[ulEntry].pucKeyword == NULL)
	{
		DBGTRACE_WARNING("Invalid keyword");
		ulEntry = HWGENERIC_INVALIDKEYWORD;
	}

	DBGTRACE_ENDFUNCTION("HWGENERIC_GetEntryFromKeyword");
	DBGTRACE_NUMERICPARAM(ulEntry);

	return ulEntry;
}


