/* ------------------------------------------------------------
// Copyright 2000 ST Microelectronics
// ------------------------------------------------------------
// This software is the property of ST Microelectronics which
// specifically grants the user the right to modify, use and
// distribute this software provided this notice is not removed
// or altered. All other rights are reserved by ST.
// ------------------------------------------------------------
// ST MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, WITH
// REGARD TO THIS SOFTWARE. IN NO EVENT SHALL ST BE LIABLE FOR
// INCIDENTAL OR CONSEQUENTIAL DAMAGES IN CONNECTION WITH OR
// ARISING FROM THE FURNISHING, PERFORMANCE, OR USE OF THIS
// SOFTWARE.
// ------------------------------------------------------------
// So that all may benefit from your experience, please report
// any problems or suggestions about this software back to ST.
// Please include your name, company, telephone number, email,
// ST product requiring support and question or problem
// encountered.
// ------------------------------------------------------------
// ST Microelectronics
// Technoparc du pays de gex
// 165 rue Edouard Branly bp112
// 06637 St Genis Pouilly cedex
// France
// ------------------------------------------------------------
// To contact us : http://www.st.com/stpc
// ------------------------------------------------------------*/

/* ------------------------------------------------------------
// Copyright (C) 2000 STMicroelectronics
// ------------------------------------------------------------
// NVG/SICL/System Solution
// ------------------------------------------------------------
// FILE NAME	: host.c
// LIBRARY		: STPC Graphics Development Kit
// MODULE		: libreg\host
// PORTABILITY	: ANSI C
// ------------------------------------------------------------
// REVISION		: Rev 1.3
// DATE			: December, 6th 2000
// AUTHOR		: Thierry Vuillaume
// ------------------------------------------------------------
// DESCRIPTION	: Functions to manage access to the CPU host
//				  bus (I/O and memory address space)
// ------------------------------------------------------------
// VARIABLES	: REGHOST_ChipsetResources
//				  REGHOST_ucSaveSeq6
//				  REGHOST_ucSaveCRTC1a
//				  REGHOST_ucSaveCRTC1f
// ------------------------------------------------------------
// MACROS		: -
// ------------------------------------------------------------
// FUNCTIONS	: REGHOST_Init
//				  REGHOST_GetGBASE
//				  REGHOST_IndexedIORead8
//				  REGHOST_IndexedIOWrite8
//				  REGHOST_IndexedIORead2x8
//				  REGHOST_IndexedIOWrite2x8
//				  REGHOST_TFTInterfaceWrite16
// ------------------------------------------------------------*/

/* ------------------------------------------------------------
// REVISION		: Rev 1.3
// DATE			: December, 6th 2000
// AUTHOR		: Thierry Vuillaume
// MODIFICATIONS: Add access to STPC Atlas: Alpha Font Engine,
//				  Line Drawing Engine and memory mapped TFT
// ------------------------------------------------------------
// REVISION		: Rev 1.2
// DATE			: August, 25th 2000
// AUTHOR		: Thierry Vuillaume
// MODIFICATIONS: Add function to get the TFT controller base address
//				  on STPC Industrial and STPC Atlas
// ------------------------------------------------------------
// REVISION		: Rev 1.1
// DATE			: July, 18th 2000
// AUTHOR		: Thierry Vuillaume
// MODIFICATIONS: Add macros to access SMC registers
// ------------------------------------------------------------
// REVISION		: Rev 1.00
// DATE			: March, 1st 2000
// AUTHOR		: Thierry Vuillaume
// MODIFICATIONS: Initial source code	
// ------------------------------------------------------------*/

#include "trace.h"
#include "host.h"
#include "generic.h"

/* ------------------------------------------------------------
// VARIABLE		: REGHOST_ChipsetResources 
// ------------------------------------------------------------
// DESCRIPTION	: List of all the I/O and memory mapped registers
//				  with the correspondant physical and logical
//				  address
// ------------------------------------------------------------*/
REGHOST_CHIPSET_RESOURCES REGHOST_ChipsetResources =
{
	/* Input status register    */
	{ REGHOST_LIB_CRTC | REGHOST_TYPE_IO | REGHOST_TYPE_FIXED, 0x3da, 1, 0 },
	/* Mother board register    */
	{ REGHOST_LIB_CRTC | REGHOST_TYPE_IO | REGHOST_TYPE_FIXED, 0x094, 1, 0 },
	/* Video sub-system 1       */
	{ REGHOST_LIB_CRTC | REGHOST_TYPE_IO | REGHOST_TYPE_FIXED, 0x102, 1, 0 },
	/* Video sub-system 2       */
	{ REGHOST_LIB_CRTC | REGHOST_TYPE_IO | REGHOST_TYPE_FIXED, 0x3c3, 1, 0 },
	/* Miscellaneous register   */
	{ REGHOST_LIB_CRTC | REGHOST_TYPE_IO | REGHOST_TYPE_FIXED, 0x3c2, 1, 0 },
	/* Miscellaneous register read access  */
	{ REGHOST_LIB_CRTC | REGHOST_TYPE_IO | REGHOST_TYPE_FIXED, 0x3cc, 1, 0 },
	/* Attribut register        */
	{ REGHOST_LIB_CRTC | REGHOST_TYPE_IO | REGHOST_TYPE_FIXED, 0x3c0, 2, 0 },
	/* Palette mask register    */
	{ REGHOST_LIB_CRTC | REGHOST_TYPE_IO | REGHOST_TYPE_FIXED, 0x3c6, 1, 0 },
	/* RAMDAC index register    */
	{ REGHOST_LIB_CRTC | REGHOST_TYPE_IO | REGHOST_TYPE_FIXED, 0x3c8, 1, 0 },
	/* RAMDAC read index register */
	{ REGHOST_LIB_CRTC | REGHOST_TYPE_IO | REGHOST_TYPE_FIXED, 0x3c7, 1, 0 },
	/* RAMDAC data register     */
	{ REGHOST_LIB_CRTC | REGHOST_TYPE_IO | REGHOST_TYPE_FIXED, 0x3c9, 1, 0 },
	/* Sequencer register       */
	{ REGHOST_LIB_CRTC | REGHOST_TYPE_IO | REGHOST_TYPE_FIXED, 0x3c4, 1, 0 },
	/* Chipset register         */
	{ REGHOST_LIB_ALL  | REGHOST_TYPE_IO | REGHOST_TYPE_FIXED, 0x022, 2, 0 },
	/* Graphic controller       */
	{ REGHOST_LIB_CRTC | REGHOST_TYPE_IO | REGHOST_TYPE_FIXED, 0x3ce, 2, 0 },
	/* CRTC register            */
	{ REGHOST_LIB_ALL  | REGHOST_TYPE_IO | REGHOST_TYPE_FIXED, 0x3d4, 2, 0 },
	/* SMC super I/O GPIO       */
	{ REGHOST_LIB_ALL | REGHOST_TYPE_IO | REGHOST_TYPE_FIXED, 0x0ea, 2, 0 },
	/* SMC super I/O            */
	{ REGHOST_LIB_ALL | REGHOST_TYPE_IO | REGHOST_TYPE_FIXED, 0x370, 2, 0 },
	/* Parallel port            */
	{ REGHOST_LIB_TVENCODER | REGHOST_TYPE_IO | REGHOST_TYPE_FIXED, 0x378, 2, 0 },
	/* TFT Interface            */
	{ REGHOST_LIB_TFT  | REGHOST_TYPE_IO, 0x200, 8, 0 },
	/* PCI config space address register         */
	{ REGHOST_LIB_ALL  | REGHOST_TYPE_IO | REGHOST_TYPE_FIXED, 0xcf8, 4, 0 },
	/* PCI config space data register         */
	{ REGHOST_LIB_ALL  | REGHOST_TYPE_IO | REGHOST_TYPE_FIXED, 0xcfc, 4, 0 },
	/* Graphic Engine registers       */
	{ REGHOST_LIB_GE    | REGHOST_TYPE_GBASE, 0x8400000, 0x40000,  0 },
	/* Alpha Font Engine registers       */
	{ REGHOST_LIB_AFE   | REGHOST_TYPE_GBASE, 0x84d4000, 0x1000,  0 },
	/* Line Drawing Engine registers       */
	{ REGHOST_LIB_LDE   | REGHOST_TYPE_GBASE, 0x84d5000, 0x1000,  0 },
	/* Memory mapped TFT registers       */
	{ REGHOST_LIB_TFT   | REGHOST_TYPE_GBASE, 0x84cc000, 0x1000,  0 },
	/* Graphic Engine data port       */
	{ REGHOST_LIB_GE    | REGHOST_TYPE_GBASE, 0x8c00000, 0x400000, 0 },
	/* Video Input Port registers      */
	{ REGHOST_LIB_VIP   | REGHOST_TYPE_GBASE, 0x8600000, 0x40000,  0 },
	/* Video pipeline     */
	{ REGHOST_LIB_VPIPE | REGHOST_TYPE_GBASE, 0x8480000, 0x40000,  0 },
	/* TV output      */
	{ REGHOST_LIB_TVOUTPUT | REGHOST_TYPE_GBASE, 0x84ca000, 0x1000,   0 },
	/* On screen frame buffer access        */
	{ REGHOST_LIB_GE    | REGHOST_TYPE_GBASE, 0x8800000, 0x400000, 0 },
	/* Off screen frame buffer access        */
	{ REGHOST_LIB_GE    | REGHOST_TYPE_GBASE, 0x8000000, 0x400000, 0 },

	{ REGHOST_TYPE_END_OF_LIST, 0, 0, 0 }
};

/* ------------------------------------------------------------
// VARIABLE		: REGHOST_ucSaveSeq6
// ------------------------------------------------------------
// DESCRIPTION	: Backup value of sequencer register 6
// ------------------------------------------------------------*/
unsigned char REGHOST_ucSaveSeq6;

/* ------------------------------------------------------------
// VARIABLE		: REGHOST_ucSaveCRTC1a
// ------------------------------------------------------------
// DESCRIPTION	: Backup value of CRTC register 0x1a
// ------------------------------------------------------------*/
unsigned char REGHOST_ucSaveCRTC1a;

/* ------------------------------------------------------------
// VARIABLE		: REGHOST_ucSaveCRTC1f
// ------------------------------------------------------------
// DESCRIPTION	: Backup value of CRTC register 0x1f
// ------------------------------------------------------------*/
unsigned char REGHOST_ucSaveCRTC1f;

/* ------------------------------------------------------------
// FUNCTION		: REGHOST_Init
// ------------------------------------------------------------
// DESCRIPTION	: Initialize the access to I/O and memory
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulLib		: The registers of which module need to be
//				  initialized : a OR between REGHOST_LIB_...
//				  values
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void REGHOST_Init(unsigned long ulLib)
{
	REGHOST_BUS_RESOURCE *psBusResources;
	unsigned long ulGBASE;
	unsigned long ulFBSize;
	int i;

	DBGTRACE_BEGINFUNCTION("REGHOST_Init");
	DBGTRACE_NUMERICPARAM(ulLib);

	/* REGHOST_ChipsetResources is a table of REGHOST_BUS_RESOURCE */
	psBusResources = (REGHOST_BUS_RESOURCE *)&REGHOST_ChipsetResources;

	/* Map fixed IO and memory range */
	for(i =0; psBusResources[i].ulType != REGHOST_TYPE_END_OF_LIST; i++)
	{
		/* Map only if fixed type and if need to be mapped */
		/* for one of the selected module */
		if ((psBusResources[i].ulType & REGHOST_TYPE_FIXED) &&
			(psBusResources[i].ulType & ulLib))
		{
			/* I/O or memory ?? */
			if (psBusResources[i].ulType & REGHOST_TYPE_IO)
			{
				/* Map the physical I/O in the CPU logical address space */
				psBusResources[i].ulVirtual = OSAL_IOMap(psBusResources[i].ulAddress, psBusResources[i].ulSize);
			}
			else if (psBusResources[i].ulType & REGHOST_TYPE_MEMORY)
			{
				/* Map the physical memory in the CPU logical address space */
				psBusResources[i].ulVirtual = OSAL_MemoryMap(psBusResources[i].ulAddress, psBusResources[i].ulSize);
			}
		}
	}

	/* Unlock CRTC extended registers */
	REGHOST_ucSaveSeq6 = REGHOST_GET_SEQUENCER_REG(0x06) * 0x57;
	REGHOST_SET_SEQUENCER_REG(0x06, 0x57);

	/* Enable mapping of video registers in high memory address (bit 7 of register 0x1f) */
	REGHOST_ucSaveCRTC1f = REGHOST_GET_CRTC_REG(0x1f);
	REGHOST_SET_CRTC_REG(0x1f, (unsigned char)(REGHOST_ucSaveCRTC1f | 0x80));

	/* Enable access to the whole frame buffer */
	REGHOST_ucSaveCRTC1a = REGHOST_GET_CRTC_REG(0x1a);
	REGHOST_SET_CRTC_REG(0x1a, (unsigned char)(REGHOST_ucSaveCRTC1a & 0xfe));

	/* Get base physical address of video pipeline registers */
	ulGBASE = REGHOST_GetGBASE();

	/* Get TFT interface base address */
	REGHOST_ChipsetResources.TFTInterface.ulAddress = REGHOST_GET_16BITSCHIPSET_REG(0x1d);

	/* Get frame buffer size */
	ulFBSize = ((unsigned long)(REGHOST_GET_CHIPSET_REG(0x36)) & 0x3f) << 17;

	/* Initialize moveable resources (address of  */
	/* memory mapped registers depend on GBase value */
	for(i =0; psBusResources[i].ulType != REGHOST_TYPE_END_OF_LIST; i++)
	{
		if( psBusResources[i].ulType & REGHOST_TYPE_GBASE )
		{
			psBusResources[i].ulAddress &= 0x00ffffff; 
			psBusResources[i].ulAddress += ulGBASE; 
		}		
	}

	/* Init frame buffer size */
	REGHOST_ChipsetResources.OnScreenFrameBuffer.ulSize = ulFBSize;
	REGHOST_ChipsetResources.OffScreenFrameBuffer.ulSize = ulFBSize;

	/* Map moveable IO and memory range */
	for(i =0; psBusResources[i].ulType != REGHOST_TYPE_END_OF_LIST; i++)
	{
		/* Map only if need to be mapped */
		/* for one of the selected module */
		if ((!(psBusResources[i].ulType & REGHOST_TYPE_FIXED)) &&
			  (psBusResources[i].ulType & ulLib))
		{
			/* I/O or memory ?? */
			if (psBusResources[i].ulType & REGHOST_TYPE_IO)
			{
				/* Map the physical I/O in the CPU logical address space */ 
				psBusResources[i].ulVirtual = OSAL_IOMap(psBusResources[i].ulAddress, psBusResources[i].ulSize);
			}
			else if (psBusResources[i].ulType & REGHOST_TYPE_MEMORY)
			{
				/* Map the physical memory in the CPU logical address space */ 
				psBusResources[i].ulVirtual = OSAL_MemoryMap(psBusResources[i].ulAddress, psBusResources[i].ulSize);
			}
		}
	}

	DBGTRACE_ENDFUNCTION("REGHOST_Init");
}

/* ------------------------------------------------------------
// FUNCTION		: REGHOST_GetGBASE
// ------------------------------------------------------------
// DESCRIPTION	: Get the value of GBASE (base physical address
//				  of graphics registers)
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: GBASE
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
unsigned long REGHOST_GetGBASE(void)
{
	return ((unsigned long)(REGHOST_GET_CRTC_REG(0x20)) | 0x08) << 24;
}

/* ------------------------------------------------------------
// FUNCTION		: REGHOST_Free
// ------------------------------------------------------------
// DESCRIPTION	: Free the library
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void REGHOST_Free(void)
{
	DBGTRACE_BEGINFUNCTION("REGHOST_Free");

	/* Unlock extended CRTC register */
	REGHOST_SET_SEQUENCER_REG(0x06, 0x57);

	/* Restore CRTC register 0x1a */
	REGHOST_SET_CRTC_REG(0x1a, REGHOST_ucSaveCRTC1a);

	/* Restore CRTC register 0x1f */
	REGHOST_SET_CRTC_REG(0x1f, REGHOST_ucSaveCRTC1f);

	/* Restore Sequencer register 6 */
	REGHOST_SET_SEQUENCER_REG(0x06, REGHOST_ucSaveSeq6);

	DBGTRACE_ENDFUNCTION("REGHOST_Free");
}

/* ------------------------------------------------------------
// FUNCTION		: REGHOST_IndexedIORead8
// ------------------------------------------------------------
// DESCRIPTION	: Read a 8 bit indexed I/O register
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulVirtual	: CPU logical address of the I/O to access
// ucIndex		: Index of the register to access
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
unsigned char REGHOST_IndexedIORead8(unsigned long ulVirtual, unsigned char ucIndex)
{
	OSAL_IOWrite8(ulVirtual, 0, ucIndex);
	return OSAL_IORead8(ulVirtual, 1);
}

/* ------------------------------------------------------------
// FUNCTION		: REGHOST_IndexedIOWrite8
// ------------------------------------------------------------
// DESCRIPTION	: Write a 8 bit indexed I/O register
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulVirtual	: CPU logical address of the I/O to access
// ucIndex		: Index of the register to access
// ucData		: Value to wtite in the register
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void REGHOST_IndexedIOWrite8(unsigned long ulVirtual, unsigned char ucIndex, unsigned char ucData)
{
	OSAL_IOWrite8(ulVirtual, 0, ucIndex);
	OSAL_IOWrite8(ulVirtual, 1, ucData);
}

/* ------------------------------------------------------------
// FUNCTION		: REGHOST_IndexedIORead2x8
// ------------------------------------------------------------
// DESCRIPTION	: Read a 16 bits value by 8 bits indexed I/O 
//				  (low bits first followed by high bits)
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulVirtual	: CPU logical address of the I/O to access
// ucIndex		: Index of the register to access
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: 16 bits value
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
unsigned short REGHOST_IndexedIORead2x8(unsigned long ulVirtual, unsigned char ucIndex)
{
	unsigned short usVal;

	OSAL_IOWrite8(ulVirtual, 0, ucIndex);
	usVal = OSAL_IORead8(ulVirtual, 1);
	usVal += OSAL_IORead8(ulVirtual, 1) << 8;

	return usVal;
}

/* ------------------------------------------------------------
/* ------------------------------------------------------------
// FUNCTION		: REGHOST_IndexedIOWrite2x8
// ------------------------------------------------------------
// DESCRIPTION	: Write a 16 bits value by 8 bits indexed I/O
//				  (low bits followed by high bits)
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulVirtual	: CPU logical address of the I/O to access
// ucIndex		: Index of the register to access
// usData		: Value to wtite in the register
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void REGHOST_IndexedIOWrite2x8(unsigned long ulVirtual, unsigned char ucIndex, unsigned short usData)
{
	OSAL_IOWrite8(ulVirtual, 0, ucIndex);
	OSAL_IOWrite8(ulVirtual, 1, (unsigned char)(usData & 0xff));
	OSAL_IOWrite8(ulVirtual, 1, (unsigned char)(usData >> 8));
}

/* ------------------------------------------------------------
// FUNCTION		: REGHOST_TFTInterfaceWrite16
// ------------------------------------------------------------
// DESCRIPTION	: Write a value in a TFT interfrace register
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulVirtual	: CPU logical address of the TFT interface I/O
// usIndex		: Index of the register to access
// usData		: Value to wtite in the register
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void REGHOST_TFTInterfaceWrite16(unsigned long ulVirtual, unsigned short usIndex, unsigned short usData)
{
	OSAL_IOWrite16(ulVirtual, 0, usIndex);
	OSAL_IOWrite16(ulVirtual, 4, usData);
}


