/* ------------------------------------------------------------
// Copyright 2000 ST Microelectronics
// ------------------------------------------------------------
// This software is the property of ST Microelectronics which
// specifically grants the user the right to modify, use and
// distribute this software provided this notice is not removed
// or altered. All other rights are reserved by ST.
// ------------------------------------------------------------
// ST MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, WITH
// REGARD TO THIS SOFTWARE. IN NO EVENT SHALL ST BE LIABLE FOR
// INCIDENTAL OR CONSEQUENTIAL DAMAGES IN CONNECTION WITH OR
// ARISING FROM THE FURNISHING, PERFORMANCE, OR USE OF THIS
// SOFTWARE.
// ------------------------------------------------------------
// So that all may benefit from your experience, please report
// any problems or suggestions about this software back to ST.
// Please include your name, company, telephone number, email,
// ST product requiring support and question or problem
// encountered.
// ------------------------------------------------------------
// ST Microelectronics
// Technoparc du pays de gex
// 165 rue Edouard Branly bp112
// 06637 St Genis Pouilly cedex
// France
// ------------------------------------------------------------
// To contact us : http://www.st.com/stpc
// ------------------------------------------------------------*/

/* ------------------------------------------------------------
// Copyright (C) 2000 STMicroelectronics
// ------------------------------------------------------------
// NVG/SICL/System Solution
// ------------------------------------------------------------
// FILE NAME	: vpipe.c
// LIBRARY		: STPC Graphics Development Kit
// MODULE		: libhard\vpipe
// PORTABILITY	: ANSI C
// ------------------------------------------------------------
// REVISION		: Rev 1.2
// DATE:		: August, 25th 2000
// AUTHOR:		: Thierry Vuillaume
// ------------------------------------------------------------
// DESCRIPTION	: Functions to manage the Video Pipeline
// ------------------------------------------------------------
// VARIABLES	: HWVPIPE_ulEnable
//				  HWVPIPE_ulIsInit
//				  HWVPIPE_ulCurrentSrcBase
//				  HWVPIPE_ulCurrentSrcPitch
//				  HWVPIPE_ulFlip
//				  HWVPIPE_ulHorizontalFilter
//				  HWVPIPE_ulVerticalFilter
//				  HWVPIPE_sClip
//				  HWVPIPE_sCurrentSrc
//				  HWVPIPE_sCurrentDst
//				  HWVPIPE_ulOffsetX
//				  HWVPIPE_ulOffsetY
//				  HWVPIPE_ulScreenWidth
//				  HWVPIPE_ulScreenHeight
// ------------------------------------------------------------
// MACROS		: -
// ------------------------------------------------------------
// FUNCTIONS	: HWVPIPE_Init
//				  HWVPIPE_NotifyGraphicsModeChange
//				  HWVPIPE_RectangleSpaceConversion
//				  HWVPIPE_ClipRectangle
//				  HWVPIPE_CopyRectangle
//				  HWVPIPE_FlipRectangle
//				  HWVPIPE_SetClippedVideoPipeline
//				  HWVPIPE_InitDestination
//				  HWVPIPE_SetColorKey
//				  HWVPIPE_VerticalFlip
//				  HWVPIPE_InitChromaKey
//				  HWVPIPE_SetFilter
//				  HWVPIPE_InitSource
//				  HWVPIPE_AdjustScaleRegisters
//				  HWVPIPE_SetSource
//				  HWVPIPE_SetDestination
//				  HWVPIPE_Flip
//				  HWVPIPE_Enable
//				  HWVPIPE_Disable
//				  HWVPIPE_SetClippingArea
//				  HWVPIPE_ExtendedSetDestination
//				  HWVPIPE_ExtendedSetSource
// ------------------------------------------------------------*/

/* ------------------------------------------------------------
// REVISION		: Rev 1.2
// DATE:		: August, 25th 2000
// AUTHOR:		: Thierry Vuillaume
// MODIFICATIONS: Add margins for video color key
// ------------------------------------------------------------
// REVISION		: Rev 1.1
// DATE:		: July, 18th 2000
// AUTHOR:		: Thierry Vuillaume
// MODIFICATIONS: Add automatic clipping on the physical screen
//				  area. HWCRTC_NotifyGraphicsModeChange to
//				  be call after any graphics mode change
// ------------------------------------------------------------
// REVISION		: Rev 1.00
// DATE			: March, 1st 2000
// AUTHOR		: Thierry Vuillaume
// MODIFICATIONS: Initial source code	
// ------------------------------------------------------------*/

#include "trace.h"
#include "vpipe.h"

/* ------------------------------------------------------------
// STRUCTURE	: HWVPIPE_COLORKEY_MARGIN 
// ------------------------------------------------------------
// DESCRIPTION	: Interval for color key on video to avoid 
//				  interpolator effect which modify the color key
// ------------------------------------------------------------*/
#define HWVPIPE_COLORKEY_MARGIN		4

/* ------------------------------------------------------------
// STRUCTURE	: HWVPIPE_RECTANGLE 
// ------------------------------------------------------------
// DESCRIPTION	: Describe a rectangle coordinate
// ------------------------------------------------------------*/
typedef struct tagHWVPIPE_RECTANGLE
{
	long lLeft;
	long lRight;
	long lTop;
	long lBottom;
}HWVPIPE_RECTANGLE;

/* ------------------------------------------------------------
// VARIABLE		: HWVPIPE_ulEnable 
// ------------------------------------------------------------
// DESCRIPTION	: 0 if video pipeline is currently disabled,
//				  <> 0 if the video pipeline is enabled
// ------------------------------------------------------------*/
unsigned long HWVPIPE_ulEnable = 0;

/* ------------------------------------------------------------
// VARIABLE		: HWVPIPE_ulIsInit 
// ------------------------------------------------------------
// DESCRIPTION	: Flag to determine if the source, destination
//				  and clipping area have been initialized. If
//				  on of this rectangle is missing, the 
//				  clipped Video Pipeline can't be displayed
// ------------------------------------------------------------*/
unsigned long HWVPIPE_ulIsInit = 0;

/* ------------------------------------------------------------
// VARIABLE		: HWVPIPE_ulCurrentSrcBase 
// ------------------------------------------------------------
// DESCRIPTION	: Current value of video pipeline base offset
//				  in the frame buffer
// ------------------------------------------------------------*/
unsigned long HWVPIPE_ulCurrentSrcBase;

/* ------------------------------------------------------------
// VARIABLE		: HWVPIPE_ulCurrentSrcPitch 
// ------------------------------------------------------------
// DESCRIPTION	: Current value of video pipeline pitch
//				  in the frame buffer. The pitch is the number
//				  of bytes between two lines of the input data
// ------------------------------------------------------------*/
unsigned long HWVPIPE_ulCurrentSrcPitch;

/* ------------------------------------------------------------
// VARIABLE		: HWVPIPE_ulFlip 
// ------------------------------------------------------------
// DESCRIPTION	: <> 0 if the Video Pipeline is verticaly flipped
//				  = 0 if the Video Pipeline is displayed without 
//				  flip
// ------------------------------------------------------------*/
unsigned long HWVPIPE_ulFlip = 0;

/* ------------------------------------------------------------
// VARIABLE		: HWVPIPE_ulHorizontalFilter 
// ------------------------------------------------------------
// DESCRIPTION	: Actual value of the horizontal interpolator
//				  configuration bits
// ------------------------------------------------------------*/
unsigned long HWVPIPE_ulHorizontalFilter = 0x180000L;

/* ------------------------------------------------------------
// VARIABLE		: HWVPIPE_ulVerticalFilter 
// ------------------------------------------------------------
// DESCRIPTION	: Actual value of the vertical interpolator
//				  configuration bits
// ------------------------------------------------------------*/
unsigned long HWVPIPE_ulVerticalFilter = 0x180000L;

/* ------------------------------------------------------------
// VARIABLE		: HWVPIPE_sClip 
// ------------------------------------------------------------
// DESCRIPTION	: Screen clipping area. Video Pipeline is
//				  displayed only in this area of the screen
// ------------------------------------------------------------*/
HWVPIPE_RECTANGLE HWVPIPE_sClip;

/* ------------------------------------------------------------
// VARIABLE		: HWVPIPE_sCurrentSrc 
// ------------------------------------------------------------
// DESCRIPTION	: Current coordinate of the portion of the
//				  source image to display
// ------------------------------------------------------------*/
HWVPIPE_RECTANGLE HWVPIPE_sCurrentSrc;

/* ------------------------------------------------------------
// VARIABLE		: HWVPIPE_sCurrentDst 
// ------------------------------------------------------------
// DESCRIPTION	: Current coordinate which describe
//				  the location of Video Pipeline on screen
// ------------------------------------------------------------*/
HWVPIPE_RECTANGLE HWVPIPE_sCurrentDst;

/* ------------------------------------------------------------
// VARIABLE		: HWVPIPE_ulOffsetX
// ------------------------------------------------------------
// DESCRIPTION	: Offset between mouse coordinate and location
//				  on screen
// ------------------------------------------------------------*/
unsigned long HWVPIPE_ulOffsetX;

/* ------------------------------------------------------------
// VARIABLE		: HWVPIPE_ulOffsetY
// ------------------------------------------------------------
// DESCRIPTION	: Offset between mouse coordinate and location
//				  on screen
// ------------------------------------------------------------*/
unsigned long HWVPIPE_ulOffsetY;

/* ------------------------------------------------------------
// VARIABLE		: HWVPIPE_ulScreenWidth
// ------------------------------------------------------------
// DESCRIPTION	: Current screen width
// ------------------------------------------------------------*/
unsigned long HWVPIPE_ulScreenWidth;

/* ------------------------------------------------------------
// VARIABLE		: HWVPIPE_ulScreenHeight
// ------------------------------------------------------------
// DESCRIPTION	: Current screen height
// ------------------------------------------------------------*/
unsigned long HWVPIPE_ulScreenHeight;

/* ------------------------------------------------------------
// FUNCTION		: HWVPIPE_Init
// ------------------------------------------------------------
// DESCRIPTION	: Init the videop pipeline cursor library
//				  This functions is necessary only when using
//				  HWVPIPE_Extended... function
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: HWVPIPE_ulOffsetY
//				  HWVPIPE_ulScreenWidth
// ------------------------------------------------------------*/
void HWVPIPE_Init(void)
{
	DBGTRACE_BEGINFUNCTION("HWVPIPE_Init");

	/* Initialize clipping area */
	HWVPIPE_sClip.lLeft = 0;
	HWVPIPE_sClip.lTop = 0;
	HWVPIPE_sClip.lRight = 0xfffffff;
	HWVPIPE_sClip.lBottom = 0xfffffff;

	/* Update information about the graphics mode */
	HWVPIPE_NotifyGraphicsModeChange();

	DBGTRACE_ENDFUNCTION("HWVPIPE_Init");
}

/* ------------------------------------------------------------
// FUNCTIONS	: HWVPIPE_NotifyGraphicsModeChange
// ------------------------------------------------------------
// DESCRIPTION	: To be send after a graphics mode change
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWVPIPE_NotifyGraphicsModeChange(void)
{
	unsigned long ulBlankStart;

	DBGTRACE_BEGINFUNCTION("HWVPIPE_NotifyGraphicsModeChange");

	HWVPIPE_ulOffsetX = 0;
	HWVPIPE_ulOffsetY = 0;

	/* get screen width */
	/* Width in characters */
	HWVPIPE_ulScreenWidth = REGHOST_GET_CRTC_REG(0x1) + 1;

	/* Multiply by the size of a characters */
	if(REGHOST_GET_SEQUENCER_REG(0x1) & 0x1)
	{
		DBGTRACE_BRANCH("Character size is 8");
		HWVPIPE_ulScreenWidth *= 8;
	}
	else
	{
		DBGTRACE_BRANCH("Character size is 9");
		HWVPIPE_ulScreenWidth *= 9;
	}

	/* End of active video from standard VGA registers */
	HWVPIPE_ulScreenHeight = REGHOST_GET_CRTC_REG(0x12) + 1;
	HWVPIPE_ulScreenHeight += (REGHOST_GET_CRTC_REG(0x7) & 0x2) << 7;
	HWVPIPE_ulScreenHeight += (REGHOST_GET_CRTC_REG(0x7) & 0x40) << 3;

	/* Start of blanking signal from standard VGA registers */
	ulBlankStart = REGHOST_GET_CRTC_REG(0x15) + 1;
	ulBlankStart += (REGHOST_GET_CRTC_REG(0x7) & 0x8) << 5;
	ulBlankStart += (REGHOST_GET_CRTC_REG(0x9) & 0x20) << 4;

	/* If extended registers are used add the last bits */
	if(REGHOST_GET_SEQUENCER_REG(0x6))
	{
		DBGTRACE_BRANCH("Extended CRTC registers are enable");
		HWVPIPE_ulScreenHeight += (REGHOST_GET_CRTC_REG(0x25) & 0x2) << 9;
		ulBlankStart += (REGHOST_GET_CRTC_REG(0x25) & 0x8) << 7;
	}

	DBGTRACE_NUMERICPARAM(HWVPIPE_ulScreenHeight);
	DBGTRACE_NUMERICPARAM(ulBlankStart);

	/* If blanking start before end of active video */
	/* the real displayed screen size is the blanking start */
	if(ulBlankStart < HWVPIPE_ulScreenHeight)
	{
		DBGTRACE_BRANCH("Blank start before end of active video");

		HWVPIPE_ulScreenHeight = ulBlankStart;

		/* NTSC mode <=> offset of 16 pixels */
		HWVPIPE_ulOffsetY = 16;
	}

	DBGTRACE_ENDFUNCTION("HWVPIPE_NotifyGraphicsModeChange");
}

/* ------------------------------------------------------------
// FUNCTION		: HWVPIPE_RectangleSpaceConversion
// ------------------------------------------------------------
// DESCRIPTION	: Translate the coordinate of a rectangle
//				  inside a source rectangular area into
//				  a destination rectangle area
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// sSrcSpace	: Source rectangle area 
// sDstSpace	: Destination rectangle area
// sRect		: Rectangle to translate 
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: 0 if the source rectangle is 0 pixels width
//				  or height (translation impossible)
//				  -1 if the translation is possible
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
int HWVPIPE_RectangleSpaceConversion(HWVPIPE_RECTANGLE *sSrcSpace, HWVPIPE_RECTANGLE *sDstSpace, HWVPIPE_RECTANGLE *sRect)
{
	double fScaleX;
	double fScaleY;

	/* Source rectangle is 0 pixels width : translation avoid */
	if(sSrcSpace->lRight - sSrcSpace->lLeft + 1 == 0)
		return 0;

	/* Source rectangle is 0 pixels height : translation avoid */
	if(sSrcSpace->lBottom - sSrcSpace->lTop + 1 == 0)
		return 0;

	/* Compute the horizontal and vertical scaling factors */
	fScaleX = (double)(sDstSpace->lRight - sDstSpace->lLeft + 1) / (double)(sSrcSpace->lRight - sSrcSpace->lLeft + 1);
	fScaleY = (double)(sDstSpace->lBottom - sDstSpace->lTop + 1) / (double)(sSrcSpace->lBottom - sSrcSpace->lTop + 1);

	/* Do the translation */
	sRect->lTop = (long)((double)(sRect->lTop - sSrcSpace->lTop) * fScaleX + sDstSpace->lTop);
	sRect->lBottom = (long)((double)(sRect->lBottom - sSrcSpace->lTop) * fScaleY + sDstSpace->lTop);
	sRect->lLeft = (long)((double)(sRect->lLeft - sSrcSpace->lLeft) * fScaleX + sDstSpace->lLeft);
	sRect->lRight = (long)((double)(sRect->lRight - sSrcSpace->lLeft) * fScaleY + sDstSpace->lLeft);

	return -1;
}

/* ------------------------------------------------------------
// FUNCTION		: HWVPIPE_ClipRectangle
// ------------------------------------------------------------
// DESCRIPTION	: Clip a rectangle inside another rectangle
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// sRect		: Rectangle to clip 
// sClipRect	: Clipping rectangle
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: 0 if the source rectangle is outside the
//				  clipping area, -1 either
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
int HWVPIPE_ClipRectangle(HWVPIPE_RECTANGLE *sRect, HWVPIPE_RECTANGLE *HWVPIPE_sClipRect)
{
	/* Rectangle outside the clipping area? */
	if(sRect->lTop > HWVPIPE_sClipRect->lBottom)
	{
		return 0;
	}

	if(sRect->lLeft > HWVPIPE_sClipRect->lRight)
	{
		return 0;
	}

	if(sRect->lBottom < HWVPIPE_sClipRect->lTop)
	{
		return 0;
	}

	if(sRect->lRight < HWVPIPE_sClipRect->lLeft)
	{
		return 0;
	}

	/* Do the clipping */
	if(sRect->lLeft < HWVPIPE_sClipRect->lLeft)
	{
		sRect->lLeft = HWVPIPE_sClipRect->lLeft;
	}

	if(sRect->lTop < HWVPIPE_sClipRect->lTop)
	{
		sRect->lTop = HWVPIPE_sClipRect->lTop;
	}

	if(sRect->lRight > HWVPIPE_sClipRect->lRight)
	{
		sRect->lRight = HWVPIPE_sClipRect->lRight;
	}

	if(sRect->lBottom > HWVPIPE_sClipRect->lBottom)
	{
		sRect->lBottom = HWVPIPE_sClipRect->lBottom;
	}

	return -1;
}

/* ------------------------------------------------------------
// FUNCTION		: HWVPIPE_CopyRectangle
// ------------------------------------------------------------
// DESCRIPTION	: Copy a rectangle
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// sDstRect		: Source rectangle 
// sSrcRect		: Destination rectangle
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWVPIPE_CopyRectangle(HWVPIPE_RECTANGLE *sDstRect, HWVPIPE_RECTANGLE *sSrcRect)
{
	sDstRect->lLeft = sSrcRect->lLeft;
	sDstRect->lTop = sSrcRect->lTop;
	sDstRect->lRight = sSrcRect->lRight;
	sDstRect->lBottom = sSrcRect->lBottom;
}

/* ------------------------------------------------------------
// FUNCTION		: HWVPIPE_FlipRectangle
// ------------------------------------------------------------
// DESCRIPTION	: Do a vertical flip of a rectangle coordinate
//				  inside the Video Pipeline source image
//				  rectangle
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// sRect		: Rectangle to flip
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWVPIPE_FlipRectangle(HWVPIPE_RECTANGLE *sRect)
{
	long ulTemp;

	ulTemp = sRect->lTop;
	sRect->lTop = HWVPIPE_sCurrentSrc.lTop + HWVPIPE_sCurrentSrc.lBottom - sRect->lBottom;
	sRect->lBottom = HWVPIPE_sCurrentSrc.lBottom - (ulTemp - HWVPIPE_sCurrentSrc.lTop);
}

/* ------------------------------------------------------------
// FUNCTION		: HWVPIPE_SetClippedVideoPipeline
// ------------------------------------------------------------
// DESCRIPTION	: Display the Video Pipeline taking care of
//				  the clipping (use the global variable
//				  to determine where to display)
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWVPIPE_SetClippedVideoPipeline(void)
{
	HWVPIPE_RECTANGLE sTmpSrc;
	HWVPIPE_RECTANGLE sTmpDst;
	HWVPIPE_RECTANGLE sTmpClip;

	DBGTRACE_BEGINFUNCTION("HWVPIPE_SetClippedVideoPipeline");

	if(HWVPIPE_ulIsInit != 0x3)
	{
		DBGTRACE_BRANCH("Parameters not yet initialized");
		return;
	}

	HWVPIPE_CopyRectangle(&sTmpDst, &HWVPIPE_sCurrentDst);
	HWVPIPE_CopyRectangle(&sTmpClip, &HWVPIPE_sClip);

	/* Clip with screen if necessary */
	if(HWVPIPE_sClip.lRight >= (long)HWVPIPE_ulScreenWidth)
	{
		sTmpClip.lRight = HWVPIPE_ulScreenWidth - 1;
	}
	if(HWVPIPE_sClip.lBottom >= (long)HWVPIPE_ulScreenHeight)
	{
		sTmpClip.lBottom = HWVPIPE_ulScreenHeight - 1;
	}

	/* Do the clipping */
	if(!HWVPIPE_ClipRectangle(&sTmpDst, &sTmpClip))
	{
		DBGTRACE_BRANCH("Rectangle outside the clipping");

		/* Hide the video pipeline */
		REGHOST_SET_VPIPE_VIDEOENABLE_REG(0x0L);
	}
	else
	{
		DBGTRACE_BRANCH("Rectangle inside the clipping");

		HWVPIPE_CopyRectangle(&sTmpSrc, &HWVPIPE_sCurrentSrc);

		/* Translate the source rectangle to the screen */
		HWVPIPE_RectangleSpaceConversion(&HWVPIPE_sCurrentDst, &HWVPIPE_sCurrentSrc, &sTmpClip);

		/* Clip the screen rectangle */
		HWVPIPE_ClipRectangle(&sTmpSrc, &sTmpClip);

		/* Flip if necessary */
		if(HWVPIPE_ulFlip)
		{
			DBGTRACE_BRANCH("Rectangle flipped");

			HWVPIPE_FlipRectangle(&sTmpSrc);
		}

		/* Display the Video Pipeline */
		HWVPIPE_SetSource(sTmpSrc.lLeft, sTmpSrc.lTop, (sTmpSrc.lRight - sTmpSrc.lLeft), (sTmpSrc.lBottom - sTmpSrc.lTop));
		HWVPIPE_SetDestination(sTmpDst.lLeft + HWVPIPE_ulOffsetX, sTmpDst.lTop + HWVPIPE_ulOffsetY, (sTmpDst.lRight - sTmpDst.lLeft + 1), (sTmpDst.lBottom - sTmpDst.lTop + 1));
		REGHOST_SET_VPIPE_VIDEOENABLE_REG(HWVPIPE_ulEnable);
	}

	DBGTRACE_ENDFUNCTION("HWVPIPE_SetClippedVideoPipeline");
}

/* ------------------------------------------------------------
// FUNCTION		: HWVPIPE_InitDestination
// ------------------------------------------------------------
// DESCRIPTION	: Init the way Video Pipeline is displayed
//				  on screen
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulColorKey	: Video is displayed only in screen area filled
//				  with this color, in the pixel format
//				  of the screen (8 bit used only in 256 colors...)
//				  HWVPIPE_COLORKEY_DISABLE <=> video always displayed
//				  HWVPIPE_COLORKEY_CHROMA  <=> use chroma key,
//				  have to call HWVPIPE_InitChromaKey function
//				  If the color key value is mixed with the
//				  HWVPIPE_COLORKEY_VIDEO flag by an OR operation
//				  a specific color of the Video Pipeline
//				  image is replaced by the graphic (color key
//				  on video in place of graphic)
// ulSourceFormat: HWVPIPE_FORMAT_RGB555, HWVPIPE_FORMAT_RGB565 or 
//				   HWVPIPE_FORMAT_YUV422
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWVPIPE_InitDestination(unsigned long ulColorKey, unsigned long ulSourceFormat)
{
	unsigned long ulTmpColorLow;
	unsigned long ulTmpColorHigh;
	unsigned long ulTmpColor;

	DBGTRACE_BEGINFUNCTION("HWVPIPE_InitDestination");
	DBGTRACE_NUMERICPARAM(ulColorKey);
	DBGTRACE_NUMERICPARAM(ulSourceFormat);

	/* Set the Video Pipeline FIFO burst config */
	REGHOST_SET_VPIPE_CRTCBURSTLENGHT_REG(0x00100010L);
	REGHOST_SET_VPIPE_VIDEOBURSTLENGHT_REG(0x00900060L);

	/* Initialize the pixel format */
	switch(ulSourceFormat)
	{
		case HWVPIPE_FORMAT_YUV422:
			DBGTRACE_BRANCH("YUV422");

			/* Set YUV422 */
			REGHOST_SET_VPIPE_COLORSPACECONVERTER_REG(0x01);

			/* Enable YUV->RGB color space converter */
			REGHOST_SET_VPIPE_PITCH_REG((REGHOST_GET_VPIPE_PITCH_REG() | 0x2000) & ~0x1000);
			break;

		case HWVPIPE_FORMAT_RGB555:
			DBGTRACE_BRANCH("RGB555");

			/* Set RGB555 */
			REGHOST_SET_VPIPE_COLORSPACECONVERTER_REG(0x0);

			/* Disable YUV->RGB color space converter */
			REGHOST_SET_VPIPE_PITCH_REG(REGHOST_GET_VPIPE_PITCH_REG() & ~0x3000);
			break;

		case HWVPIPE_FORMAT_RGB565:
			DBGTRACE_BRANCH("RGB565");

			/* Set RGB565 */
			REGHOST_SET_VPIPE_COLORSPACECONVERTER_REG(0x0);

			/* Disable YUV->RGB color space converter */
			REGHOST_SET_VPIPE_PITCH_REG((REGHOST_GET_VPIPE_PITCH_REG() | 0x1000) & ~0x2000);
			break;
	}

	/* Set the mix mode */
	if (ulColorKey == HWVPIPE_COLORKEY_DISABLE)
	{
		DBGTRACE_BRANCH("No color key");

		REGHOST_SET_VPIPE_MIXMODE_REG(0x0);
	}
	else if(ulColorKey & HWVPIPE_COLORKEY_VIDEO)
	{
		DBGTRACE_BRANCH("Video color key");

		REGHOST_SET_VPIPE_MIXMODE_REG(0x2);

		/* Add a margin as interpolator can modify the color key */
		ulTmpColor = (ulColorKey & 0xff) - HWVPIPE_COLORKEY_MARGIN;
		if(ulTmpColor > 0xff)
			ulTmpColor = 0x0;
		ulTmpColorLow = ulTmpColor;
		ulTmpColor = (ulColorKey & 0xff) + HWVPIPE_COLORKEY_MARGIN;
		if(ulTmpColor > 0xff)
			ulTmpColor = 0xff;
		ulTmpColorHigh = ulTmpColor;

		ulTmpColor = ((ulColorKey >> 8) & 0xff) - HWVPIPE_COLORKEY_MARGIN;
		if(ulTmpColor > 0xff)
			ulTmpColor = 0x0;
		ulTmpColorLow += ulTmpColor << 8;
		ulTmpColor = ((ulColorKey >> 8) & 0xff) + HWVPIPE_COLORKEY_MARGIN;
		if(ulTmpColor > 0xff)
			ulTmpColor = 0xff;
		ulTmpColorHigh += ulTmpColor << 8;

		ulTmpColor = ((ulColorKey >> 16) & 0xff) - HWVPIPE_COLORKEY_MARGIN;
		if(ulTmpColor > 0xff)
			ulTmpColor = 0x0;
		ulTmpColorLow += ulTmpColor << 16;
		ulTmpColor = ((ulColorKey >> 16) & 0xff) + HWVPIPE_COLORKEY_MARGIN;
		if(ulTmpColor > 0xff)
			ulTmpColor = 0xff;
		ulTmpColorHigh += ulTmpColor << 16;

		REGHOST_SET_VPIPE_CHROMALOW_REG(ulTmpColorLow);
		REGHOST_SET_VPIPE_CHROMAHIGH_REG(ulTmpColorHigh);
	}
	else if (!(ulColorKey & HWVPIPE_COLORKEY_CHROMA))
	{
		DBGTRACE_BRANCH("Graphic color key");
		HWVPIPE_SetColorKey(ulColorKey);
		REGHOST_SET_VPIPE_MIXMODE_REG(0x1);
	}

	DBGTRACE_ENDFUNCTION("HWVPIPE_InitDestination");
}

/* ------------------------------------------------------------
// FUNCTION		: HWVPIPE_SetColorKey
// ------------------------------------------------------------
// DESCRIPTION	: Set the color key value
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulColorKey	: Value of color key in the screen format
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWVPIPE_SetColorKey(unsigned long ulColorKey)
{
	unsigned long pulBppList[] = { 1, 1, 2, 2, 3, 4, 4, 4 };
	unsigned long ulColorKeyReg;
	unsigned long ulBpp;

	DBGTRACE_BEGINFUNCTION("HWVPIPE_SetColorKey");

	/* Pixel format of the current screen */
	ulBpp = pulBppList[(REGHOST_GET_CRTC_REG(0x28) & 0x7)];

	/* Set appropriate color key */
	switch(ulBpp)
	{
		case 1:
			ulColorKeyReg = ulColorKey & 0xff;
			ulColorKeyReg += ulColorKeyReg << 8;
			ulColorKeyReg += ulColorKeyReg << 16;
			break;

		case 2:
			ulColorKeyReg = ulColorKey & 0xffff;
			ulColorKeyReg += ulColorKeyReg << 16;
			break;

		case 3:
			ulColorKeyReg = ulColorKey & 0xffffff;
			break;

		case 4:
			ulColorKeyReg = ulColorKey;
			break;
	}

	REGHOST_SET_VPIPE_COLORKEY_REG(ulColorKeyReg);

	DBGTRACE_ENDFUNCTION("HWVPIPE_SetColorKey");
}

/* ------------------------------------------------------------
// FUNCTION		: HWVPIPE_VerticalFlip
// ------------------------------------------------------------
// DESCRIPTION	: Flip verticaly the Video Pipeline image
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWVPIPE_VerticalFlip(void)
{
	unsigned long ulBase;
	unsigned long ulHeight;

	DBGTRACE_BEGINFUNCTION("HWVPIPE_VerticalFlip");

	/* Is the video pipeline initialized to work with clipping? */
	if(HWVPIPE_ulIsInit == 0x3)
	{
		DBGTRACE_BRANCH("Clipped Video Pipeline");

		HWVPIPE_ulFlip ^= 1;
		HWVPIPE_SetClippedVideoPipeline();
	}
	else
	{
		DBGTRACE_BRANCH("Not clipped Video Pipeline");

		/* Get current base and height */
		ulBase = REGHOST_GET_VPIPE_SRCBASE_REG();
		ulHeight = (REGHOST_GET_VPIPE_SRCDIM_REG() >> 16) & 0x7ff;

		/* Is already flipped */
		if(HWVPIPE_ulFlip)
		{
			DBGTRACE_BRANCH("No flip");

			/* Suppress the flip */
			HWVPIPE_ulFlip = 0;
			REGHOST_SET_VPIPE_PITCH_REG(REGHOST_GET_VPIPE_PITCH_REG() & ~0x800);
			REGHOST_SET_VPIPE_SRCBASE_REG(ulBase - ((ulHeight - 1) * HWVPIPE_ulCurrentSrcPitch));
		}
		else
		{
			DBGTRACE_BRANCH("Flip");

			/* Flip */
			HWVPIPE_ulFlip = 1;
			REGHOST_SET_VPIPE_PITCH_REG(REGHOST_GET_VPIPE_PITCH_REG() | 0x800);
			REGHOST_SET_VPIPE_SRCBASE_REG(ulBase + ((ulHeight - 1) * HWVPIPE_ulCurrentSrcPitch));
		}
	}

	DBGTRACE_ENDFUNCTION("HWVPIPE_VerticalFlip");
}

/* ------------------------------------------------------------
// FUNCTION		: HWVPIPE_InitChromaKey
// ------------------------------------------------------------
// DESCRIPTION	: Use chroma key mix mode (replace a range of
//				  video color by the graphic pixels)
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulCSC		: TRUE if compare after the color space converter
//				  <=> in RGB in case of YUV input format
// ulLow		: Lowest value of the range
// ulHigh		: Highest value of the range
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWVPIPE_InitChromaKey(unsigned long ulCSC, unsigned long ulLow, unsigned long ulHigh)
{
	unsigned long ulConfig;

	DBGTRACE_BEGINFUNCTION("HWVPIPE_InitChromaKey");
	DBGTRACE_NUMERICPARAM(ulCSC);
	DBGTRACE_NUMERICPARAM(ulLow);
	DBGTRACE_NUMERICPARAM(ulHigh);

	/* Set chroma key mode */
	REGHOST_SET_VPIPE_MIXMODE_REG(0x2);

	/* Before or after the color space converter? */
	if(ulCSC)
	{
		ulConfig = 0x8000000L;
	}
	else
	{
		ulConfig = 0x0000000L;
	}

	/* Set the low and high value of the range */
	REGHOST_SET_VPIPE_CHROMALOW_REG(ulConfig + ulLow & 0xffffff);
	REGHOST_SET_VPIPE_CHROMAHIGH_REG(ulHigh);

	DBGTRACE_ENDFUNCTION("HWVPIPE_InitChromaKey");
}

/* ------------------------------------------------------------
// FUNCTION		: HWVPIPE_SetFilter
// ------------------------------------------------------------
// DESCRIPTION	: Enable or disable scaler interpolators
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// In YUV input format :
// ulHorizontalY	: TRUE to enable horizontal luminance filter
// ulHorizontalUV	: TRUE to enable horizontal chrominance filter
// ulVerticalY		: TRUE to enable vertical luminance filter
// ulVerticalUV		: TRUE to enable vertical chrominance filter
//
// In RGB input format :
// ulHorizontalY and bHorizontalUV : TRUE to enable horizontal filter
// ulVerticalY and bVerticalUV     : TRUE to enable vertical filter
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWVPIPE_SetFilter(unsigned long ulHorizontalY, unsigned long ulHorizontalUV, unsigned long ulVerticalY, unsigned long ulVerticalUV)
{
	DBGTRACE_BEGINFUNCTION("HWVPIPE_SetFilter");
	DBGTRACE_NUMERICPARAM(ulHorizontalY);
	DBGTRACE_NUMERICPARAM(ulHorizontalUV);
	DBGTRACE_NUMERICPARAM(ulVerticalY);
	DBGTRACE_NUMERICPARAM(ulVerticalUV);

	/* Compute the registers value */
	HWVPIPE_ulHorizontalFilter = 0;	
	HWVPIPE_ulVerticalFilter = 0;	

	if(ulHorizontalY)
	{
		HWVPIPE_ulHorizontalFilter |= 0x80000L;
	}

	if(ulHorizontalUV)
	{
		HWVPIPE_ulHorizontalFilter |= 0x100000L;
	}

	if(ulVerticalY)
	{
		HWVPIPE_ulVerticalFilter |= 0x80000L;
	}

	if(ulVerticalUV)
	{
		HWVPIPE_ulVerticalFilter |= 0x100000L;
	}

	/* Set registers */
	REGHOST_SET_VPIPE_HORIZONTALSCALE_REG((REGHOST_GET_VPIPE_HORIZONTALSCALE_REG() & ~0x180000L) | HWVPIPE_ulHorizontalFilter);
	REGHOST_SET_VPIPE_VERTICALSCALE_REG((REGHOST_GET_VPIPE_VERTICALSCALE_REG() & ~0x180000L) | HWVPIPE_ulVerticalFilter);

	DBGTRACE_ENDFUNCTION("HWVPIPE_SetFilter");
}

/* ------------------------------------------------------------
// FUNCTION		: HWVPIPE_InitSource
// ------------------------------------------------------------
// DESCRIPTION	: Initialize the source image
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulSrcBase	: Offset of the source location in frame buffer
// ulSrcPitch	: Number of bytes between to lines of source data
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWVPIPE_InitSource(unsigned long ulSrcBase, unsigned long ulSrcPitch)
{
	DBGTRACE_BEGINFUNCTION("HWVPIPE_SetFilter");
	DBGTRACE_NUMERICPARAM(ulSrcBase);
	DBGTRACE_NUMERICPARAM(ulSrcPitch);

	HWVPIPE_ulCurrentSrcBase = ulSrcBase;
	HWVPIPE_ulCurrentSrcPitch = ulSrcPitch;

	REGHOST_SET_VPIPE_SRCBASE_REG(ulSrcBase);
	REGHOST_SET_VPIPE_PITCH_REG((REGHOST_GET_VPIPE_PITCH_REG() & ~0x7ff) + ulSrcPitch);

	DBGTRACE_ENDFUNCTION("HWVPIPE_SetFilter");
}

/* ------------------------------------------------------------
// FUNCTION		: HWVPIPE_AdjustScaleRegisters
// ------------------------------------------------------------
// DESCRIPTION	: Update the contend of the scales register
//				  after a modification of source or destination
//				  size
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWVPIPE_AdjustScaleRegisters(void)
{
	unsigned long ulSrcReg;
	unsigned long ulDstReg;
	unsigned long ulDstWidth;
	unsigned long ulDstHeight;
	unsigned long ulSrcWidth;
	unsigned long ulSrcHeight;
	unsigned long ulFactor;

	DBGTRACE_BEGINFUNCTION("HWVPIPE_AdjustScaleRegisters");

	/* Get source size */
	ulSrcReg = REGHOST_GET_VPIPE_SRCDIM_REG();
	ulSrcWidth = ulSrcReg & 0x7ff;
	ulSrcHeight = (ulSrcReg >> 16) & 0x7ff;

	/* Get destination size */
	ulDstReg = REGHOST_GET_VPIPE_DSTDIM_REG();
	ulDstWidth = ulDstReg & 0x7ff;
	ulDstHeight = (ulDstReg >> 16) & 0x7ff;

	/* Check if no divide by 0 and if no downscaling */
	if((ulDstWidth) && (ulSrcWidth <= ulDstWidth))
	{
		ulFactor = (ulSrcWidth * 4096) / ulDstWidth;
	}
	else
	{
		ulFactor = 4096;
	}

	/* Update horizontal scale register */
	REGHOST_SET_VPIPE_HORIZONTALSCALE_REG(HWVPIPE_ulHorizontalFilter + ulFactor);

	/* Check if no divide by 0 and if no downscaling */
	if((ulDstHeight) && (ulSrcHeight <= ulDstHeight))
	{
		ulFactor = (ulSrcHeight * 4096) / ulDstHeight;
	}
	else
	{
		ulFactor = 4096;
	}

	/* Update vertical scale register */
	REGHOST_SET_VPIPE_VERTICALSCALE_REG(HWVPIPE_ulVerticalFilter + ulFactor);

	DBGTRACE_ENDFUNCTION("HWVPIPE_AdjustScaleRegisters");
}

/* ------------------------------------------------------------
// FUNCTION		: HWVPIPE_SetSource
// ------------------------------------------------------------
// DESCRIPTION	: Initialize the source rectangle
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulSrcX		: X coordinate of the portion of the source
//				  image to display
// ulSrcY		: Y coordinate of the portion of the source
//				  image to display
// ulSrcWidth	: Width of the portion of the source
//				  image to display
// ulSrcHeight	: Height of the portion of the source
//				  image to display
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWVPIPE_SetSource(unsigned long ulSrcX, unsigned long ulSrcY, unsigned long ulSrcWidth, unsigned long ulSrcHeight)
{
	DBGTRACE_BEGINFUNCTION("HWVPIPE_SetSource");
	DBGTRACE_NUMERICPARAM(ulSrcX);
	DBGTRACE_NUMERICPARAM(ulSrcY);
	DBGTRACE_NUMERICPARAM(ulSrcWidth);
	DBGTRACE_NUMERICPARAM(ulSrcHeight);

	/* Go to bottom if flipped */
	if(HWVPIPE_ulFlip)
	{
		ulSrcY += ulSrcHeight - 1;
	}

	/* Set new source */
	REGHOST_SET_VPIPE_SRCBASE_REG(HWVPIPE_ulCurrentSrcBase + 2 * ulSrcX + HWVPIPE_ulCurrentSrcPitch * ulSrcY);
	REGHOST_SET_VPIPE_SRCDIM_REG((ulSrcHeight << 16) + ulSrcWidth);	                    /* Source dimension */

	/* Update scaling registers */
	HWVPIPE_AdjustScaleRegisters();

	DBGTRACE_BEGINFUNCTION("HWVPIPE_SetSource");
}

/* ------------------------------------------------------------
// FUNCTION		: HWVPIPE_SetDestination
// ------------------------------------------------------------
// DESCRIPTION	: Initialize the destination rectangle
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulDstX		: X coordinate of the location of the video
//				  pipeline on screen
// ulDstY		: Y coordinate of the location of the video
//				  pipeline on screen
// ulDstWidth	: Width of the video pipeline on screen
// ulDstHeight	: Height of the video pipeline on screen
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWVPIPE_SetDestination(unsigned long ulDstX, unsigned long ulDstY, unsigned long ulDstWidth, unsigned long ulDstHeight)
{
	DBGTRACE_BEGINFUNCTION("HWVPIPE_SetDestination");
	DBGTRACE_NUMERICPARAM(ulDstX);
	DBGTRACE_NUMERICPARAM(ulDstY);
	DBGTRACE_NUMERICPARAM(ulDstWidth);
	DBGTRACE_NUMERICPARAM(ulDstHeight);

	/* Video pipeline doesn't support more than 1023 lines */
	if(ulDstHeight > 1023)
		ulDstHeight = 1023;

	/* Set the new destination rectangle on screen */
	REGHOST_SET_VPIPE_DSTXY_REG((ulDstY << 16) + ulDstX);
	REGHOST_SET_VPIPE_DSTDIM_REG((ulDstHeight << 16) + ulDstWidth);

	/* Update scaling registers  */
	HWVPIPE_AdjustScaleRegisters();

	DBGTRACE_ENDFUNCTION("HWVPIPE_SetDestination");
}

/* ------------------------------------------------------------
// FUNCTION		: HWVPIPE_Flip
// ------------------------------------------------------------
// DESCRIPTION	: Change the source data of video pipeline
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulBase		: New base address
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWVPIPE_Flip(unsigned long ulBase)
{
	DBGTRACE_BEGINFUNCTION("HWVPIPE_Flip");
	DBGTRACE_NUMERICPARAM(ulBase);

	REGHOST_SET_VPIPE_SRCBASE_REG(REGHOST_GET_VPIPE_SRCBASE_REG() - HWVPIPE_ulCurrentSrcBase + ulBase);
	HWVPIPE_ulCurrentSrcBase = ulBase;

	DBGTRACE_ENDFUNCTION("HWVPIPE_Flip");
}

/* ------------------------------------------------------------
// FUNCTION		: HWVPIPE_Enable
// ------------------------------------------------------------
// DESCRIPTION	: Enable the Video Pipeline
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWVPIPE_Enable(void)
{
	DBGTRACE_BEGINFUNCTION("HWVPIPE_Enable");

	REGHOST_SET_VPIPE_VIDEOENABLE_REG(0x80000000L);
	HWVPIPE_ulEnable = 0x80000000L;

	DBGTRACE_ENDFUNCTION("HWVPIPE_Enable");
}

/* ------------------------------------------------------------
// FUNCTION		: HWVPIPE_Disable
// ------------------------------------------------------------
// DESCRIPTION	: Disable the Video Pipeline
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWVPIPE_Disable(void)
{
	DBGTRACE_BEGINFUNCTION("HWVPIPE_Disable");

	REGHOST_SET_VPIPE_VIDEOENABLE_REG(0x0);
	HWVPIPE_ulEnable = 0;

	DBGTRACE_ENDFUNCTION("HWVPIPE_Disable");
}

/* ------------------------------------------------------------
// FUNCTION		: HWVPIPE_SetClippingArea
// ------------------------------------------------------------
// DESCRIPTION	: Set clipping rectangle for Video Pipeline
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// ulLeft		: Left side of the clipping area
// ulRight		: Right side of the clipping area
// ulTop		: Top side of the clipping area
// ulBottom		: Bottom side of the clipping area
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWVPIPE_SetClippingArea(unsigned long ulLeft, unsigned long ulRight, unsigned long ulTop, unsigned long ulBottom)
{
	DBGTRACE_BEGINFUNCTION("HWVPIPE_SetClippingArea");
	DBGTRACE_NUMERICPARAM(ulLeft);
	DBGTRACE_NUMERICPARAM(ulRight);
	DBGTRACE_NUMERICPARAM(ulTop);
	DBGTRACE_NUMERICPARAM(ulBottom);

	/* Set new clipping area */
	HWVPIPE_sClip.lLeft = ulLeft;
	HWVPIPE_sClip.lRight = ulRight;
	HWVPIPE_sClip.lTop = ulTop;
	HWVPIPE_sClip.lBottom = ulBottom;

	DBGTRACE_ENDFUNCTION("HWVPIPE_SetClippingArea");
}

/* ------------------------------------------------------------
// FUNCTION		: HWVPIPE_ExtendedSetDestination
// ------------------------------------------------------------
// DESCRIPTION	: Initialize the destination rectangle. This 
//				  function is similar to HWVPIPE_SetDestination
//				  but manage clipping
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// lDstX		: X coordinate of the location of the video
//				  pipeline on screen
// lDstY		: Y coordinate of the location of the video
//				  pipeline on screen
// ulDstWidth	: Width of the video pipeline on screen
// ulDstHeight	: Height of the video pipeline on screen
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWVPIPE_ExtendedSetDestination(long lDstX, long lDstY, unsigned long ulDstWidth, unsigned long ulDstHeight)
{
	DBGTRACE_BEGINFUNCTION("HWVPIPE_ExtendedSetDestination");
	DBGTRACE_NUMERICPARAM(lDstX);
	DBGTRACE_NUMERICPARAM(lDstY);
	DBGTRACE_NUMERICPARAM(ulDstWidth);
	DBGTRACE_NUMERICPARAM(ulDstHeight);

	/* Destination have been set */
	HWVPIPE_ulIsInit |= 0x2;

	/* Set the new destination rectangle */
	HWVPIPE_sCurrentDst.lLeft = lDstX;
	HWVPIPE_sCurrentDst.lRight = lDstX + ulDstWidth - 1;
	HWVPIPE_sCurrentDst.lTop = lDstY;
	HWVPIPE_sCurrentDst.lBottom = lDstY + ulDstHeight - 1;

	/* Display the Video Pipeline with the new destination */
	HWVPIPE_SetClippedVideoPipeline();

	DBGTRACE_ENDFUNCTION("HWVPIPE_ExtendedSetDestination");
}

/* ------------------------------------------------------------
// FUNCTION		: HWVPIPE_ExtendedSetSource
// ------------------------------------------------------------
// DESCRIPTION	: Initialize the source rectangle. This function
//				  is similar to HWVPIPE_SetSource but manage
//				  clipping
// ------------------------------------------------------------
// IN PARAMETERS
// ------------------------------------------------------------
// lSrcX		: X coordinate of the portion of the source
//				  image to display
// lSrcY		: Y coordinate of the portion of the source
//				  image to display
// ulSrcWidth	: Width of the portion of the source
//				  image to display
// ulSrcHeight	: Height of the portion of the source
//				  image to display
// ------------------------------------------------------------
// OUT PARAMETERS 
// ------------------------------------------------------------
// ------------------------------------------------------------
// RETURN		: -
// ------------------------------------------------------------
// MODIFIED		: -
// ------------------------------------------------------------*/
void HWVPIPE_ExtendedSetSource(unsigned long ulSrcX, unsigned long ulSrcY, unsigned long ulSrcWidth, unsigned long ulSrcHeight)
{
	DBGTRACE_BEGINFUNCTION("HWVPIPE_ExtendedSetSource");
	DBGTRACE_NUMERICPARAM(ulSrcX);
	DBGTRACE_NUMERICPARAM(ulSrcY);
	DBGTRACE_NUMERICPARAM(ulSrcWidth);
	DBGTRACE_NUMERICPARAM(ulSrcHeight);

	/* Source have been set */
	HWVPIPE_ulIsInit |= 0x1;

	/* Set the new source rectangle */
	HWVPIPE_sCurrentSrc.lLeft = ulSrcX;
	HWVPIPE_sCurrentSrc.lRight = ulSrcX + ulSrcWidth - 1;
	HWVPIPE_sCurrentSrc.lTop = ulSrcY;
	HWVPIPE_sCurrentSrc.lBottom = ulSrcY + ulSrcHeight - 1;

	/* Display the Video Pipeline with the new source */
	HWVPIPE_SetClippedVideoPipeline();

	DBGTRACE_ENDFUNCTION("HWVPIPE_ExtendedSetSource");
}

